@@ -32,13 +32,14 @@
 ## Paths and configuration                                                   ##
 ###############################################################################
 
-# The xine hack helps some systems.  See bug #7132.
+# The xine hack helps older systems.  See bug #7132.
 #
-# As of Miro 2.0, there is a command-line option to enable/disable the hack.
-# It's probably best to leave this as True allowing the hack to be compiled
-# in, and tell users to enable/disable at the command line with the xine-xvhack
-# setting.
-USE_XINE_HACK = True
+# If this hack is enabled and Miro crashes at the end of playback when
+# the xine renderer is used, then try disabling the hack.
+#
+# As of Miro 2.0, there is a command-line option to enable/disable the 
+# hack.
+USE_XINE_HACK = False
 
 
 # The following properties allow you to explicitly set xulrunner paths and
@@ -115,8 +116,10 @@ XPCOM_LIB = None
 GTKMOZEMBED_LIB = None
 XULRUNNER_19 = None
 
-# The name of the boost library.  Used for building extensions.
-BOOST_LIB = 'boost_python'
+# The name of the boost python library.  Used for building extensions.
+# If this is set to None, setup.py will divine the right lib to use.
+# It should probably be either "boost_python" or "boost_python-mt".
+BOOST_LIB = None
 
 
 ###############################################################################
@@ -291,6 +294,26 @@ def parse_pkg_config(command, components, options_dict = None):
 
     return options_dict
 
+all_libs = []
+if os.path.exists(os.path.join(sysconfig.PREFIX, "lib")):
+    all_libs.extend(os.listdir(os.path.join(sysconfig.PREFIX, "lib")))
+if os.path.exists(os.path.join(sysconfig.PREFIX, "lib64")):
+    all_libs.extend(os.listdir(os.path.join(sysconfig.PREFIX, "lib64")))
+all_libs = [mem for mem in all_libs if mem.startswith("libboost")]
+
+def mt_or_not(s, all_libs=all_libs):
+    for mem in all_libs:
+        if mem.startswith("lib%s-mt" % s):
+            print "Using %s-mt" % s
+            return "%s-mt" % s
+    print "using %s" % s
+    return s
+
+if BOOST_LIB is None:
+    BOOST_LIB = mt_or_not("boost_python")
+else:
+    print "Using %s" % BOOST_LIB
+
 def compile_xine_extractor():
     rv = os.system("gcc %s -o %s `pkg-config --libs --cflags gdk-pixbuf-2.0 glib-2.0 libxine`" %
                    (os.path.join(platform_dir, "xine/xine_extractor.c"), os.path.join(platform_dir, "xine/xine_extractor")))
@@ -304,7 +327,7 @@ def generate_miro(xpcom_path):
 
     f = open(os.path.join(platform_dir, "miro"), "w")
     if xpcom_path:
-        runtimelib = "LD_LIBRARY_PATH=%s " % xpcom_path
+        runtimelib = "MOZILLA_FIVE_HOME=%s LD_LIBRARY_PATH=%s " % (xpcom_path, xpcom_path)
 
     f.write( \
 """#!/bin/sh
@@ -384,31 +407,13 @@ def get_libtorrent_extension(portable_dir):
     if is_x64():
         extra_compile_args.append("-DAMD64")
 
-    # check for mt
     libraries = ['z', 'pthread', 'ssl']
-    all_libs = []
-    if os.path.exists(os.path.join(sysconfig.PREFIX, "lib")):
-        all_libs.extend(os.listdir(os.path.join(sysconfig.PREFIX, "lib")))
-    if os.path.exists(os.path.join(sysconfig.PREFIX, "lib64")):
-        all_libs.extend(os.listdir(os.path.join(sysconfig.PREFIX, "lib64")))
-    all_libs = [mem for mem in all_libs if mem.startswith("libboost")]
 
-    def mt_or_not(s, all_libs=all_libs, libraries=libraries):
-        for mem in all_libs:
-            if mem.startswith("lib%s-mt" % s):
-                print "using %s-mt" % s
-                libraries += ["%s-mt" % s]
-                break
-        else:
-            print "using %s" % s
-            libraries += [s]
-
-    mt_or_not("boost_python")
-    mt_or_not("boost_filesystem")
-    mt_or_not("boost_date_time")
-    mt_or_not("boost_thread")
-    # mt_or_not("boost_regex")
-    # mt_or_not("boost_serialization")
+    # get mt or non-mt versions of the boost libraries
+    libraries += [mt_or_not("boost_python"),
+                  mt_or_not("boost_filesystem"),
+                  mt_or_not("boost_date_time"),
+                  mt_or_not("boost_thread")]
 
     config_vars = sysconfig.get_config_vars()
     if "CFLAGS" in config_vars and "-Wstrict-prototypes" in config_vars["CFLAGS"]:
@@ -458,6 +463,10 @@ def get_mozilla_stuff():
             xpcom_lib = 'xulrunner-xpcom'
             gtkmozembed_lib = 'xulrunner-gtkmozembed'
 
+        elif re.search("^seamonkey-xpcom", packages, re.MULTILINE):
+            xpcom_lib = 'seamonkey-xpcom'
+            gtkmozembed_lib = 'seamonkey-gtkmozembed'
+
         elif re.search("^mozilla-xpcom", packages, re.MULTILINE):
             xpcom_lib = 'mozilla-xpcom'
             gtkmozembed_lib = 'mozilla-gtkmozembed'
@@ -467,7 +476,7 @@ def get_mozilla_stuff():
             gtkmozembed_lib = 'firefox-gtkmozembed'
 
         else:
-            sys.exit("Can't find libxul, xulrunner-xpcom, mozilla-xpcom or firefox-xpcom")
+            sys.exit("Can't find libxul, xulrunner-xpcom, seamonkey-xpcom, mozilla-xpcom or firefox-xpcom")
 
     print "using xpcom_lib: ", repr(xpcom_lib)
     print "using gtkmozembed_lib: ", repr(gtkmozembed_lib)
@@ -644,11 +653,16 @@ for dir in ('searchengines', 'images', 'testdata',
     dest_dir = os.path.join('/usr/share/miro/resources/', dir)
     data_files.append((dest_dir, listfiles(source_dir)))
 
+for mem in ["24", "48", "72", "128"]:
+    d = os.path.join("icons", "hicolor", "%sx%s" % (mem, mem), "apps")
+    source = os.path.join(platform_dir, d, "miro.png")
+    dest = os.path.join("/usr/share/", d)
+    data_files.append((dest, [source]))
 
-# add the desktop file, icons, mime data, and man page.
+# add ADOPTERS file, the desktop file, mime data, and man page
 data_files += [
-    ('/usr/share/pixmaps',
-     glob(os.path.join(platform_dir, 'miro-*.png'))),
+    ('/usr/share/miro/resources',
+     [os.path.join(root_dir, 'ADOPTERS')]),
     ('/usr/share/pixmaps',
      glob(os.path.join(platform_dir, 'miro.xpm'))),
     ('/usr/share/applications',
@@ -686,9 +700,9 @@ class install_data(distutils.command.install_data.install_data):
         dest = '/usr/share/miro/resources/app.config'
 
         config_file = util.read_simple_config_file(source)
-        print "Trying to figure out the svn revision...."
-        if config_file["appVersion"].endswith("svn"):
-            revision = util.query_revision(root_dir)
+        print "Trying to figure out the git revision...."
+        if config_file["appVersion"].endswith("git"):
+            revision = util.query_revision()
             if revision is None:
                 revision = "unknown"
                 revisionurl = "unknown"
@@ -696,7 +710,7 @@ class install_data(distutils.command.install_data.install_data):
             else:
                 revisionurl = revision[0]
                 revisionnum = revision[1]
-                revision = "%s - %s" % revision
+                revision = "%s - %s" % (revisionurl, revisionnum)
         else:
             revisionurl = ""
             revisionnum = ""
@@ -834,7 +848,6 @@ ext_modules.append(mozprompt_ext)
 ext_modules.append(httpobserver_ext)
 ext_modules.append(windowcreator_ext)
 ext_modules.append(pluginsdir_ext)
-ext_modules.append(Extension("miro.database", [os.path.join(portable_dir, 'database.pyx')]))
 ext_modules.append(Extension("miro.sorts", [os.path.join(portable_dir, 'sorts.pyx')]))
 
 #### Run setup ####
