@@ -43,12 +43,12 @@ from ExceptionHandling import NSExceptionHandler, NSLogAndHandleEveryExceptionMa
 
 from miro import app
 from miro import prefs
-from miro import views
 from miro import config
+from miro import downloader
 from miro import messages
 from miro import filetypes
 from miro import eventloop
-from miro import singleclick
+from miro import commandline
 from miro.frontends.widgets import menus
 from miro.frontends.widgets.application import Application
 from miro.plat import utils
@@ -57,7 +57,7 @@ from miro.plat import bundle
 from miro.plat import _growlImage
 from miro.plat import migrateappname
 from miro.plat.utils import ensureDownloadDaemonIsTerminated, filenameTypeToOSFilename, osFilenamesToFilenameTypes
-from miro.plat.frontends.widgets import video, osxmenus, sparkleupdater, threads
+from miro.plat.frontends.widgets import quicktime, osxmenus, sparkleupdater, threads
 from miro.plat.frontends.widgets.rect import Rect
 from miro.gtcache import gettext as _
 
@@ -77,6 +77,8 @@ class OSXApplication(Application):
 
     def connect_to_signals(self):
         Application.connect_to_signals(self)
+        eventloop.connect('thread-will-start', self.beginLoop)
+        eventloop.connect('thread-did-start', self.endLoop)
         eventloop.connect('begin-loop', self.beginLoop)
         eventloop.connect('end-loop', self.endLoop)
         config.add_change_callback(self.on_pref_changed)
@@ -85,7 +87,7 @@ class OSXApplication(Application):
         migrateappname.migrateVideos('Democracy', 'Miro')
         osxmenus.populate_menu()
         Application.startup_ui(self)
-        video.register_quicktime_components()
+        quicktime.register_components()
 
     # This callback should only be called once, after startup is done.
     # (see superclass implementation)
@@ -108,6 +110,7 @@ class OSXApplication(Application):
         if self.window is not None:
             windowFrame = self.window.nswindow.frame()
             windowFrame.size.height -= 22
+            config.set(prefs.LEFT_VIEW_SIZE, self.window.splitter.get_left_width())
             config.set(prefs.MAIN_WINDOW_FRAME, NSStringFromRect(windowFrame))
             config.save()
         Application.do_quit(self)
@@ -128,7 +131,8 @@ class OSXApplication(Application):
         # We could use Python's webbrowser.open() here, but
         # unfortunately, it doesn't have the same semantics under UNIX
         # as under other OSes. Sometimes it blocks, sometimes it doesn't.
-        NSWorkspace.sharedWorkspace().openURL_(NSURL.URLWithString_(url))
+        if url is not None:
+            NSWorkspace.sharedWorkspace().openURL_(NSURL.URLWithString_(url))
 
     def reveal_file(self, fn):
         filename = filenameTypeToOSFilename(fn)
@@ -166,7 +170,7 @@ class OSXApplication(Application):
     def handle_download_complete(self, obj, item):
         title = _('Download Completed')
         body = _('Download of video \'%s\' is finished.') % item.get_title()
-        icon = _growlImage.Image.imageFromPath(item.getThumbnail())
+        icon = _growlImage.Image.imageFromPath(item.get_thumbnail())
         self.app_controller.growl_notifier.notify(GROWL_DOWNLOAD_COMPLETE_NOTIFICATION, title, body, icon=icon)
 
     def handle_unwatched_count_changed(self):
@@ -258,23 +262,24 @@ class AppController(NSObject):
         self.growl_notifier = growl.GrowlNotifier(app_name, notifications)
         self.growl_notifier.register()
 
+    def applicationWillFinishLaunching_(self, notification):
+        NSExceptionHandler.defaultExceptionHandler().setExceptionHandlingMask_(NSLogAndHandleEveryExceptionMask)
+        NSExceptionHandler.defaultExceptionHandler().setDelegate_(self)
+
+        man = NSAppleEventManager.sharedAppleEventManager()
+        man.setEventHandler_andSelector_forEventClass_andEventID_(
+            self,
+            "openURL:withReplyEvent:",
+            struct.unpack(">i", "GURL")[0],
+            struct.unpack(">i", "GURL")[0])
+
+        ws = NSWorkspace.sharedWorkspace()
+        wsnc = ws.notificationCenter()
+        wsnc.addObserver_selector_name_object_(self, 'workspaceWillSleep:', NSWorkspaceWillSleepNotification, nil)
+        wsnc.addObserver_selector_name_object_(self, 'workspaceDidWake:',   NSWorkspaceDidWakeNotification,   nil)
+
     def applicationDidFinishLaunching_(self, notification):
         try:
-            NSExceptionHandler.defaultExceptionHandler().setExceptionHandlingMask_(NSLogAndHandleEveryExceptionMask)
-            NSExceptionHandler.defaultExceptionHandler().setDelegate_(self)
-
-            man = NSAppleEventManager.sharedAppleEventManager()
-            man.setEventHandler_andSelector_forEventClass_andEventID_(
-                self,
-                "openURL:withReplyEvent:",
-                struct.unpack(">i", "GURL")[0],
-                struct.unpack(">i", "GURL")[0])
-
-            ws = NSWorkspace.sharedWorkspace()
-            wsnc = ws.notificationCenter()
-            wsnc.addObserver_selector_name_object_(self, 'workspaceWillSleep:', NSWorkspaceWillSleepNotification, nil)
-            wsnc.addObserver_selector_name_object_(self, 'workspaceDidWake:',   NSWorkspaceDidWakeNotification,   nil)
-
             self.setup_growl_notifier()
             self.application.startup()
         except:
@@ -309,7 +314,7 @@ class AppController(NSObject):
         NSApplication.sharedApplication().setApplicationIconImage_(defaultAppIcon)
 
         ensureDownloadDaemonIsTerminated()    
-        app.controller.onShutdown()
+        app.controller.on_shutdown()
 
     def exceptionHandler_shouldLogException_mask_(self, handler, exception, mask):
         logging.warn("Unhandled exception: %s", exception.name())
@@ -356,17 +361,15 @@ class AppController(NSObject):
 
     def workspaceWillSleep_(self, notification):
         def pauseRunningDownloaders(self=self):
-            if views.initialized:
-                views.remoteDownloads.confirmDBThread()
-                self.pausedDownloaders = list()
-                for dl in views.remoteDownloads:
-                    if dl.get_state() == 'downloading':
-                        self.pausedDownloaders.append(dl)
-                dlCount = len(self.pausedDownloaders)
-                if dlCount > 0:
-                    logging.info("System is going to sleep, suspending %d download(s)." % dlCount)
-                    for dl in self.pausedDownloaders:
-                        dl.pause(block=True)
+            self.pausedDownloaders = list()
+            for dl in downloader.RemoteDownloader.make_view():
+                if dl.get_state() == 'downloading':
+                    self.pausedDownloaders.append(dl)
+            dlCount = len(self.pausedDownloaders)
+            if dlCount > 0:
+                logging.info("System is going to sleep, suspending %d download(s)." % dlCount)
+                for dl in self.pausedDownloaders:
+                    dl.pause()
         dc = eventloop.addUrgentCall(lambda:pauseRunningDownloaders(), "Suspending downloaders for sleep")
         # Until we can get proper delayed call completion notification, we're
         # just going to wait a few seconds here :)
@@ -388,29 +391,9 @@ class AppController(NSObject):
     @signature('v@:@@')
     def openURL_withReplyEvent_(self, event, replyEvent):
         keyDirectObject = struct.unpack(">i", "----")[0]
-        url = event.paramDescriptorForKeyword_(keyDirectObject).stringValue().decode('utf8')
+        url = event.paramDescriptorForKeyword_(keyDirectObject).stringValue().encode('utf8')
 
-        urlPattern = re.compile(r"^(.*?)://(.*)$")
-        match = urlPattern.match(url)
-        if match and match.group(1) == 'feed':
-            url = match.group(2)
-            match = urlPattern.match(url)
-            if not match:
-                url = u'http://%s' % url
-
-        if url.startswith('http'):
-            components = urlparse.urlparse(url)
-            path = components[2]
-            if filetypes.is_video_filename(path):
-                messages.DownloadURL(url).send_to_backend()
-            elif filetypes.is_audio_filename(path):
-                messages.DownloadURL(url).send_to_backend()
-            else:
-                messages.NewFeed(url).send_to_backend()
-        elif url.startswith('miro:'):
-            eventloop.addIdle(lambda:singleclick.add_subscription_url('miro:', 'application/x-miro', url), "Open Miro URL")
-        elif url.startswith('democracy:'):
-            eventloop.addIdle(lambda:singleclick.add_subscription_url('democracy:', 'application/x-democracy', url), "Open Democracy URL")
+        eventloop.addIdle(lambda: commandline.parse_command_line_args([url]), "Open URL")
 
     def validateUserInterfaceItem_(self, menuitem):
         action = menuitem.representedObject()
