@@ -72,7 +72,6 @@ class OverlayPalette (NSWindowController):
     shareButton         = IBOutlet('shareButton')
     keepButton          = IBOutlet('keepButton')
     deleteButton        = IBOutlet('deleteButton')
-    addToLibButton      = IBOutlet('addToLibButton')
     fsButton            = IBOutlet('fsButton')
     popInOutButton      = IBOutlet('popInOutButton')
     popInOutLabel       = IBOutlet('popInOutLabel')
@@ -87,7 +86,7 @@ class OverlayPalette (NSWindowController):
     timeIndicator       = IBOutlet('timeIndicator')
     volumeSlider        = IBOutlet('volumeSlider')
     
-    HOLD_TIME = 4
+    HOLD_TIME = 2
 
     @classmethod
     def get_instance(cls):
@@ -127,10 +126,6 @@ class OverlayPalette (NSWindowController):
         self.deleteButton.setAlternateImage_(getOverlayButtonAlternateImage(self.deleteButton.bounds().size))
         self.deleteButton.setTitle_(_("Delete"))
 
-        self.addToLibButton.setImage_(getOverlayButtonImage(self.addToLibButton.bounds().size))
-        self.addToLibButton.setAlternateImage_(getOverlayButtonAlternateImage(self.addToLibButton.bounds().size))
-        self.addToLibButton.setTitle_(_("Add to Library"))
-
         self.seekForwardButton.setCell_(SkipSeekButtonCell.cellFromButtonCell_direction_delay_(self.seekForwardButton.cell(), 1, 0.0))
         self.seekForwardButton.cell().setAllowsSkipping(False)
         self.seekBackwardButton.setCell_(SkipSeekButtonCell.cellFromButtonCell_direction_delay_(self.seekBackwardButton.cell(), -1, 0.0))
@@ -143,6 +138,7 @@ class OverlayPalette (NSWindowController):
         self.progressSlider.setShowCursor_(True)
 
         self.volumeSlider.cursor = NSImage.imageNamed_(u'fs-volume-slider')
+        self.volumeSlider.sliderWasClicked = self.volumeSliderWasClicked
         self.volumeSlider.sliderWasDragged = self.volumeSliderWasDragged
         self.volumeSlider.setShowCursor_(True)
 
@@ -157,9 +153,9 @@ class OverlayPalette (NSWindowController):
             self.feedLabel.setStringValue_("")
         self.keepButton.setEnabled_(item_info.can_be_saved)
         self.shareButton.setEnabled_(item_info.has_sharable_url)
-        self.addToLibButton.setHidden_(not item_info.is_external)
         self.adjustContent(video_window, False)
         self.update_(nil)
+        self.suspendAutoHiding()
         self.reveal(video_window)
 
     def on_items_changed(self, changed):
@@ -167,7 +163,6 @@ class OverlayPalette (NSWindowController):
             if item_info.id == self.item_info.id:
                 self.keepButton.setEnabled_(item_info.can_be_saved)
                 self.shareButton.setEnabled_(item_info.has_sharable_url)
-                self.addToLibButton.setHidden_(not item_info.is_external)
                 self.update_(nil)
                 
     def enter_fullscreen(self, videoWindow):
@@ -212,6 +207,19 @@ class OverlayPalette (NSWindowController):
             self.fsButton.setImage_(NSImage.imageNamed_('fs-button-enterfullscreen'))
             self.fsButton.setAlternateImage_(NSImage.imageNamed_('fs-button-enterfullscreen-alt'))
 
+        newFrame = self.window().frame() 
+        if videoWindow.is_fullscreen or app.playback_manager.detached_window is not None: 
+            self.titleLabel.setHidden_(NO)
+            self.feedLabel.setHidden_(NO)
+            newFrame.size.height = 198 
+        else: 
+            self.titleLabel.setHidden_(YES)
+            self.feedLabel.setHidden_(YES)
+            newFrame.size.height = 144
+        newFrame.origin.x = self.getHorizontalPosition(videoWindow, newFrame.size.width)
+        self.window().setFrame_display_animate_(newFrame, YES, animate)
+        self.playbackControls.setNeedsDisplay_(YES)
+
     def fit_in_video_window(self, video_window):
         return self.window().frame().size.width <= video_window.frame().size.width
 
@@ -238,6 +246,8 @@ class OverlayPalette (NSWindowController):
             self.revealing = True
             params = {NSViewAnimationTargetKey: self.window(), NSViewAnimationEffectKey: NSViewAnimationFadeInEffect}
             self.animate(params, 0.3)
+        else:
+            self.resumeAutoHiding()
     
     def hide(self):
         threads.warn_if_not_on_main_thread('OverlayPalette.hide')
@@ -262,13 +272,15 @@ class OverlayPalette (NSWindowController):
         self.holdStartTime = time.time()
     
     def suspendAutoHiding(self):
-        self.autoHidingTimer.invalidate()
-        self.autoHidingTimer = nil
+        if self.autoHidingTimer is not nil:
+            self.autoHidingTimer.invalidate()
+            self.autoHidingTimer = nil
     
     def resumeAutoHiding(self):
-        self.holdStartTime = time.time()
-        self.autoHidingTimer = NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(
-            1.0, self, 'hideAfterDelay:', nil, YES)
+        self.resetAutoHiding()
+        if self.autoHidingTimer is None:
+            self.autoHidingTimer = NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(
+                1.0, self, 'hideAfterDelay:', nil, YES)
     
     def animate(self, params, duration):
         self.anim = NSViewAnimation.alloc().initWithDuration_animationCurve_(duration, 0)
@@ -310,9 +322,6 @@ class OverlayPalette (NSWindowController):
         item_info = self.item_info
         app.widgetapp.share_item(item_info)
     
-    def addToLibrary_(self, sender):
-        messages.AddItemToLibrary(self.item_info.id).send_to_backend()
-    
     def handleShareItem_(self, sender):
         pass
 
@@ -379,9 +388,14 @@ class OverlayPalette (NSWindowController):
             self.renderer.play()
 
     def volumeSliderWasDragged(self, slider):
-        app.playback_manager.set_volume(slider.floatValue())
+        volume = slider.floatValue()
+        app.playback_manager.set_volume(volume)
+        app.widgetapp.window.videobox.volume_slider.set_value(volume)
         self.resetAutoHiding()
 
+    def volumeSliderWasClicked(self, slider):
+        self.volumeSliderWasDragged(slider)
+
     def video_will_play(self, obj, duration):
         self.playPauseButton.setImage_(NSImage.imageNamed_(u'fs-button-pause'))
         self.playPauseButton.setAlternateImage_(NSImage.imageNamed_(u'fs-button-pause-alt'))
@@ -392,9 +406,7 @@ class OverlayPalette (NSWindowController):
 
     def remove(self):
         threads.warn_if_not_on_main_thread('OverlayPalette.remove')
-        if self.autoHidingTimer is not nil:
-            self.autoHidingTimer.invalidate()
-            self.autoHidingTimer = nil
+        self.suspendAutoHiding()
         if self.updateTimer is not nil:
             self.updateTimer.invalidate()
             self.updateTimer = nil
@@ -443,11 +455,12 @@ class OverlayPaletteControlsView (NSView):
         path.setLineWidth_(2)
         path.stroke()
 
-        path = NSBezierPath.bezierPath()
-        path.moveToPoint_(NSPoint(0.5, bounds.size.height-1.5))
-        path.relativeLineToPoint_(NSPoint(bounds.size.width, 0))
-        path.setLineWidth_(2)
-        path.stroke()
+        if app.playback_manager.is_fullscreen or app.playback_manager.detached_window is not None:
+            path = NSBezierPath.bezierPath()
+            path.moveToPoint_(NSPoint(0.5, bounds.size.height-1.5))
+            path.relativeLineToPoint_(NSPoint(bounds.size.width, 0))
+            path.setLineWidth_(2)
+            path.stroke()
         
     def hitTest_(self, point):
         # Our buttons have transparent parts, but we still want mouse clicks
