@@ -330,6 +330,8 @@ def calc_row_height(view, model_row):
         cell.setObjectValue_(value_dict)
         cell_height = cell.calcHeight_(view)
         row_height = max(row_height, cell_height)
+    if row_height == 0:
+        row_height = 12
     return row_height
 
 class TableViewDelegate(NSObject):
@@ -434,6 +436,11 @@ class TableViewCommon(object):
             focused = self.isDescendantOf_(self.window().firstResponder())
             for row in get_all_indexes(self, self.selectedRowIndexes()):
                 self.drawBackgroundGradient(context, focused, row)
+    
+    def setFrameSize_(self, size):
+        if size.height == 0:
+            size.height = 4
+        self.SuperClass.setFrameSize_(self, size)
 
     def drawBackgroundGradient(self, context, focused, row):
         if focused:
