@@ -249,8 +249,8 @@ class MiroBuild (py2app):
         self.setup_options()
 
     def setup_config(self):
-        # Get subversion revision information.
-        revision = util.query_revision(ROOT_DIR)
+        # Get revision information.
+        revision = util.query_revision()
         if revision is None:
             revisionURL = 'unknown'
             revisionNum = '0000'
@@ -290,7 +290,6 @@ class MiroBuild (py2app):
         self.distribution.ext_modules.append(self.get_growl_ext())
         self.distribution.ext_modules.append(self.get_growl_image_ext())
         self.distribution.ext_modules.append(self.get_shading_ext())
-        self.distribution.ext_modules.append(self.get_database_ext())
         self.distribution.ext_modules.append(self.get_sorts_ext())
         self.distribution.ext_modules.append(self.get_fasttypes_ext())
         self.distribution.ext_modules.append(self.get_libtorrent_ext())
@@ -359,10 +358,6 @@ class MiroBuild (py2app):
         shading_link_args = ['-framework', 'ApplicationServices']
         return Extension("miro.plat.shading", sources=shading_src, extra_link_args=shading_link_args)
     
-    def get_database_ext(self):
-        database_src = glob(os.path.join(ROOT_DIR, 'portable', 'database.pyx'))
-        return Extension("miro.database", sources=database_src)
-    
     def get_sorts_ext(self):
         sorts_src = glob(os.path.join(ROOT_DIR, 'portable', 'sorts.pyx'))
         return Extension("miro.sorts", sources=sorts_src)
