@@ -28,6 +28,7 @@
 
 import os.path
 import os
+import shutil
 import time
 import socket
 import copy
@@ -67,6 +68,8 @@ BOOST_INCLUDE_PATH = os.path.join(BOOST_ROOT, 'include')
 BOOST_LIBRARIES = [os.path.splitext(os.path.basename(f))[0] for f in
         glob(os.path.join(BOOST_LIB_PATH, '*.lib'))]
 
+print BOOST_LIBRARIES
+
 ZLIB_INCLUDE_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'include')
 ZLIB_LIB_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib', 'lib')
 ZLIB_RUNTIME_LIBRARY_PATH = os.path.join(BINARY_KIT_ROOT, 'zlib')
@@ -155,9 +158,6 @@ from miro import util
 
 #### Extensions ####
 
-database_ext = Extension("miro.database", 
-        sources=[os.path.join(root_dir, 'portable', 'database.pyx')])
-
 sorts_ext = Extension("miro.sorts", 
         sources=[os.path.join(root_dir, 'portable', 'sorts.pyx')])
 
@@ -244,7 +244,8 @@ libtorrent_ext = Extension(
         libraries=OPENSSL_LIBRARIES + BOOST_LIBRARIES + [
             'wsock32', 'gdi32', 'ws2_32', 'zdll'
         ],
-        extra_compile_args=['-DBOOST_WINDOWS',
+        extra_compile_args=[
+            '-DBOOST_WINDOWS',
             '-DWIN32',
             '-DWIN32_LEAN_AND_MEAN',
             '-D_WIN32_WINNT=0x0500',
@@ -263,7 +264,6 @@ libtorrent_ext = Extension(
 # Private extension modules to build.
 ext_modules = [
     fasttypes_ext,
-    database_ext,
     sorts_ext,
     libtorrent_ext,
     pygtkhacks_ext,
@@ -300,9 +300,30 @@ for dir in ('searchengines', 'images'):
     source_dir = os.path.join(resources_dir, dir)
     data_files.extend(find_data_files(dest_dir, source_dir))
 
+data_files.append(('resources', [os.path.join(root_dir, 'ADOPTERS')]))
+
+locale_temp_dir = os.path.join(os.path.dirname(__file__), "build", "locale")
+
+# handle locale files
+for source in glob(os.path.join(resources_dir, "locale", "*.mo")):
+    lang = os.path.basename(source)[:-3]
+    dest = os.path.join(locale_temp_dir, lang, "LC_MESSAGES", "miro.mo")
+    try:
+        os.makedirs(os.path.dirname(dest))
+    except WindowsError:
+        pass
+    shutil.copyfile(source, dest)
+
+data_files.extend(find_data_files(os.path.join("resources", "locale"),
+                                  locale_temp_dir))
+
 app_config = os.path.join(resources_dir, 'app.config.template')
 template_vars = util.read_simple_config_file(app_config)
 
+# pixmap for the about dialog
+icon_path = os.path.join("icons", "hicolor", "128x128", "apps")
+data_files.append((os.path.join("resources", icon_path), [os.path.join(platform_dir, icon_path, "miro.png")]))
+
 ###############################################################################
 
 #### Our specialized install_data command ####
@@ -314,7 +335,7 @@ class install_data(distutils.command.install_data.install_data):
     def install_app_config(self):
         template = os.path.join(resources_dir, 'app.config.template')
         dest = os.path.join(self.install_dir, 'resources', 'app.config')
-        revision = util.query_revision(root_dir)
+        revision = util.query_revision()
         if revision is None:
             revision = "unknown"
             revisionurl = "unknown"
@@ -411,13 +432,25 @@ class runmiro(Command):
 class bdist_nsis(Command):
     description = "create Miro installer using NSIS"
 
-    user_options = []
+    user_options = [('generic', None, 'Build a generic installer instead of the Miro-branded installer.'),
+                    ('install-icon=', None, 'ICO file to use for the installer.'),
+                    ('install-image=', None, 'BMP file to use for the welcome/finish pages.')]
 
     def initialize_options(self):
-        pass
+        self.generic = False
+        self.install_icon = None
+        self.install_image = None
 
     def finalize_options(self):
-        pass
+        if self.generic and (self.install_icon or self.install_icon):
+            raise AssertionError('cannot specify install images with generic installer')
+        if self.generic:
+            self.install_icon = 'miro-installer-generic.ico'
+            self.install_image = 'miro-install-generic.bmp'
+        if self.install_icon is None:
+            self.install_icon = 'miro-installer.ico'
+        if self.install_image is None:
+            self.install_image = 'miro-install-image.bmp'
 
     def run(self):
         self.run_command('bdist_miro')
@@ -426,12 +459,11 @@ class bdist_nsis(Command):
         log.info("building installer")
 
         self.copy_file(os.path.join(platform_dir, 'Miro.nsi'), self.dist_dir)
-        self.copy_file("miro-installer.ico", self.dist_dir)
-        self.copy_file("miro-install-image.bmp", self.dist_dir)
+        self.copy_file(self.install_icon, self.dist_dir)
+        self.copy_file(self.install_image, self.dist_dir)
         self.copy_file("MiroBar-installer-page.ini", self.dist_dir)
         self.copy_file("askBarSetup-4.1.0.2.exe", self.dist_dir)
         self.copy_file("ask_toolbar.bmp", self.dist_dir)
-        self.copy_file("AskInstallChecker.exe", self.dist_dir)
 
         nsisVars = {}
         for (ourName, nsisName) in [
@@ -449,9 +481,17 @@ class bdist_nsis(Command):
                 template_vars['shortAppName']
         nsisVars['CONFIG_ICON'] = "%s.ico" % template_vars['shortAppName']
         nsisVars['CONFIG_PROG_ID'] = template_vars['longAppName'].replace(" ", ".") + ".1"
+        nsisVars['MIRO_INSTALL_ICON'] = self.install_icon
+        nsisVars['MIRO_INSTALL_IMAGE'] = self.install_image
+        if self.generic:
+            nsisVars['GENERIC_INSTALLER'] = '1'
 
         # One stage installer
-        outputFile = "%s-%s.exe" % \
+        if self.generic:
+            outputFile = "%s-%s-generic.exe"
+        else:
+            outputFile = "%s-%s.exe"
+        outputFile = outputFile % \
                 (template_vars['shortAppName'], template_vars['appVersion'])
         nsisVars['CONFIG_OUTPUT_FILE'] = outputFile
         nsisVars['CONFIG_TWOSTAGE'] = "No"
@@ -466,7 +506,11 @@ class bdist_nsis(Command):
             return
 
         # Two stage installer
-        outputFile = "%s-%s-twostage.exe" % \
+        if self.generic:
+            outputFile = '%s-%s-generic-twostage.exe'
+        else:
+            outputFile = "%s-%s-twostage.exe"
+        outputFile = outputFile % \
             (template_vars['shortAppName'], template_vars['appVersion'])
         nsisVars['CONFIG_OUTPUT_FILE'] = outputFile
         nsisVars['CONFIG_TWOSTAGE'] = "Yes"
