@@ -1,8 +1,38 @@
+# Miro - an RSS based video player application
+# Copyright (C) 2005-2009 Participatory Culture Foundation
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+#
+# In addition, as a special exception, the copyright holders give
+# permission to link the code of portions of this program with the OpenSSL
+# library.
+#
+# You must obey the GNU General Public License in all respects for all of
+# the code used other than OpenSSL. If you modify file(s) with this
+# exception, you may extend this exception to your version of the file(s),
+# but you are not obligated to do so. If you do not wish to do so, delete
+# this exception statement from your version. If you delete this exception
+# statement from all source files in the program, then also delete it here.
+
 import os
-from miro.gtcache import gettext as _
 import re
 from threading import RLock
 from copy import copy
+import sys
+
+from miro.gtcache import gettext as _
 
 try:
     import miro.libtorrent as lt
@@ -22,7 +52,7 @@ from miro import prefs
 
 from miro.dl_daemon import command, daemon
 from miro.util import checkF, checkU, stringify
-from miro.plat.utils import get_available_bytes_for_movies
+from miro.plat.utils import get_available_bytes_for_movies, utf8_to_filename
 
 chatter = True
 
@@ -205,19 +235,29 @@ class TorrentSession:
     def setUploadLimit(self):
         limit = -1
         if config.get(prefs.LIMIT_UPSTREAM):
-            limit = config.get(prefs.UPSTREAM_LIMIT_IN_KBS) * (2 ** 10)
+            limit = config.get(prefs.UPSTREAM_LIMIT_IN_KBS)
+            limit = limit * (2 ** 10)
+            if limit > sys.maxint:
+                limit = sys.maxint # avoid OverflowErrors by keeping the value
+                                   # an integer
         self.session.set_upload_rate_limit(limit)
 
     def setDownloadLimit(self):
         limit = -1
         if config.get(prefs.LIMIT_DOWNSTREAM_BT):
-            limit = config.get(prefs.DOWNSTREAM_BT_LIMIT_IN_KBS) * (2 ** 10)
+            limit = config.get(prefs.DOWNSTREAM_BT_LIMIT_IN_KBS)
+            limit = limit * (2 ** 10)
+            if limit > sys.maxint:
+                limit = sys.maxint # avoid OverflowErrors by keeping the value
+                                   # an integer
         self.session.set_download_rate_limit(limit)
 
     def setConnectionLimit(self):
         limit = -1
         if config.get(prefs.LIMIT_CONNECTIONS_BT):
             limit = config.get(prefs.CONNECTION_LIMIT_BT_NUM)
+            if limit > 65536:
+                limit = 65536 # there are only 2**16 TCP port numbers
         self.session.set_max_connections(limit)
 
     def setEncryption(self):
@@ -273,8 +313,7 @@ class TorrentSession:
     def updateTorrents(self):
         # Copy this set into a list in case any of the torrents gets removed during the iteration.
         for torrent in [x for x in self.torrents]:
-            torrent.updateStatus()
-        
+            torrent.update_status()
 
 torrentSession = TorrentSession()
 
@@ -296,7 +335,7 @@ class DownloadStatusUpdater:
     happen often enough to cause CPU problems.
     """
 
-    UPDATE_CLIENT_INTERVAL = 1
+    UPDATE_CLIENT_INTERVAL = 2
 
     def __init__(self):
         self.toUpdate = set()
@@ -351,7 +390,7 @@ class BGDownloader:
         self.retryTime = None
         self.retryCount = -1
 
-    def getURL(self):
+    def get_url(self):
         return self.url
 
     def getStatus(self):
@@ -378,25 +417,29 @@ class BGDownloader:
         x = command.UpdateDownloadStatus(daemon.lastDaemon, self.getStatus())
         return x.send()
         
-    ##
-    def pickInitialFilename(self, suffix=".part"):
+    def pickInitialFilename(self, suffix=".part", torrent=False):
         """Pick a path to download to based on self.shortFilename.
 
         This method sets self.filename, as well as creates any leading paths
         needed to start downloading there.
+
+        If the torrent flag is true, then the filename we're working with
+        is utf-8 and shouldn't be transformed in any way.
+
+        If the torrent flag is false, then the filename we're working with
+        is ascii and needs to be transformed into something sane.  (default)
         """
 
         downloadDir = os.path.join(config.get(prefs.MOVIES_DIRECTORY),
                 'Incomplete Downloads')
         # Create the download directory if it doesn't already exist.
-        try:
+        if not os.path.exists(downloadDir):
             fileutil.makedirs(downloadDir)
-        except (SystemExit, KeyboardInterrupt):
-            raise
-        except:
-            pass
-        cleaned = cleanFilename(self.shortFilename + suffix)
-        self.filename = nextFreeFilename(os.path.join(downloadDir, cleaned))
+        filename = self.shortFilename + suffix
+        if not torrent:
+            # this is an ascii filename and needs to be fixed
+            filename = cleanFilename(filename)
+        self.filename = nextFreeFilename(os.path.join(downloadDir, filename))
 
     def moveToMoviesDirectory(self):
         """Move our downloaded file from the Incomplete Downloads directory to
@@ -821,7 +864,7 @@ class BTDownloader(BGDownloader):
         else:
             self.firstTime = True
             BGDownloader.__init__(self,url,item)
-            self.runDownloader()
+            self.run_downloader()
 
     def _startTorrent(self):
         try:
@@ -847,7 +890,7 @@ class BTDownloader(BGDownloader):
                 self.torrent = torrentSession.session.add_torrent(torrent_info, save_path, None, lt.storage_mode_t.storage_mode_allocate)
             try:
                 if (lt.version_major, lt.version_minor) > (0, 13):
-                    logging.info("libtorrent version is (%d, %d), setting auto_managed to False", lt.version_major, lt.version_minor)
+                    logging.debug("libtorrent version is (%d, %d), setting auto_managed to False", lt.version_major, lt.version_minor)
                     self.torrent.auto_managed(False)
             except AttributeError:
                 logging.warning("libtorrent module doesn't have version_major or version_minor")
@@ -894,7 +937,7 @@ class BTDownloader(BGDownloader):
         else:
             self._startTorrent()
 
-    def updateStatus(self):
+    def update_status(self):
         """
         activity -- string specifying what's currently happening or None for
                 normal operations.  
@@ -962,7 +1005,7 @@ class BTDownloader(BGDownloader):
         self.upRate = 0
         self.uploadedStart = self.uploaded
         if self.state in ('downloading', 'uploading'):
-            self.runDownloader(done=True)
+            self.run_downloader(done=True)
         elif self.state == 'offline':
             self.start()
 
@@ -1046,9 +1089,8 @@ class BTDownloader(BGDownloader):
                 self.handleError(_("Corrupt Torrent"),
                                  _("The torrent file at %(url)s was not valid") % {"url": stringify(self.url)})
                 return
-            name = name.decode('utf-8', 'replace')
-            self.shortFilename = cleanFilename(name)
-            self.pickInitialFilename("")
+            self.shortFilename = utf8_to_filename(name)
+            self.pickInitialFilename(suffix="", torrent=True)
         self.updateClient()
         self._resumeTorrent()
 
@@ -1071,7 +1113,9 @@ class BTDownloader(BGDownloader):
                 except IOError:
                     self.handleError(_("Torrent file deleted"),
                                      _("The torrent file for this item was deleted "
-                                       "outside of Miro."))
+                                       "outside of %(appname)s.",
+                                       {"appname": config.get(prefs.SHORT_APP_NAME)}
+                                       ))
 
                     return
                 try:
@@ -1081,12 +1125,12 @@ class BTDownloader(BGDownloader):
 
                 self.handleMetainfo(metainfo)
             else:
-                httpclient.grabURL(self.getURL(), self.onDescriptionDownload,
+                httpclient.grabURL(self.get_url(), self.onDescriptionDownload,
                         self.onDescriptionDownloadFailed)
         else:
             self.gotMetainfo()
                 
-    def runDownloader(self,done=False):
+    def run_downloader(self, done=False):
         self.restarting = done
         self.updateClient()
         self.getMetainfo()
