@@ -30,17 +30,17 @@ from base64 import b64encode
 from miro.gtcache import gettext as _
 import os
 
-from miro.database import DDBObject, defaultDatabase, ObjectNotFoundError
+from miro.database import DDBObject, ObjectNotFoundError
 from miro.dl_daemon import daemon, command
 from miro.download_utils import nextFreeFilename, getFileURLPath, filterDirectoryName
 from miro.util import get_torrent_info_hash, returnsUnicode, checkU, returnsFilename, unicodify, checkF, toUni
 # from miro import app
 from miro import config
+from miro import dialogs
 from miro import httpclient
-from miro import indexes
+from miro import models
 from miro import prefs
 import random
-from miro import views
 from miro.plat.utils import samefile, FilenameType, unicodeToFilename
 from miro import flashscraper
 import logging
@@ -53,7 +53,7 @@ _downloads = {}
 
 # Returns an HTTP auth object corresponding to the given host, path or
 # None if it doesn't exist
-def findHTTPAuth(host, path, realm=None, scheme=None):
+def find_http_auth(host, path, realm=None, scheme=None):
     checkU(host)
     checkU(path)
     if realm:
@@ -61,8 +61,7 @@ def findHTTPAuth(host, path, realm=None, scheme=None):
     if scheme:
         checkU(scheme)
     #print "Trying to find HTTPAuth with host %s, path %s, realm %s, and scheme %s" %(host,path,realm,scheme)
-    defaultDatabase.confirmDBThread()
-    for obj in views.httpauths:
+    for obj in HTTPAuthPassword.make_view():
         if (obj.host == host and path.startswith(obj.path) and
                 (realm is None or obj.realm == realm) and
                 (scheme is None or obj.authScheme == scheme)):
@@ -71,88 +70,113 @@ def findHTTPAuth(host, path, realm=None, scheme=None):
 
 
 class HTTPAuthPassword(DDBObject):
-    def __init__(self, username, password, host, realm, path, authScheme=u"Basic"):
+    def setup_new(self, username, password, host, realm, path, authScheme=u"Basic"):
         checkU(username)
         checkU(password)
         checkU(host)
         checkU(realm)
         checkU(path)
         checkU(authScheme)
-        oldAuth = findHTTPAuth(host, path, realm, authScheme)
+        oldAuth = find_http_auth(host, path, realm, authScheme)
         while not oldAuth is None:
             oldAuth.remove()
-            oldAuth = findHTTPAuth(host, path, realm, authScheme)
+            oldAuth = find_http_auth(host, path, realm, authScheme)
         self.username = username
         self.password = password
         self.host = host
         self.realm = realm
         self.path = os.path.dirname(path)
         self.authScheme = authScheme
-        DDBObject.__init__(self)
 
-    def getAuthToken(self):
+    def get_auth_token(self):
         authString = u':'
-        self.confirmDBThread()
+        self.confirm_db_thread()
         authString = self.username+u':'+self.password
         return b64encode(authString)
 
-    def getAuthScheme(self):
-        self.confirmDBThread()
+    def get_auth_scheme(self):
+        self.confirm_db_thread()
         return self.authScheme
 
 totalUpRate = 0
 totalDownRate = 0
 
-def _getDownloader(dlid):
-    return views.remoteDownloads.getItemWithIndex(indexes.downloadsByDLID, dlid)
+def get_downloader_by_dlid(dlid):
+    try:
+        return RemoteDownloader.get_by_dlid(dlid)
+    except ObjectNotFoundError:
+        return None
 
 @returnsUnicode
-def generateDownloadID():
+def generate_dlid():
     dlid = u"download%08d" % random.randint(0, 99999999)
-    while _getDownloader (dlid=dlid):
+    while get_downloader_by_dlid(dlid=dlid):
         dlid = u"download%08d" % random.randint(0, 99999999)
     return dlid
 
 class RemoteDownloader(DDBObject):
     """Download a file using the downloader daemon."""
-    def __init__(self, url, item, contentType=None, channelName=None):
+    def setup_new(self, url, item, contentType=None, channelName=None):
         checkU(url)
         if contentType:
             checkU(contentType)
         self.origURL = self.url = url
         self.itemList = []
-        self.dlid = generateDownloadID()
+        self.child_deleted = False
+        self.main_item_id = None
+        self.dlid = generate_dlid()
         self.status = {}
+        self.state = u'downloading'
         if contentType is None:
             # HACK:  Some servers report the wrong content-type for torrent
             # files.  We try to work around that by assuming if the enclosure
             # states that something is a torrent, it's a torrent.
             # Thanks to j@v2v.cc
-            enclosureContentType = item.getFirstVideoEnclosureType()
-            if enclosureContentType == u'application/x-bittorrent':
-                contentType = enclosureContentType
+            if item.enclosure_type == u'application/x-bittorrent':
+                contentType = item.enclosure_type
         self.contentType = u""
         self.deleteFiles = True
         self.channelName = channelName
         self.manualUpload = False
-        DDBObject.__init__(self)
         if contentType is None:
             self.contentType = u""
         else:
             self.contentType = contentType
 
         if self.contentType == u'':
-            self.getContentType()
+            self.get_content_type()
         else:
-            self.runDownloader()
+            self.run_downloader()
 
-    def signalChange(self, needsSave=True, needsSignalItem=True):
+    @classmethod
+    def finished_view(cls):
+        return cls.make_view("state in ('finished', 'uploading', "
+                "'uploading-paused')")
+
+    @classmethod
+    def auto_uploader_view(cls):
+        return cls.make_view("state == 'uploading' AND NOT manualUpload")
+
+    @classmethod
+    def get_by_dlid(cls, dlid):
+        return cls.make_view('dlid=?', (dlid,)).get_singleton()
+
+    @classmethod
+    def get_by_url(cls, url):
+        return cls.make_view('origURL=?', (url,)).get_singleton()
+
+    @classmethod
+    def orphaned_view(cls):
+        """Downloaders with no items associated with them."""
+        return cls.make_view('id NOT IN (SELECT downloader_id from item)')
+
+    def signal_change(self, needsSave=True, needsSignalItem=True):
+        DDBObject.signal_change(self, needsSave=needsSave)
         if needsSignalItem:
             for item in self.itemList:
-                item.signalChange(needsSave=False)
-        DDBObject.signalChange(self, needsSave=needsSave)
+                item.signal_change(needsSave=False)
 
-    def onContentType(self, info):
+    def on_content_type(self, info):
         if not self.idExists():
             return
 
@@ -165,28 +189,29 @@ class RemoteDownloader(DDBObject):
                 raise
             except:
                 self.contentType = None
-            self.runDownloader()
+            self.run_downloader()
         else:
             error = httpclient.UnexpectedStatusCode(info['status'])
-            self.onContentTypeError(error)
+            self.on_content_type_error(error)
 
-    def onContentTypeError(self, error):
+    def on_content_type_error(self, error):
         if not self.idExists():
             return
 
         self.status['state'] = u"failed"
         self.status['shortReasonFailed'] = error.getFriendlyDescription()
         self.status['reasonFailed'] = error.getLongDescription()
-        self.signalChange()
+        self.signal_change()
 
-    def getContentType(self):
-        httpclient.grabHeaders(self.url, self.onContentType, self.onContentTypeError)
+    def get_content_type(self):
+        httpclient.grabHeaders(self.url,
+                               self.on_content_type, self.on_content_type_error)
  
     @classmethod
-    def initializeDaemon(cls):
+    def initialize_daemon(cls):
         RemoteDownloader.dldaemon = daemon.ControllerDaemon()
 
-    def _getRates(self):
+    def _get_rates(self):
         state = self.get_state()
         if state == u'downloading':
             return (self.status.get('rate', 0), self.status.get('upRate', 0))
@@ -194,26 +219,26 @@ class RemoteDownloader(DDBObject):
             return (0, self.status.get('upRate', 0))
         return (0, 0)
 
-    def beforeChangingStatus(self):
+    def before_changing_status(self):
         global totalDownRate
         global totalUpRate
-        rates = self._getRates()
+        rates = self._get_rates()
         totalDownRate -= rates[0]
         totalUpRate -= rates[1]
 
-    def afterChangingStatus(self):
+    def after_changing_status(self):
         global totalDownRate
         global totalUpRate
-        rates = self._getRates()
+        rates = self._get_rates()
         totalDownRate += rates[0]
         totalUpRate += rates[1]
 
     @classmethod
-    def updateStatus(cls, data):
+    def update_status(cls, data):
         for field in data:
             if field not in ['filename', 'shortFilename', 'channelName', 'metainfo', 'fastResumeData']:
                 data[field] = unicodify(data[field])
-        self = _getDownloader(dlid=data['dlid'])
+        self = get_downloader_by_dlid(dlid=data['dlid'])
         # print data
         if self is not None:
             try:
@@ -225,7 +250,8 @@ class RemoteDownloader(DDBObject):
                 print "WARNING exception when comparing status: %s" % e
 
             wasFinished = self.isFinished()
-            self.beforeChangingStatus()
+            old_filename = self.get_filename()
+            self.before_changing_status()
 
             # FIXME: how do we get all of the possible bit torrent
             # activity strings into gettext? --NN
@@ -233,25 +259,31 @@ class RemoteDownloader(DDBObject):
                 data['activity'] = _(data['activity'])
 
             self.status = data
+            self._recalc_state()
 
             # Store the time the download finished
             finished = self.isFinished() and not wasFinished
-            self.afterChangingStatus()
+            file_migrated = (self.isFinished() and
+                    self.get_filename() != old_filename)
+            needsSignalItem = not (finished or file_migrated)
+            self.after_changing_status()
 
             if self.get_state() == u'uploading' and not self.manualUpload and self.getUploadRatio() > 1.5:
                 self.stopUpload()
 
-            self.signalChange(needsSignalItem = not finished)
+            self.signal_change(needsSignalItem = needsSignalItem)
             if finished:
                 for item in self.itemList:
                     item.on_download_finished()
+            elif file_migrated:
+                self._file_migrated(old_filename)
 
-    def runDownloader(self):
+    def run_downloader(self):
         """This is the actual download thread.
         """
-        flashscraper.try_scraping_url(self.url, self._runDownloader)
+        flashscraper.try_scraping_url(self.url, self._run_downloader)
 
-    def _runDownloader(self, url, contentType = None):
+    def _run_downloader(self, url, contentType = None):
         if not self.idExists():
             return # we got deleted while we were doing the flash scraping
         if contentType is not None:
@@ -267,18 +299,18 @@ class RemoteDownloader(DDBObject):
             self.status["state"] = u'failed'
             self.status["shortReasonFailed"] = _('File not found')
             self.status["reasonFailed"] = _('Flash URL Scraping Error')
-        self.signalChange()
+        self.signal_change()
 
-    def pause(self, block=False):
+    def pause(self):
         """Pauses the download."""
         if _downloads.has_key(self.dlid):
             c = command.PauseDownloadCommand(RemoteDownloader.dldaemon, self.dlid)
             c.send()
         else:
-            self.beforeChangingStatus()
+            self.before_changing_status()
             self.status["state"] = u"paused"
-            self.afterChangingStatus()
-            self.signalChange()
+            self.after_changing_status()
+            self.signal_change()
 
     def stop(self, delete):
         """Stops the download and removes the partially downloaded
@@ -294,7 +326,7 @@ class RemoteDownloader(DDBObject):
             if delete:
                 self.delete()
             self.status["state"] = u"stopped"
-            self.signalChange()
+            self.signal_change()
 
     def delete(self):
         try:
@@ -303,9 +335,7 @@ class RemoteDownloader(DDBObject):
             return
         try:
             fileutil.delete(filename)
-        except (SystemExit, KeyboardInterrupt):
-            raise
-        except:
+        except OSError:
             logging.exception("Error deleting downloaded file: %s", toUni(filename))
 
         parent = os.path.join(fileutil.expand_filename(filename), os.path.pardir)
@@ -316,9 +346,7 @@ class RemoteDownloader(DDBObject):
                 len(os.listdir(parent)) == 0):
             try:
                 os.rmdir(parent)
-            except (SystemExit, KeyboardInterrupt):
-                raise
-            except:
+            except OSError:
                 logging.exception("Error deleting empty download directory: %s", toUni(parent))
 
     def start(self):
@@ -327,16 +355,15 @@ class RemoteDownloader(DDBObject):
         if self.get_state() == u'failed':
             if _downloads.has_key (self.dlid):
                 del _downloads[self.dlid]
-            self.dlid = generateDownloadID()
-            views.remoteDownloads.recomputeIndex(indexes.downloadsByDLID)
-            self.beforeChangingStatus()
+            self.dlid = generate_dlid()
+            self.before_changing_status()
             self.status = {}
-            self.afterChangingStatus()
+            self.after_changing_status()
             if self.contentType == u"":
-                self.getContentType()
+                self.get_content_type()
             else:
-                self.runDownloader()
-            self.signalChange()
+                self.run_downloader()
+            self.signal_change()
         elif self.get_state() in (u'stopped', u'paused', u'offline'):
             if _downloads.has_key(self.dlid):
                 c = command.StartDownloadCommand(RemoteDownloader.dldaemon,
@@ -345,7 +372,7 @@ class RemoteDownloader(DDBObject):
             else:
                 self.status['state'] = u'downloading'
                 self.restart()
-                self.signalChange()
+                self.signal_change()
 
     def migrate(self, directory):
         if _downloads.has_key(self.dlid):
@@ -374,9 +401,7 @@ URL was %s""" % self.url
                     directory = os.path.join (directory, channelName)
                 try:
                     fileutil.makedirs(directory)
-                except (SystemExit, KeyboardInterrupt):
-                    raise
-                except:
+                except OSError:
                     pass
                 newfilename = os.path.join(directory, shortFilename)
                 if newfilename == filename:
@@ -384,15 +409,23 @@ URL was %s""" % self.url
                 newfilename = nextFreeFilename(newfilename)
                 def callback():
                     self.status['filename'] = newfilename
-                    self.signalChange()
+                    self.signal_change(needsSignalItem=False)
+                    self._file_migrated(filename)
                 fileutil.migrate_file(filename, newfilename, callback)
         for i in self.itemList:
             i.migrate_children(directory)
 
+    def _file_migrated(self, old_filename):
+        # Make sure that itemList is populated with items, see (#12202)
+        for item in models.Item.downloader_view(self.id):
+            self.addItem(item)
+        for item in self.itemList:
+            item.on_downloader_migrated(old_filename, self.get_filename())
+
     def set_delete_files(self, deleteFiles):
         self.deleteFiles = deleteFiles
 
-    def setChannelName(self, channelName):
+    def set_channel_name(self, channelName):
         if self.channelName is None:
             if channelName:
                 checkF(channelName)
@@ -403,17 +436,17 @@ URL was %s""" % self.url
         """
         global totalDownRate
         global totalUpRate
-        rates = self._getRates()
+        rates = self._get_rates()
         totalDownRate -= rates[0]
         totalUpRate -= rates[1]
         self.stop(self.deleteFiles)
         DDBObject.remove(self)
 
-    def getType(self):
+    def get_type(self):
         """Get the type of download.  Will return either "http" or
         "bittorrent".
         """
-        self.confirmDBThread()
+        self.confirm_db_thread()
         if self.contentType == u'application/x-bittorrent':
             return u"bittorrent"
         else:
@@ -425,23 +458,29 @@ URL was %s""" % self.url
         """
         if item not in self.itemList:
             self.itemList.append(item)
+            if self.main_item_id is None:
+                self.main_item_id = item.id
+                self.signal_change()
 
     def removeItem(self, item):
         self.itemList.remove(item)
         if len (self.itemList) == 0:
             self.remove()
+        elif item.id == self.main_item_id:
+            self.main_item_id = self.itemList[0].id
+            self.signal_change()
 
     def getRate(self):
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status.get('rate', 0)
 
     def getETA(self):
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status.get('eta', 0)
 
     @returnsUnicode
     def get_startup_activity(self):
-        self.confirmDBThread()
+        self.confirm_db_thread()
         activity = self.status.get('activity')
         if activity is None:
             return _("starting up")
@@ -456,7 +495,7 @@ URL was %s""" % self.url
         if not self.get_state() == u'failed':
             msg = u"getReasonFailed() called on a non-failed downloader"
             raise ValueError(msg)
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status['reasonFailed']
 
     @returnsUnicode
@@ -464,14 +503,14 @@ URL was %s""" % self.url
         if not self.get_state() == u'failed':
             msg = u"getShortReasonFailed() called on a non-failed downloader"
             raise ValueError(msg)
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status['shortReasonFailed']
 
     @returnsUnicode
-    def getURL(self):
+    def get_url(self):
         """Returns the URL we're downloading
         """
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.url
 
     @returnsUnicode    
@@ -479,8 +518,8 @@ URL was %s""" % self.url
         """Returns the state of the download: downloading, paused, stopped,
         failed, or finished
         """
-        self.confirmDBThread()
-        return self.status.get('state', u'downloading')
+        self.confirm_db_thread()
+        return self.state
 
     def isFinished(self):
         return self.get_state() in (u'finished', u'uploading', u'uploading-paused')
@@ -488,13 +527,13 @@ URL was %s""" % self.url
     def getTotalSize(self):
         """Returns the total size of the download in bytes
         """
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status.get(u'totalSize', -1)
 
     def get_current_size(self):
         """Returns the current amount downloaded in bytes
         """
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status.get(u'currentSize', 0)
 
     @returnsFilename
@@ -502,20 +541,25 @@ URL was %s""" % self.url
         """Returns the filename that we're downloading to. Should not be
         called until state is "finished."
         """
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status.get('filename', FilenameType(''))
 
-    def onRestore(self):
-        DDBObject.onRestore(self)
+    def setup_restored(self):
         self.deleteFiles = True
         self.itemList = []
         if self.dlid == 'noid':
             # this won't happen nowadays, but it can for old databases
-            self.dlid = generateDownloadID()
+            self.dlid = generate_dlid()
         self.status['rate'] = 0
         self.status['upRate'] = 0
         self.status['eta'] = 0
 
+    def on_signal_change(self):
+        self._recalc_state()
+
+    def _recalc_state(self):
+        self.state = self.status.get('state', u'downloading')
+
     def getUploadRatio(self):
         size = self.get_current_size()
         if size == 0:
@@ -534,9 +578,9 @@ URL was %s""" % self.url
     def restart(self):
         if len(self.status) == 0 or self.status.get('dlerType') is None:
             if self.contentType == u"":
-                self.getContentType()
+                self.get_content_type()
             else:
-                self.runDownloader()
+                self.run_downloader()
         else:
             _downloads[self.dlid] = self
             c = command.RestoreDownloaderCommand(RemoteDownloader.dldaemon, 
@@ -544,8 +588,18 @@ URL was %s""" % self.url
             c.send()
 
     def startUpload(self):
-        if (self.get_state() not in (u'finished', u'uploading-paused')
-                or self.getType() != u'bittorrent'):
+        if self.get_type() != u'bittorrent':
+            logging.warn("called startUpload for non-bittorrent downloader")
+            return
+        if self.child_deleted:
+            title = "Can't Resume Seeding"
+            msg = ("Seeding cannot resume because part of this torrent "
+                    "has been deleted.")
+            dialogs.MessageBoxDialog(title, msg).run()
+            return
+        if self.get_state() not in (u'finished', u'uploading-paused'):
+            logging.warn("called startUpload when downloader state "
+                        "is: %s", self.get_state())
             return
         self.manualUpload = True
         if _downloads.has_key(self.dlid):
@@ -553,11 +607,11 @@ URL was %s""" % self.url
                                              self.dlid)
             c.send()
         else:
-            self.beforeChangingStatus()
+            self.before_changing_status()
             self.status['state'] = u'uploading'
-            self.afterChangingStatus()
+            self.after_changing_status()
             self.restart()
-            self.signalChange()
+            self.signal_change()
 
     def stopUpload(self):
         """
@@ -568,10 +622,10 @@ URL was %s""" % self.url
                                           self.dlid)
             c.send()
             del _downloads[self.dlid]
-        self.beforeChangingStatus()
+        self.before_changing_status()
         self.status["state"] = u"finished"
-        self.afterChangingStatus()
-        self.signalChange()
+        self.after_changing_status()
+        self.signal_change()
 
     def pauseUpload(self):
         """
@@ -582,21 +636,20 @@ URL was %s""" % self.url
                                            self.dlid)
             c.send()
             del _downloads[self.dlid]
-        self.beforeChangingStatus()
+        self.before_changing_status()
         self.status["state"] = u"uploading-paused"
-        self.afterChangingStatus()
-        self.signalChange()
+        self.after_changing_status()
+        self.signal_change()
 
 
-def cleanupIncompleteDownloads():
+def cleanup_incomplete_downloads():
     downloadDir = os.path.join(config.get(prefs.MOVIES_DIRECTORY),
                                           'Incomplete Downloads')
     if not fileutil.exists(downloadDir):
         return
 
     filesInUse = set()
-    views.remoteDownloads.confirmDBThread()
-    for downloader in views.remoteDownloads:
+    for downloader in RemoteDownloader.make_view():
         if downloader.get_state() in ('downloading', 'paused',
                                      'offline', 'uploading', 'finished',
                                      'uploading-paused'):
@@ -614,36 +667,36 @@ def cleanupIncompleteDownloads():
                     fileutil.remove (f)
                 elif fileutil.isdir(f):
                     fileutil.rmtree (f)
-            except (SystemExit, KeyboardInterrupt):
-                raise
-            except:
+            except OSError:
                 # FIXME - maybe a permissions error?
                 pass
 
-def killUploaders(*args):
+def kill_uploaders(*args):
     torrent_limit = config.get(prefs.UPSTREAM_TORRENT_LIMIT)
-    while (views.autoUploads.len() > torrent_limit):
-        views.autoUploads[0].stopUpload()
+    auto_uploads = list(RemoteDownloader.auto_uploader_view())
+    for dler in auto_uploads[torrent_limit:]:
+        dler.stopUpload()
 
-def configChangeUploaders(key, value):
+def config_change_uploaders(key, value):
     if key == prefs.UPSTREAM_TORRENT_LIMIT.key:
-        killUploaders()
+        kill_uploaders()
 
-def limitUploaders():
-    views.autoUploads.addAddCallback(killUploaders)
-    config.add_change_callback(configChangeUploaders)
-    killUploaders()
+def limit_uploaders():
+    tracker = RemoteDownloader.auto_uploader_view().make_tracker()
+    tracker.connect('added', kill_uploaders)
+    config.add_change_callback(config_change_uploaders)
+    kill_uploaders()
 
 class DownloadDaemonStarter(object):
     def __init__(self):
-        RemoteDownloader.initializeDaemon()
-        self.downloads_at_startup = list(views.remoteDownloads)
+        RemoteDownloader.initialize_daemon()
+        self.downloads_at_startup = list(RemoteDownloader.make_view())
         self.started = False
 
     def startup(self):
-        cleanupIncompleteDownloads()
+        cleanup_incomplete_downloads()
         RemoteDownloader.dldaemon.start_downloader_daemon()
-        limitUploaders()
+        limit_uploaders()
         self.restart_downloads()
         self.started = True
 
@@ -655,18 +708,18 @@ class DownloadDaemonStarter(object):
         if not self.started:
             callback()
         else:
-            RemoteDownloader.dldaemon.shutdownDownloaderDaemon(callback=callback)
+            RemoteDownloader.dldaemon.shutdown_downloader_daemon(callback=callback)
 
-def initController():
+def init_controller():
     """Intializes the download daemon controller.
 
     This doesn't actually start up the downloader daemon, that's done in
-    startupDownloader.  Commands will be queued until then.
+    startup_downloader.  Commands will be queued until then.
     """
     global daemon_starter
     daemon_starter = DownloadDaemonStarter()
 
-def startupDownloader():
+def startup_downloader():
     """Initialize the downloaders.
 
     This method currently does 2 things.  It deletes any stale files self in
@@ -676,30 +729,36 @@ def startupDownloader():
     """
     daemon_starter.startup()
 
-def shutdownDownloader(callback=None):
+def shutdown_downloader(callback=None):
     if daemon_starter:
         daemon_starter.shutdown(callback)
     elif callback:
         callback()
 
-def lookupDownloader(url):
-    return views.remoteDownloads.getItemWithIndex(indexes.downloadsByURL, url)
-
-def getExistingDownloaderByURL(url):
-    downloader = lookupDownloader(url)
-    return downloader
-
-def getExistingDownloader(item):
+def lookup_downloader(url):
     try:
-        return views.remoteDownloads.getObjectByID(item.downloader_id)
+        return RemoteDownloader.get_by_url(url)
     except ObjectNotFoundError:
         return None
 
-def getDownloader(item):
-    existing = getExistingDownloader(item)
+def get_existing_downloader_by_url(url):
+    downloader = lookup_downloader(url)
+    return downloader
+
+def get_existing_downloader(item):
+    try:
+        return RemoteDownloader.get_by_id(item.downloader_id)
+    except ObjectNotFoundError:
+        return None
+
+def get_downloader_for_item(item):
+    existing = get_existing_downloader(item)
+    if existing:
+        return existing
+    url = item.get_url()
+    existing = get_existing_downloader_by_url(url)
     if existing:
         return existing
-    url = item.getURL()
     channelName = unicodeToFilename(item.get_channel_title(True))
     if not channelName:
         channelName = None
