@@ -26,13 +26,12 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-"""Event loop handler.
+"""``miro.eventloop`` -- Event loop handler.
 
-This module handles the democracy event loop which is responsible for network
-requests and scheduling.
+This module handles the democracy event loop which is responsible for
+network requests and scheduling.
 
-TODO:
-    handle user setting clock back
+TODO: handle user setting clock back
 """
 
 import threading
@@ -126,6 +125,7 @@ class Scheduler(object):
 class CallQueue(object):
     def __init__(self):
         self.queue = Queue.Queue()
+        self.quit_flag = False
 
     def addIdle(self, function, name, args=None, kwargs=None):
         if args is None:
@@ -144,7 +144,7 @@ class CallQueue(object):
         return not self.queue.empty()
 
     def processIdles(self):
-        while self.hasPendingIdle():
+        while self.hasPendingIdle() and not self.quit_flag:
             self.processNextIdle()
 
 class ThreadPool(object):
@@ -208,7 +208,11 @@ class ThreadPool(object):
             
 class EventLoop(signals.SignalEmitter):
     def __init__(self):
-        signals.SignalEmitter.__init__(self, 'thread-started', 'begin-loop', 'end-loop')
+        signals.SignalEmitter.__init__(self, 'thread-will-start', 
+                                             'thread-started', 
+                                             'thread-did-start', 
+                                             'begin-loop', 
+                                             'end-loop')
         self.scheduler = Scheduler()
         self.idleQueue = CallQueue()
         self.urgentQueue = CallQueue()
@@ -252,8 +256,11 @@ class EventLoop(signals.SignalEmitter):
         self.threadPool.queueCall(callback, errback, function, name, *args, **kwargs)
 
     def loop(self):
-        self.emit('thread-started', threading.currentThread())
         self.loop_ready.set()
+        self.emit('thread-will-start')
+        self.emit('thread-started', threading.currentThread())
+        self.emit('thread-did-start')
+
         while not self.quitFlag:
             self.emit('begin-loop')
             self.clearRemovedCallbacks()
@@ -315,6 +322,11 @@ class EventLoop(signals.SignalEmitter):
                         del map[fd] 
                 yield callbackEvent
 
+    def quit(self):
+        self.quitFlag = True
+        self.idleQueue.quit_flag = True
+        self.urgentQueue.quit_flag = True
+
 _eventLoop = EventLoop()
 
 def addReadCallback(socket, callback):
@@ -417,7 +429,7 @@ def join():
 
 def quit():
     threadPoolQuit()
-    _eventLoop.quitFlag = True
+    _eventLoop.quit()
     _eventLoop.wakeup()
 
 def finished():
