@@ -26,6 +26,10 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
+"""``miro.flashscraper`` -- functions for converting a web-page url
+to a media url.
+"""
+
 import logging
 import re
 from miro import httpclient
@@ -36,6 +40,14 @@ from urllib import unquote_plus
 from miro.util import checkU
 
 def is_maybe_flashscrapable(url):
+    """Returns whether or not the given url is possibly handled by one of
+    the flash url converters we have.
+
+    Example:
+
+    >>> is_maybe_flashscrapable(u"http://www.youtube.com/watch?v=qRuNxHqwazs")
+    True
+    """
     return _get_scrape_function_for(url) is not None
 
 def try_scraping_url(url, callback):
@@ -67,19 +79,16 @@ def _scrape_youtube_url(url, callback):
     components = urlparse.urlsplit(url)
     params = cgi.parse_qs(components[3])
 
-    # if it's a watch url, we convert it to a form we already handle
     if components[2] == u'/watch' and 'v' in params:
-        url = 'http://www.youtube.com/v/%s&f=gdata_videos' % params["v"][0]
+        videoID = params['v'][0]
+    elif components[2].startswith('/v/'):
+        videoID = re.compile(r'/v/([\w-]+)').match(components[2]).group(1)
+    else:
+        logging.warning('_scrape_youtube_url: unable to scrape YouTube Video URL')
+        callback(None)
+        return
 
-    httpclient.grabHeaders(url, lambda x: _youtube_callback(x, callback),
-                           lambda x:_youtube_errback(x, callback))
-
-def _youtube_callback(info, callback):
-    redirected_url = info['redirected-url']
     try:
-        components = urlparse.urlsplit(redirected_url)
-        params = cgi.parse_qs(components[3])
-        videoID = params['video_id'][0]
         url = u"http://www.youtube.com/get_video_info?video_id=%s&el=embedded&ps=default&eurl=" % videoID
         httpclient.grabURL(url, lambda x: _youtube_callback_step2(x, videoID, callback),
                            lambda x: _youtube_errback(x, callback))
@@ -314,7 +323,7 @@ def _scrape_vimeo_errback(err, callback):
 # =============================================================================
 
 scraperInfoMap = [
-    {'pattern': re.compile(r'http://([^/]+\.)?youtube.com/(?!get_video(\.php)?)'), 'func': _scrape_youtube_url},
+    {'pattern': re.compile(r'http://([^/]+\.)?youtube.com/(watch|v)'), 'func': _scrape_youtube_url},
     {'pattern': re.compile(r'http://video.google.com/googleplayer.swf'), 'func': _scrape_google_video_url},
     {'pattern': re.compile(r'http://([^/]+\.)?lulu.tv/wp-content/flash_play/flvplayer'), 'func': _scrape_lulu_video_url},
     {'pattern': re.compile(r'http://([^/]+\.)?vmix.com/flash/super_player.swf'), 'func': _scrape_vmix_video_url},
