@@ -26,92 +26,93 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-from miro import indexes
+"""``miro.folder`` -- Holds ``Folder`` class and related things.
+"""
+
+from miro import feed
 from miro import playlist
 from miro import sorts
 from miro import util
-from miro import views
-from miro.database import DDBObject
-from miro.databasehelper import makeSimpleGetSet
+from miro.database import DDBObject, ObjectNotFoundError
+from miro.databasehelper import make_simple_get_set
 
 class FolderBase(DDBObject):
     """Base class for ChannelFolder and Playlist folder classes."""
 
-    def __init__(self, title):
+    def setup_new(self, title):
         self.title = title
         self.expanded = True
-        DDBObject.__init__(self)
 
-    get_title, setTitle = makeSimpleGetSet('title')
+    get_title, set_title = make_simple_get_set('title')
 
     def getExpanded(self):
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.expanded
 
     def setExpanded(self, newExpanded):
-        self.confirmDBThread()
+        self.confirm_db_thread()
         self.expanded = newExpanded
-        self.signalChange()
+        self.signal_change()
         for child in self.getChildrenView():
-            child.signalChange(needsSave=False)
-
-    def getNextTab(self):
-        """Get the first tab that isn't in this folder our.  If there are no
-        items afterwards, return None.
-        """
-        seenSelf = False
-        # Find the tab directly after this folder and move the tabs above
-        # that one.
-        for tab in self.getTabOrder().getView():
-            if not seenSelf and tab.obj is self: 
-                seenSelf = True
-            elif seenSelf and tab.obj.getFolder() is not self:
-                return tab.obj
-        return None
+            child.signal_change(needsSave=False)
 
     def remove(self, moveItemsTo=None):
-        children = [child for child in self.getChildrenView()]
-        for child in children:
-            child.remove(moveItemsTo)
-        DDBObject.remove(self)
+        """Remove this folder and children.
+        """
+        raise NotImplementedError()
 
-    # getFolder and setFolder are here so that channels/playlists and folders
+    # get_folder and set_folder are here so that channels/playlists and folders
     # have a consistent API.  They don't do much since we don't allow nested
     # folders.
-    def getFolder(self):
+    def get_folder(self):
         return None
 
-    def setFolder(self, newFolder):
+    def set_folder(self, newFolder, signal_items=False):
         if newFolder is not None:
             raise TypeError("Nested folders not allowed")
 
-    def getTabOrder(self):
-        """Return the TabOrder object that this folder belongs to."""
-        raise NotImplementedError()
-
     def getChildrenView(self):
         """Return the children of this folder."""
         raise NotImplementedError()
 
 class ChannelFolder(FolderBase):
-    def __init__(self, title, section=u'video'):
+    def setup_new(self, title, section=u'video'):
         self.section = section
-        FolderBase.__init__(self, title)
-        self._init_restore()
+        FolderBase.setup_new(self, title)
+        self.setup_common()
 
-    def onRestore(self):
-        FolderBase.onRestore(self)
-        self._init_restore()
+    def setup_restored(self):
+        self.setup_common()
 
-    def _init_restore(self):
+    def setup_common(self):
         self.itemSort = sorts.ItemSort()
         self.itemSortDownloading = sorts.ItemSort()
         self.itemSortWatchable = sorts.ItemSortUnwatchedFirst()
 
-    def getTabOrder(self):
-        return util.getSingletonDDBObject(views.channelTabOrder)
+    def remove(self, moveItemsTo=None):
+        children = list(self.getChildrenView())
+        for child in children:
+            if child.is_watched_folder():
+                child.setVisible(False) # just hide watched folders
+                child.set_folder(None)
+            else:
+                child.remove(moveItemsTo)
+        DDBObject.remove(self)
+
+    @classmethod
+    def video_view(cls):
+        return cls.make_view("section='video'")
+
+    @classmethod
+    def audio_view(cls):
+        return cls.make_view("section='audio'")
+
+    @classmethod
+    def get_by_title(cls, title):
+        return cls.make_view('title=?', (title,)).get_singleton()
+
     def getChildrenView(self):
-        return views.feeds.filterWithIndex(indexes.byFolder, self)
+        return feed.Feed.folder_view(self.id)
 
     def hasDownloadedItems(self):
         for feed in self.getChildrenView():
@@ -126,46 +127,67 @@ class ChannelFolder(FolderBase):
         return False
 
     # Returns string with number of unwatched videos in feed
-    def numUnwatched(self):
+    def num_unwatched(self):
         unwatched = 0
         for child in self.getChildrenView():
-            unwatched += child.numUnwatched()
+            unwatched += child.num_unwatched()
         return unwatched
 
     # Returns string with number of available videos in feed
-    def numAvailable(self):
+    def num_available(self):
         available = 0
         for child in self.getChildrenView():
-            available += child.numAvailable()
+            available += child.num_available()
         return available
 
-    def markAsViewed(self):
+    def mark_as_viewed(self):
         for child in self.getChildrenView():
-            child.markAsViewed()
-    
+            child.mark_as_viewed()
+
+class PlaylistFolderItemMap(playlist.PlaylistItemMap):
+    """Single row in the map that associates playlist folders with their 
+    child items.
+    """
+    def setup_new(self, playlist_id, item_id):
+        playlist.PlaylistItemMap.setup_new(self, playlist_id, item_id)
+        self.count = 1
+
+    def inc_count(self):
+        self.count += 1
+        self.signal_change()
+
+    def dec_count(self):
+        if self.count > 1:
+            self.count -= 1
+            self.signal_change()
+        else:
+            self.remove()
+
+    @classmethod
+    def add_item_id(cls, playlist_id, item_id):
+        view = cls.make_view('playlist_id=? AND item_id=?',
+                (playlist_id, item_id))
+        try:
+            map = view.get_singleton()
+            map.inc_count()
+        except ObjectNotFoundError:
+            cls(playlist_id, item_id)
+
+    @classmethod
+    def remove_item_id(cls, playlist_id, item_id):
+        view = cls.make_view('playlist_id=? AND item_id=?',
+                (playlist_id, item_id))
+        map = view.get_singleton()
+        map.dec_count()
+
 class PlaylistFolder(FolderBase, playlist.PlaylistMixin):
-    def __init__(self, title):
-        self.item_ids = []
-        self.setupTrackedItemView()
-        FolderBase.__init__(self, title)
+    MapClass = PlaylistFolderItemMap
 
-    def onRestore(self):
-        FolderBase.onRestore(self)
-        self.setupTrackedItemView()
-
-    def checkItemIDRemoved(self, id):
-        index = indexes.playlistsByItemAndFolderID
-        value = (id, self.getID())
-        view = views.playlists.filterWithIndex(index, value)
-        if view.len() == 0 and id in self.trackedItems:
-            self.removeID(id)
-
-    def checkItemIDAdded(self, id):
-        if id not in self.trackedItems:
-            self.trackedItems.appendID(id)
-
-    def getTabOrder(self):
-        return util.getSingletonDDBObject(views.playlistTabOrder)
+    def remove(self, moveItemsTo=None):
+        children = list(self.getChildrenView())
+        for child in children:
+            child.remove(moveItemsTo)
+        DDBObject.remove(self)
 
     def getChildrenView(self):
-        return views.playlists.filterWithIndex(indexes.byFolder, self)
+        return playlist.SavedPlaylist.folder_view(self.id)
