@@ -26,7 +26,18 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-"""Application class.  Portable code to handle the high-level running of Miro.
+"""``miro.frontends.widgets.application -- The application module
+holds :class:`Application` and the portable code to handle the high
+level running of the Miro application.
+
+It also holds:
+
+* :class:`InfoUpdater` -- tracks channel/item updates from the backend
+  and sends the information to the frontend
+* :class:`InfoUpdaterCallbackList` -- tracks the list of callbacks for
+  info updater
+* :class:`WidgetsMessageHandler` -- frontend message handler
+* :class:`FrontendStatesStore` -- stores state of the frontend
 """
 
 import os
@@ -68,6 +79,9 @@ from miro.plat.frontends.widgets.widgetset import Rect
 from miro.plat.utils import unicodeToFilename, FilenameType
 
 class Application:
+    """This class holds the portable application code.  Each platform
+    extends this class with a platform-specific version.
+    """
     def __init__(self):
         app.widgetapp = self
         self.ignore_errors = False
@@ -83,18 +97,26 @@ class Application:
         self.unwatched_count = 0
 
     def startup(self):
+        """Connects to signals, installs handlers, and calls :meth:`startup`
+        from the :mod:`miro.startup` module.
+        """
         self.connect_to_signals()
-        startup.install_movies_directory_gone_handler(self.handle_movies_gone)
+        startup.install_movies_directory_gone_handler(self.handle_movies_directory_gone)
         startup.install_first_time_handler(self.handle_first_time)
         startup.startup()
 
     def startup_ui(self):
+        """Starts up the widget ui by sending a bunch of messages
+        requesting data from the backend.  Also sets up managers,
+        initializes the ui, and displays the :class:`MiroWindow`.
+        """
         # Send a couple messages to the backend, when we get responses,
         # WidgetsMessageHandler() will call build_window()
         messages.TrackGuides().send_to_backend()
         messages.QuerySearchInfo().send_to_backend()
         messages.TrackWatchedFolders().send_to_backend()
         messages.QueryFrontendState().send_to_backend()
+        messages.TrackChannels().send_to_backend()
 
         app.item_list_controller_manager = \
                 itemlistcontroller.ItemListControllerManager()
@@ -127,19 +149,21 @@ class Application:
                 app.playback_manager.detached_window is None):
             return playback.handle_key_press(key, mods)
 
-    def handle_movies_gone(self, continue_callback):
-        call_on_ui_thread(lambda: self._handle_movies_gone(continue_callback))
+    def handle_movies_directory_gone(self, continue_callback):
+        call_on_ui_thread(self._handle_movies_directory_gone, continue_callback)
 
-    def _handle_movies_gone(self, continue_callback):
+    def _handle_movies_directory_gone(self, continue_callback):
         title = _("Movies directory gone")
         description = _(
-            "%(shortappname)s can't find the primary video directory "
+            "%(shortappname)s can't use the primary video directory "
             "located at:\n"
             "\n"
             "%(moviedirectory)s\n"
             "\n"
             "This may be because it is located on an external drive that "
-            "is not connected.\n"
+            "is not connected, is a directory that %(shortappname)s does "
+            "not have write permission to, or there is something that is "
+            "not a directory at that path.\n"
             "\n"
             "If you continue, the primary video directory will be reset "
             "to a location on this drive.  If you had videos downloaded "
@@ -188,11 +212,11 @@ class Application:
         videobox.controls.previous.connect('released', self.on_stop_fast_playback)
         videobox.controls.fullscreen.connect('clicked', self.on_fullscreen_clicked)
         self.window.show()
-        messages.TrackChannels().send_to_backend()
         messages.TrackPlaylists().send_to_backend()
         messages.TrackDownloadCount().send_to_backend()
         messages.TrackPausedCount().send_to_backend()
-        messages.TrackNewCount().send_to_backend()
+        messages.TrackNewVideoCount().send_to_backend()
+        messages.TrackNewAudioCount().send_to_backend()
         messages.TrackUnwatchedCount().send_to_backend()
 
     def get_main_window_dimensions(self):
@@ -302,8 +326,9 @@ class Application:
             basename = os.path.basename(filename)
             dialogs.show_message(
                 _("Error Revealing File"),
-                _("The file \"%(filename)s\" was deleted from outside Miro.",
-                  {"filename": basename}),
+                _("The file %(filename)s was deleted from outside %(appname)s.",
+                  {"filename": basename,
+                   "appname": config.get(prefs.SHORT_APP_NAME)}),
                 dialogs.WARNING_MESSAGE)
         else:
             self.reveal_file(filename)
@@ -374,8 +399,10 @@ class Application:
         # this gets called by the backend, so it has to send a message to
         # the frontend to open a dialog
         def up_to_date_callback():
-            messages.MessageToUser(_("Miro is up to date"),
-                                   _("Miro is up to date!")).send_to_frontend()
+            messages.MessageToUser(_("%(appname)s is up to date",
+                                     {'appname': config.get(prefs.SHORT_APP_NAME)}),
+                                   _("%(appname)s is up to date!",
+                                     {'appname': config.get(prefs.SHORT_APP_NAME)})).send_to_frontend()
 
         messages.CheckVersion(up_to_date_callback).send_to_backend()
 
@@ -391,9 +418,10 @@ class Application:
                 return
 
         external_count = len([s for s in selection if s.is_external])
+        folder_count = len([s for s in selection if s.is_container_item])
         total_count = len(selection)
 
-        if total_count == 1 and external_count == 0:
+        if total_count == 1 and external_count == folder_count == 0:
             messages.DeleteVideo(selection[0].id).send_to_backend()
             return
 
@@ -409,6 +437,16 @@ class Application:
 
                 external_count
             )
+            if folder_count > 0:
+                description += '\n\n' + ngettext(
+                    'One of these items is a folder.  Delete/Removeing a '
+                    "folder will delete/remove it's contents",
+
+                    'Some of these items are folders.  Delete/Removeing a '
+                    "folder will delete/remove it's contents",
+
+                    folder_count
+                )
             ret = dialogs.show_choice_dialog(title, description,
                                              [dialogs.BUTTON_REMOVE_ENTRY,
                                               dialogs.BUTTON_DELETE_FILE,
@@ -421,6 +459,16 @@ class Application:
                 total_count,
                 {"count": total_count}
             )
+            if folder_count > 0:
+                description += '\n\n' + ngettext(
+                    'One of these items is a folder.  Deleting a '
+                    "folder will delete it's contents",
+
+                    'Some of these items are folders.  Deleting a '
+                    "folder will delete it's contents",
+
+                    folder_count
+                )
             ret = dialogs.show_choice_dialog(title, description,
                                              [dialogs.BUTTON_DELETE,
                                               dialogs.BUTTON_CANCEL])
@@ -591,7 +639,9 @@ class Application:
 
     def export_feeds(self):
         title = _('Export OPML File')
-        filepath = dialogs.ask_for_save_pathname(title, "miro_subscriptions.opml")
+        slug = config.get(prefs.SHORT_APP_NAME).lower()
+        slug = slug.replace(' ', '_').replace('-', '_')
+        filepath = dialogs.ask_for_save_pathname(title, "%s_subscriptions.opml" % slug)
 
         if not filepath:
             return
@@ -719,15 +769,20 @@ class Application:
     def remove_current_site(self):
         t, infos = app.tab_list_manager.get_selection()
         if t == 'site':
-            info = infos[0] # Multiple guide selection is not allowed
-            title = _('Remove %(name)s', {"name": info.name})
-            description = _('Are you sure you want to remove %(name)s?',
-                            {"name": info.name})
+            title = ngettext('Remove site', 'Remove sites', len(infos))
+            description = ngettext(
+                'Are you sure you want to remove this site?',
+                'Are you sure you want to remove these %(count)s sites?',
+                len(infos),
+                {"count": len(infos)}
+                )
+
             ret = dialogs.show_choice_dialog(title, description,
                     [dialogs.BUTTON_REMOVE, dialogs.BUTTON_CANCEL])
 
             if ret == dialogs.BUTTON_REMOVE:
-                messages.DeleteSite(info.id).send_to_backend()
+                for si in infos:
+                    messages.DeleteSite(si.id).send_to_backend()
 
     def quit_ui(self):
         """Quit  out of the UI event loop."""
@@ -785,10 +840,10 @@ class Application:
         call_on_ui_thread(rundialog.run, dialog)
 
     def handle_update_available(self, obj, item):
-        print "FIXME - update available!"
+        print "Update available! -- not implemented"
 
     def handle_up_to_date(self):
-        print "FIXME - up to date!"
+        print "Up to date! -- not implemented"
 
     def handle_error(self, obj, report):
         call_on_ui_thread(self._handle_error, obj, report)
@@ -806,25 +861,42 @@ class Application:
         logging.info('Shutting down...')
 
 class InfoUpdaterCallbackList(object):
-    """Tracks the list of callbacks for InfoUpdater."""
+    """Tracks the list of callbacks for InfoUpdater.
+    """
 
     def __init__(self):
         self._callbacks = {}
 
-    def add(self, type, id, callback):
-        key = (type, id)
+    def add(self, type_, id_, callback):
+        """Adds the callback to the list for ``type_`` ``id_``.
+
+        :param type_: the type of the thing (feed, audio-feed, site, ...)
+        :param id_: the id for the thing
+        :param callback: the callback function to add
+        """
+        key = (type_, id_)
         self._callbacks.setdefault(key, set()).add(callback)
 
-    def remove(self, type, id, callback):
-        key = (type, id)
+    def remove(self, type_, id_, callback):
+        """Removes the callback from the list for ``type_`` ``id_``.
+
+        :param type_: the type of the thing (feed, audio-feed, site, ...)
+        :param id_: the id for the thing
+        :param callback: the callback function to remove
+        """
+        key = (type_, id_)
         callback_set = self._callbacks[key]
         callback_set.remove(callback)
         if len(callback_set) == 0:
             del self._callbacks[key]
 
-    def get(self, type, id):
-        """Get the list of callbacks for type, id."""
-        key = (type, id)
+    def get(self, type_, id_):
+        """Get the list of callbacks for ``type_``, ``id_``.
+
+        :param type_: the type of the thing (feed, audio-feed, site, ...)
+        :param id_: the id for the thing
+        """
+        key = (type_, id_)
         if key not in self._callbacks:
             return []
         else:
@@ -836,23 +908,23 @@ class InfoUpdaterCallbackList(object):
 class InfoUpdater(signals.SignalEmitter):
     """Track channel/item updates from the backend.
 
-    To track item updates, use add_item_callback().  To track tab
+    To track item updates, use ``add_item_callback()``.  To track tab
     updates, connect to one of the signals below.
 
     Signals:
 
-        feeds-added (self, info_list) -- New video feeds were added
-        feeds-changed (self, info_list) -- Video feeds were changed
-        feeds-removed (self, info_list) -- Video feeds were removed
-        audio-feeds-added (self, info_list) -- New audio feeds were added
-        audio-feeds-changed (self, info_list) -- Audio feeds were changed
-        audio-feeds-removed (self, info_list) -- Audio feeds were removed
-        sites-added (self, info_list) -- New sites were added
-        sites-changed (self, info_list) -- Sites were changed
-        sites-removed (self, info_list) -- Sites were removed
-        playlists-added (self, info_list) -- New playlists were added
-        playlists-changed (self, info_list) -- Playlists were changed
-        playlists-removed (self, info_list) -- Playlists were removed
+    * feeds-added (self, info_list) -- New video feeds were added
+    * feeds-changed (self, info_list) -- Video feeds were changed
+    * feeds-removed (self, info_list) -- Video feeds were removed
+    * audio-feeds-added (self, info_list) -- New audio feeds were added
+    * audio-feeds-changed (self, info_list) -- Audio feeds were changed
+    * audio-feeds-removed (self, info_list) -- Audio feeds were removed
+    * sites-added (self, info_list) -- New sites were added
+    * sites-changed (self, info_list) -- Sites were changed
+    * sites-removed (self, info_list) -- Sites were removed
+    * playlists-added (self, info_list) -- New playlists were added
+    * playlists-changed (self, info_list) -- Playlists were changed
+    * playlists-removed (self, info_list) -- Playlists were removed
     """
     def __init__(self):
         signals.SignalEmitter.__init__(self)
@@ -893,6 +965,13 @@ class InfoUpdater(signals.SignalEmitter):
             self.emit('%s-removed' % signal_start, message.removed)
 
 class WidgetsMessageHandler(messages.MessageHandler):
+    """Handles frontend messages.
+
+    There's a method to handle each frontend message type.  See
+    :mod:`miro.messages` (``portable/messages.py``) and
+    :mod:`miro.messagehandler` (``portable/messagehandler.py``) for
+    more details.
+    """
     def __init__(self):
         messages.MessageHandler.__init__(self)
         # Messages that we need to see before the UI is ready
@@ -901,7 +980,32 @@ class WidgetsMessageHandler(messages.MessageHandler):
             'search-info',
             'frontend-state',
         ])
+        if config.get(prefs.OPEN_CHANNEL_ON_STARTUP) is not None or \
+                config.get(prefs.OPEN_FOLDER_ON_STARTUP) is not None:
+            self._pre_startup_messages.add('feed-tab-list')
+            self._pre_startup_messages.add('audio-feed-tab-list')
         self.migration_progress_dialog = None
+        self.dbupgrade_progress_dialog = None
+
+    def handle_frontend_quit(self, message):
+        app.widgetapp.do_quit()
+
+    def handle_database_upgrade_start(self, message):
+        if self.dbupgrade_progress_dialog is None:
+            self.dbupgrade_progress_dialog = dialogs.DBUpgradeProgressDialog(
+                    _('Upgrading database'))
+            self.dbupgrade_progress_dialog.run()
+            # run() will return when we destroy the dialog because of a future
+            # message.
+            return
+
+    def handle_database_upgrade_progress(self, message):
+        self.dbupgrade_progress_dialog.update(message.stage,
+                message.stage_progress, message.total_progress)
+
+    def handle_database_upgrade_end(self, message):
+        self.dbupgrade_progress_dialog.destroy()
+        self.dbupgrade_progress_dialog = None
 
     def handle_startup_failure(self, message):
         dialogs.show_message(message.summary, message.description,
@@ -912,6 +1016,10 @@ class WidgetsMessageHandler(messages.MessageHandler):
         app.widgetapp.startup_ui()
 
     def _saw_pre_startup_message(self, name):
+        if name not in self._pre_startup_messages:
+            # we get here with the (audio-)?feed-tab-list messages when
+            # OPEN_(CHANNEL|FOLDER)_ON_STARTUP isn't defined
+            return
         self._pre_startup_messages.remove(name)
         if len(self._pre_startup_messages) == 0:
             app.widgetapp.build_window()
@@ -940,6 +1048,9 @@ class WidgetsMessageHandler(messages.MessageHandler):
     def handle_tab_list(self, message):
         tablist = self.tablist_for_message(message)
         tablist.reset_list(message)
+        if 'feed' in message.type:
+            pre_startup_message = message.type + '-tab-list'
+            self._saw_pre_startup_message(pre_startup_message)
 
     def handle_guide_list(self, message):
         app.widgetapp.default_guide_info = message.default_guide
@@ -982,23 +1093,27 @@ class WidgetsMessageHandler(messages.MessageHandler):
 
     def handle_item_list(self, message):
         app.info_updater.handle_item_list(message)
-        app.item_list_controller_manager.handle_playable_items()
+        app.menu_manager.update_menus()
 
     def handle_items_changed(self, message):
         app.info_updater.handle_items_changed(message)
-        app.item_list_controller_manager.handle_playable_items()
+        app.menu_manager.update_menus()
 
     def handle_download_count_changed(self, message):
         app.widgetapp.download_count = message.count
-        static_tab_list = app.tab_list_manager.static_tab_list
-        static_tab_list.update_download_count(message.count)
+        library_tab_list = app.tab_list_manager.library_tab_list
+        library_tab_list.update_download_count(message.count)
 
     def handle_paused_count_changed(self, message):
         app.widgetapp.paused_count = message.count
 
-    def handle_new_count_changed(self, message):
-        static_tab_list = app.tab_list_manager.static_tab_list
-        static_tab_list.update_new_count(message.count)
+    def handle_new_video_count_changed(self, message):
+        library_tab_list = app.tab_list_manager.library_tab_list
+        library_tab_list.update_new_video_count(message.count)
+
+    def handle_new_audio_count_changed(self, message):
+        library_tab_list = app.tab_list_manager.library_tab_list
+        library_tab_list.update_new_audio_count(message.count)
 
     def handle_unwatched_count_changed(self, message):
         app.widgetapp.unwatched_count = message.count
@@ -1028,7 +1143,7 @@ class WidgetsMessageHandler(messages.MessageHandler):
             app.search_manager.handle_search_complete(message)
 
     def handle_current_frontend_state(self, message):
-        app.list_view_memory = ListViewDisplayStore(message)
+        app.frontend_states_memory = FrontendStatesStore(message)
         self._saw_pre_startup_message('frontend-state')
 
     def handle_migration_progress(self, message):
@@ -1049,27 +1164,44 @@ class WidgetsMessageHandler(messages.MessageHandler):
             self.migration_progress_dialog.update(_('Migrating files'),
                     message.iteration, message.total_files)
 
-class ListViewDisplayStore(object):
-    """Stores which views were left in list mode by the user."""
+    def handle_feedless_download_started(self, message):
+        library_tab_list = app.tab_list_manager.library_tab_list
+        library_tab_list.blink_tab("downloading")
 
-    # Maybe this should get its own module, but I'm it seems small enough to
+class FrontendStatesStore(object):
+    """Stores which views were left in list mode by the user.
+    """
+
+    # Maybe this should get its own module, but it seems small enough to
     # me -- BDK
 
     def __init__(self, message):
         self.current_displays = set(message.list_view_displays)
+        self.sort_states = message.sort_states
 
     def _key(self, type, id):
         return '%s:%s' % (type, id)
 
-    def query(self, type, id):
+    def query_list_view(self, type, id):
         return self._key(type, id) in self.current_displays
 
-    def add(self, type, id):
-        self.current_displays.add(self._key(type, id))
-        m = messages.SaveFrontendState(list(self.current_displays))
+    def query_sort_state(self, type, id):
+        key = self._key(type, id)
+        if key in self.sort_states:
+            return self.sort_states[key]
+        return None
+
+    def set_sort_state(self, type, id, sorter):
+        self.sort_states[self._key(type, id)] = sorter
+        m = messages.SaveFrontendState(list(self.current_displays), self.sort_states)
         m.send_to_backend()
 
-    def remove(self, type, id):
+    def set_list_view(self, type, id):
+        self.current_displays.add(self._key(type, id))
+        m = messages.SaveFrontendState(list(self.current_displays), self.sort_states)
+        m.send_to_backend()
+
+    def set_std_view(self, type, id):
         self.current_displays.discard(self._key(type, id))
-        m = messages.SaveFrontendState(list(self.current_displays))
+        m = messages.SaveFrontendState(list(self.current_displays), self.sort_states)
         m.send_to_backend()
