@@ -36,6 +36,9 @@ import os
 from miro import app
 from miro import messages
 from miro import signals
+from miro import config
+from miro import prefs
+from miro import filetypes
 from miro.gtcache import gettext as _
 from miro.gtcache import ngettext
 from miro.frontends.widgets import browser
@@ -90,7 +93,7 @@ class TabDisplay(Display):
         raise NotImplementedError()
 
     def on_activate(self):
-        app.tab_list_manager.update_menus()
+        app.menu_manager.update_menus()
 
     @staticmethod
     def should_display(tab_type, selected_tabs):
@@ -113,9 +116,9 @@ class DisplayManager(object):
                 PlaylistDisplay,
                 SiteDisplay,
                 SearchDisplay,
-                LibraryDisplay,
-                IndividualDownloadsDisplay,
-                NewVideosDisplay,
+                VideoItemsDisplay,
+                AudioItemsDisplay,
+                OtherItemsDisplay,
                 DownloadingDisplay,
                 GuideDisplay,
                 MultipleSelectionDisplay,
@@ -187,6 +190,9 @@ class DisplayManager(object):
         display.cleanup()
         display.emit("removed")
 
+    def push_folder_contents_display(self, folder_info):
+        self.push_display(FolderContentsDisplay(folder_info))
+
 class GuideDisplay(TabDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
@@ -218,26 +224,14 @@ class SiteDisplay(TabDisplay):
             self._open_sites[guide_info.id] = browser.BrowserNav(guide_info)
         self.widget = self._open_sites[guide_info.id]
 
-class ItemListDisplay(TabDisplay):
-    def __init__(self, tab_type, selected_tabs):
-        Display.__init__(self)
-        tab = selected_tabs[0]
-        self.controller = self.make_controller(tab)
-        self.widget = self.controller.widget
-        if app.list_view_memory.query(tab_type, tab.id):
-            self.widget.switch_to_list_view()
-        self.type = tab_type
-        self.id = tab.id
-
+class ItemListDisplayMixin(object):
     def on_selected(self):
         app.item_list_controller_manager.controller_created(self.controller)
         self.controller.start_tracking()
 
     def on_activate(self):
-        TabDisplay.on_activate(self)
         app.item_list_controller_manager.controller_displayed(self.controller)
-        selected_items = self.controller.get_selection()
-        app.menu_manager.handle_item_list_selection(selected_items)
+        super(ItemListDisplayMixin, self).on_activate()
 
     def on_deactivate(self):
         app.item_list_controller_manager.controller_no_longer_displayed(
@@ -247,9 +241,21 @@ class ItemListDisplay(TabDisplay):
         self.controller.stop_tracking()
         app.item_list_controller_manager.controller_destroyed(self.controller)
         if self.widget.in_list_view:
-            app.list_view_memory.add(self.type, self.id)
+            app.frontend_states_memory.set_list_view(self.type, self.id)
         else:
-            app.list_view_memory.remove(self.type, self.id)
+            app.frontend_states_memory.set_std_view(self.type, self.id)
+
+
+class ItemListDisplay(ItemListDisplayMixin, TabDisplay):
+    def __init__(self, tab_type, selected_tabs):
+        Display.__init__(self)
+        tab = selected_tabs[0]
+        self.controller = self.make_controller(tab)
+        self.widget = self.controller.widget
+        if app.frontend_states_memory.query_list_view(tab_type, tab.id):
+            self.widget.switch_to_list_view()
+        self.type = tab_type
+        self.id = tab.id
 
     def make_controller(self, tab):
         raise NotImplementedError()
@@ -310,22 +316,6 @@ class PlaylistDisplay(ItemListDisplay):
     def make_controller(self, playlist_info):
         return playlist.PlaylistView(playlist_info)
 
-class DownloadingDisplay(ItemListDisplay):
-    @staticmethod
-    def should_display(tab_type, selected_tabs):
-        return tab_type == 'static' and selected_tabs[0].id == 'downloading'
-
-    def make_controller(self, tab):
-        return downloadscontroller.DownloadsController()
-
-class NewVideosDisplay(ItemListDisplay):
-    @staticmethod
-    def should_display(tab_type, selected_tabs):
-        return tab_type == 'static' and selected_tabs[0].id == 'new'
-
-    def make_controller(self, tab):
-        return itemlistcontroller.NewController()
-
 class SearchDisplay(ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
@@ -334,28 +324,55 @@ class SearchDisplay(ItemListDisplay):
     def make_controller(self, tab):
         return itemlistcontroller.SearchController()
 
-class LibraryDisplay(ItemListDisplay):
+class VideoItemsDisplay(ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'static' and selected_tabs[0].id == 'library'
+        return tab_type == 'library' and selected_tabs[0].id == 'videos'
 
     def make_controller(self, tab):
-        return itemlistcontroller.LibraryController()
+        return itemlistcontroller.VideoItemsController()
 
-class IndividualDownloadsDisplay(ItemListDisplay):
+class AudioItemsDisplay(ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'static' and selected_tabs[0].id == 'individual_downloads'
+        return tab_type == 'library' and selected_tabs[0].id == 'audios'
 
     def make_controller(self, tab):
-        return itemlistcontroller.IndividualDownloadsController()
+        return itemlistcontroller.AudioItemsController()
+
+class OtherItemsDisplay(ItemListDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return tab_type == 'library' and selected_tabs[0].id == 'others'
+
+    def make_controller(self, tab):
+        return itemlistcontroller.OtherItemsController()
+
+class DownloadingDisplay(ItemListDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return tab_type == 'library' and selected_tabs[0].id == 'downloading'
+
+    def make_controller(self, tab):
+        return downloadscontroller.DownloadsController()
+
+class FolderContentsDisplay(ItemListDisplayMixin, Display):
+    def __init__(self, info):
+        self.type = 'folder-contents'
+        self.id = info.id
+        self.controller = itemlistcontroller.FolderContentsController(info)
+        self.widget = self.controller.widget
+        Display.__init__(self)
 
 class CantPlayWidget(widgetset.SolidBackground):
     def __init__(self):
         widgetset.SolidBackground.__init__(self, (0, 0, 0))
         vbox = widgetset.VBox()
-        label = widgetset.Label(_("Miro can't play this file.  You may "
-            "be able to open it with a different program"))
+        label = widgetset.Label(_(
+            "%(appname)s can't play this file.  You may "
+            "be able to open it with a different program",
+            {"appname": config.get(prefs.SHORT_APP_NAME)}
+            ))
         label.set_color((1, 1, 1))
         vbox.pack_start(label)
         table = widgetset.Table(2, 2)
@@ -371,13 +388,13 @@ class CantPlayWidget(widgetset.SolidBackground):
         vbox.pack_start(widgetutil.align_left(table, top_pad=12))
         hbox = widgetset.HBox(spacing=12)
         reveal_button = widgetset.Button(_('Reveal File'))
-        play_externally_button = widgetset.Button(_('Play Externally'))
+        self.play_externally_button = widgetset.Button(_('Play Externally'))
+        self.play_externally_button.connect('clicked', self._on_play_externally)
         skip_button = widgetset.Button(_('Skip'))
         reveal_button.connect('clicked', self._on_reveal)
-        play_externally_button.connect('clicked', self._on_play_externally)
         skip_button.connect('clicked', self._on_skip)
         hbox.pack_start(reveal_button)
-        hbox.pack_start(play_externally_button)
+        hbox.pack_start(self.play_externally_button)
         hbox.pack_start(skip_button)
         vbox.pack_start(widgetutil.align_center(hbox, top_pad=24))
         alignment = widgetset.Alignment(xalign=0.5, yalign=0.5)
@@ -407,13 +424,17 @@ class CantPlayWidget(widgetset.SolidBackground):
         self.video_path = video_path
         self.filename_label.set_text(os.path.split(video_path)[-1])
         self.filetype_label.set_text(os.path.splitext(video_path)[1])
+        if filetypes.is_playable_filename(video_path):
+            self.play_externally_button.set_text(_('Play Externally'))
+        else:
+            self.play_externally_button.set_text(_('Open Externally'))
 
 class VideoDisplay(Display):
-    def __init__(self):
+    def __init__(self, renderer):
         Display.__init__(self)
         self.create_signal('cant-play')
         self.create_signal('ready-to-play')
-        self.renderer = widgetset.VideoRenderer()
+        self.renderer = renderer
         self.widget = widgetset.VBox()
         self.widget.pack_start(self.renderer, expand=True)
         self.cant_play_widget = CantPlayWidget()
@@ -445,43 +466,9 @@ class VideoDisplay(Display):
         self.show_renderer()
         self.cant_play_widget.set_video_path(item_info.video_path)
         self.item_info_id = item_info.id
-        self.renderer.set_movie_item(item_info, self._open_success,
-                self._open_error)
-        self.set_volume(volume)
-
-    def set_volume(self, volume):
+        self.renderer.set_item(item_info, self._open_success, self._open_error)
         self.renderer.set_volume(volume)
 
-    def get_elapsed_playback_time(self):
-        return self.renderer.get_elapsed_playback_time()
-
-    def get_total_playback_time(self):
-        return self.renderer.get_total_playback_time()
-
-    def play(self):
-        self.renderer.play()
-
-    def play_from_time(self, resume_time=0):
-        self.renderer.play_from_time(resume_time)
-
-    def pause(self):
-        self.renderer.pause()
-
-    def stop(self):
-        self.renderer.stop()
-
-    def set_playback_rate(self, rate):
-        self.renderer.set_playback_rate(rate)
-
-    def skip_forward(self):
-        self.renderer.skip_forward()
-
-    def skip_backward(self):
-        self.renderer.skip_backward()
-
-    def seek_to(self, position):
-        self.renderer.seek_to(position)
-
     def enter_fullscreen(self):
         self.renderer.enter_fullscreen()
         self.in_fullscreen = True
@@ -520,10 +507,12 @@ class MultipleSelectionDisplay(TabDisplay):
             tab_list = app.tab_list_manager.feed_list
         elif tab_type == 'audio-feed':
             tab_list = app.tab_list_manager.audio_feed_list
+        elif tab_type == 'site':
+            tab_list = app.tab_list_manager.site_list
         else:
             tab_list = app.tab_list_manager.playlist_list
         for tab in selected_tabs:
-            if tab.is_folder:
+            if hasattr(tab, "is_folder") and tab.is_folder:
                 self.folder_count += 1
                 self.folder_child_count += tab_list.get_child_count(tab.id)
             else:
@@ -534,7 +523,7 @@ class MultipleSelectionDisplay(TabDisplay):
         label.set_color((0.3, 0.3, 0.3))
         vbox.pack_start(widgetutil.align_center(label))
         vbox.pack_start(widgetutil.align_center(
-            self._make_buttons()))
+            self._make_buttons(tab_type)))
         self.widget = widgetutil.align_middle(vbox)
 
     def _make_label(self, tab_type, selected_tabs):
@@ -574,6 +563,12 @@ class MultipleSelectionDisplay(TabDisplay):
                         '%(count)d Feeds Selected',
                         self.child_count,
                         {"count": self.child_count}))
+            elif tab_type == "site":
+                label_parts.append(ngettext(
+                        '%(count)d Site Selected',
+                        '%(count)d Sites Selected',
+                        self.child_count,
+                        {"count": self.child_count}))
             else:
                 label_parts.append(ngettext(
                         '%(count)d Playlist Selected',
@@ -582,10 +577,10 @@ class MultipleSelectionDisplay(TabDisplay):
                         {"count": self.child_count}))
         return widgetset.Label('\n'.join(label_parts))
 
-    def _make_buttons(self):
+    def _make_buttons(self, tab_type):
         delete_button = widgetset.Button(_('Delete All'))
         delete_button.connect('clicked', self._on_delete_clicked)
-        if self.folder_count > 0:
+        if self.folder_count > 0 or tab_type == "site":
             return delete_button
         create_folder_button = widgetset.Button(_('Put Into a New Folder'))
         create_folder_button.connect('clicked', self._on_new_folder_clicked)
@@ -597,6 +592,8 @@ class MultipleSelectionDisplay(TabDisplay):
     def _on_delete_clicked(self, button):
         if self.type in ('feed', 'audio-feed'):
             app.widgetapp.remove_current_feed()
+        elif self.type == 'site':
+            app.widgetapp.remove_current_site()
         else:
             app.widgetapp.remove_current_playlist()
 
