@@ -35,6 +35,7 @@ import gtk
 import logging
 
 from miro import app
+from miro import player
 from miro.gtcache import gettext as _
 from miro import signals
 from miro import util
@@ -109,9 +110,9 @@ class ClickableImageButton(CustomButton):
     def draw(self, context, layout):
         self.image.draw(context, 0, 0, self.image.width, self.image.height)
 
-class NullRenderer(signals.SignalEmitter):
+class NullRenderer:
     def __init__(self):
-        signals.SignalEmitter.__init__(self, 'cant-play', 'ready-to-play')
+        pass
 
     def reset(self):
         pass
@@ -127,9 +128,11 @@ def make_label(text, handler, visible=True):
         lab.connect('clicked', handler)
         return lab
 
-    lab = Label(text)
+    # if the widget isn't visible, then we stick in an empty string--we just
+    # need a placeholder so that things don't move around when the item state
+    # changes.
+    lab = Label("")
     lab.set_size(0.77)
-    lab.set_color((0.0, 0.0, 0.0))
     return lab
 
 def make_image_button(image_path, handler):
@@ -178,13 +181,6 @@ def _window():
         return app.playback_manager.detached_window._window
     return app.widgetapp.window._window
 
-def can_play_file(path, yes_callback, no_callback):
-    if app.renderer is not None:
-        app.renderer.can_play_file(path, yes_callback, no_callback)
-        return
-    logging.warn("can_play_file: app.renderer is None")
-    no_callback()
-
 class VideoOverlay(Window):
     def __init__(self):
         Window.__init__(self, 'Miro Video Overlay')
@@ -246,85 +242,51 @@ class VideoDetailsWidget(Background):
 
         outer_hbox = HBox(5)
 
-        # left side
-        v = VBox()
-        self._item_name = Label(util.clampText(info.name, 100))
-        self._item_name.set_bold(True)
-        self._item_name.set_size(1.0)
-        self._item_name.set_color(WHITE)
-        v.pack_start(_align_left(self._item_name, left_pad=5, top_pad=3))
-
-        self._channel_name = Label("")
-        if not info.is_external:
-            channels = app.tab_list_manager.feed_list.get_feeds()
-            channels = [ci for ci in channels if ci.id == info.feed_id]
-            if len(channels) == 0:
-                self._channel_name.set_text("")
-            else:
-                self._channel_name.set_text(util.clampText(channels[0].name, 100))
-        self._channel_name.set_color(WHITE)
-        v.pack_start(_align_left(self._channel_name, left_pad=5, bottom_pad=3))
-
-        outer_hbox.pack_start(_align_middle(v), expand=True)
-
         if info.is_external:
-            if info.is_single:
-                outer_hbox.pack_start(_align_middle(make_label(_("Add to library"), self.handle_add_to_library, True)))
-            else:
-                self._delete_link = make_label(_("Delete"), self.handle_delete, True)
-                outer_hbox.pack_start(_align_middle(self._delete_link))
+            self._delete_link = make_label(_("Delete"), self.handle_delete, True)
+            outer_hbox.pack_start(_align_middle(self._delete_link))
+
         else:
-            # keep / delete
-            v = VBox()
-            lab = make_label(_("Keep"), self.handle_keep, info.expiration_date is not None)
-            v.pack_start(_align_center(lab, bottom_pad=2))
-
-            self._delete_link = make_label(_("Delete"), self.handle_delete)
-            v.pack_start(_align_center(self._delete_link))
-            outer_hbox.pack_start(_align_middle(v))
-
-            outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
-
-            # expires, share - link
-            v = VBox()
             if info.expiration_date is not None:
                 text = displaytext.expiration_date(info.expiration_date)
-            else:
-                text = " "
-            self._expiration_label = Label(text)
-            self._expiration_label.set_size(0.77)
-            self._expiration_label.set_color((152.0 / 255.0, 152.0 / 255.0, 152.0 / 255.0))
-            v.pack_start(_align_center(self._expiration_label, bottom_pad=2))
+                self._expiration_label = Label(text)
+                self._expiration_label.set_size(0.77)
+                self._expiration_label.set_color((152.0 / 255.0, 152.0 / 255.0, 152.0 / 255.0))
+                outer_hbox.pack_start(_align_middle(self._expiration_label))
+                outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
 
-            h2 = HBox()
-            lab = make_label(_("Share"), self.handle_share, info.has_sharable_url)
-            h2.pack_start(_align_center(lab))
+            lab = make_label(_("Keep"), self.handle_keep, info.expiration_date is not None)
+            outer_hbox.pack_start(_align_middle(lab))
+            outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
 
-            self._dash = Label("|")
-            self._dash.set_size(0.77)
-            self._dash.set_color(WHITE)
-            h2.pack_start(_align_middle(self._dash, left_pad=5, right_pad=5))
+            self._delete_link = make_label(_("Delete"), self.handle_delete)
+            outer_hbox.pack_start(_align_middle(self._delete_link))
+            outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
+
+            self._share_link = make_label(_("Share"), self.handle_share, info.has_sharable_url)
+            outer_hbox.pack_start(_align_middle(self._share_link))
+            outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
 
             if info.commentslink:
                 self._permalink_link = make_label(_("Comments"), self.handle_commentslink, info.commentslink)
             else:
                 self._permalink_link = make_label(_("Permalink"), self.handle_permalink, info.permalink)
-            h2.pack_start(self._permalink_link)
-            if not info.commentslink and not info.permalink:
-                self._dash.hide()
-            v.pack_start(_align_center(h2))
-            outer_hbox.pack_start(_align_middle(v))
+            outer_hbox.pack_start(_align_middle(self._permalink_link))
 
         outer_hbox.pack_start(_align_middle(Divider(), top_pad=3, bottom_pad=3, left_pad=5, right_pad=5))
 
         if app.playback_manager.detached_window is not None:
+            popin_link = make_label(_("Pop-in"), self.handle_popin_popout)
+            outer_hbox.pack_start(_align_middle(popin_link))
             popin_image = make_image_button(resources.path('images/popin.png'), self.handle_popin_popout)
-            outer_hbox.pack_start(_align_middle(popin_image, right_pad=15))
+            outer_hbox.pack_start(_align_middle(popin_image))
         else:
+            popout_link = make_label(_("Pop-out"), self.handle_popin_popout)
+            outer_hbox.pack_start(_align_middle(popout_link))
             popout_image = make_image_button(resources.path('images/popout.png'), self.handle_popin_popout)
-            outer_hbox.pack_start(_align_middle(popout_image, right_pad=15))
+            outer_hbox.pack_start(_align_middle(popout_image))
 
-        self.add(outer_hbox)
+        self.add(_align_right(outer_hbox, left_pad=15, right_pad=15))
 
     def hide(self):
         self._widget.hide()
@@ -339,9 +301,7 @@ class VideoDetailsWidget(Background):
 
     def handle_keep(self, widget):
         messages.KeepVideo(self.item_info.id).send_to_backend()
-
-    def handle_add_to_library(self, widget):
-        messages.AddItemToLibrary(self.item_info.id).send_to_backend()
+        self._widget.window.set_cursor(None)
 
     def handle_delete(self, widget):
         item_info = self.item_info
@@ -386,19 +346,19 @@ class VideoDetailsWidget(Background):
         if self._delete_link:
             self._delete_link.on_leave_notify(None, None)
 
-class VideoRenderer(VBox):
+class VideoPlayer(player.Player, VBox):
     """Video renderer widget.
 
-    Note: app.renderer must be initialized before instantiating this class.
-    If no renderers can be found, set app.renderer to None.
+    Note: ``app.video_renderer`` must be initialized before instantiating this
+    class.  If no renderers can be found, set ``app.video_renderer`` to ``None``.
     """
-
     HIDE_CONTROLS_TIMEOUT = 2000
 
     def __init__(self):
+        player.Player.__init__(self)
         VBox.__init__(self)
-        if app.renderer is not None:
-            self.renderer = app.renderer
+        if app.video_renderer is not None:
+            self.renderer = app.video_renderer
         else:
             self.renderer = NullRenderer()
 
@@ -436,9 +396,9 @@ class VideoRenderer(VBox):
     def update_for_presentation_mode(self, mode):
         pass
 
-    def set_movie_item(self, item_info, callback, errback):
+    def set_item(self, item_info, success_callback, error_callback):
         self._video_details.set_video_details(item_info)
-        self.renderer.select_file(item_info.video_path, callback, errback)
+        self.renderer.select_file(item_info.video_path, success_callback, error_callback)
         self._item_id = item_info.id
 
     def get_elapsed_playback_time(self):
@@ -456,13 +416,16 @@ class VideoRenderer(VBox):
         self.on_mouse_motion(None, None)
 
     def play_from_time(self, resume_time=0):
+        # Note: This overrides player.Player's version of play_from_time, but
+        # this one seeks directly rather than fiddling with
+        # total_playback_time.
         self.seek_to_time(resume_time)
         self.play()
 
     def pause(self):
         self.renderer.pause()
 
-    def stop(self):
+    def stop(self, will_play_another=False):
         self._video_details.reset()
         self.renderer.stop()
 
@@ -476,20 +439,6 @@ class VideoRenderer(VBox):
     def seek_to_time(self, time_pos):
         self.renderer.set_current_time(time_pos)
 
-    def skip_forward(self):
-        current = self.get_elapsed_playback_time()
-        duration = self.get_total_playback_time()
-        if duration <= 0.0:
-            pos = current + 30.0
-        else:
-            pos = min(duration, current + 30.0)
-        self.seek_to_time(pos)
-
-    def skip_backward(self):
-        current = self.get_elapsed_playback_time()
-        pos = max(0, current - 15.0)
-        self.seek_to_time(pos)
-
     def enter_fullscreen(self):
         self.screensaver_manager = screensaver.create_manager()
         if self.screensaver_manager is not None:
