@@ -30,6 +30,7 @@
 
 import StringIO
 
+import pango
 import gobject
 import gtk
 
@@ -131,6 +132,7 @@ class WindowBase(signals.SignalEmitter):
         self.create_signal('key-press')
         self.create_signal('show')
         self.create_signal('hide')
+        self._setup_ui_manager()
 
     def set_window(self, window):
         self._window = window
@@ -152,6 +154,73 @@ class WindowBase(signals.SignalEmitter):
         self.use_custom_style = ((base.red == base.green == base.blue) and
                 base.red >= 61680)
 
+    def connect_menu_keyboard_shortcuts(self):
+        self._window.add_accel_group(self.ui_manager.get_accel_group())
+
+    def _setup_ui_manager(self):
+        self.ui_manager = gtk.UIManager()
+        self.make_actions()
+        uistring = StringIO.StringIO()
+        uistring.write('<ui><menubar name="MiroMenu">')
+        extra_accelerators = []
+        for menu in menubar.menubar:
+            uistring.write('<menu action="Menu%s">' % menu.action)
+            for menuitem in menu.menuitems:
+                if isinstance(menuitem, menubar.Separator):
+                    uistring.write("<separator />")
+                else:
+                    uistring.write('<menuitem action="%s" />' % menuitem.action)
+                    if len(menuitem.shortcuts) > 1:
+                        extra_accelerators.append(menuitem)
+            uistring.write('</menu>')
+        uistring.write('</menubar>')
+        if extra_accelerators:
+            for menuitem in extra_accelerators:
+                for shortcut in menuitem.shortcuts[1:]:
+                    uistring.write('<accelerator action="%s%i" />' % (menuitem.action, id(shortcut)))
+        uistring.write('</ui>')
+        self.ui_manager.add_ui_from_string(uistring.getvalue())
+
+    def make_action(self, action, label, shortcuts=None):
+        gtk_action = gtk.Action(action, label, None, get_stock_id(action))
+        callback = menus.lookup_handler(action)
+        if callback is not None:
+            gtk_action.connect("activate", self.on_activate, callback)
+        action_group_name = menus.get_action_group_name(action)
+        action_group = self.action_groups[action_group_name]
+        if shortcuts is None or len(shortcuts) == 0:
+            action_group.add_action(gtk_action)
+        else:
+            action_group.add_action_with_accel(gtk_action,
+                    get_accel_string(shortcuts[0]))
+            for shortcut in shortcuts[1:]:
+                extra_action = gtk.Action(action + str(id(shortcut)), '', None,
+                                          None)
+                extra_action.set_visible(False)
+                if callback is not None:
+                    extra_action.connect('activate', self.on_activate,
+                                         callback)
+                action_group.add_action_with_accel(extra_action,
+                                                   get_accel_string(shortcut))
+
+    def make_actions(self):
+        self.action_groups = {}
+        for name in menus.action_group_names():
+            self.action_groups[name] = gtk.ActionGroup(name)
+
+        for menu in menubar.menubar:
+            self.make_action('Menu' + menu.action, menu.label)
+            for menuitem in menu.menuitems:
+                if isinstance(menuitem, menubar.Separator):
+                    continue
+                self.make_action(menuitem.action, menuitem.label,
+                        menuitem.shortcuts)
+        for action_group in self.action_groups.values():
+            self.ui_manager.insert_action_group(action_group, -1)
+
+    def on_activate(self, action, callback):
+        callback()
+
 class Window(WindowBase):
     """The main Miro window.  """
 
@@ -233,7 +302,8 @@ class MainWindow(Window):
         self.vbox = gtk.VBox()
         self._window.add(self.vbox)
         self.vbox.show()
-        self.add_menus()
+        self._add_menubar()
+        self.connect_menu_keyboard_shortcuts()
         self.create_signal('save-dimensions')
         self.create_signal('save-maximized')
         app.menu_manager.connect('enabled-changed', self.on_menu_change)
@@ -263,53 +333,10 @@ class MainWindow(Window):
             if gtk.gdk.keyval_name(event.keyval) in ('Right', 'Left', 'Up', 'Down'):
                 return True
 
-    def add_menus(self):
-        self.ui_manager = gtk.UIManager()
-        self.make_actions()
-        uistring = StringIO.StringIO()
-        uistring.write('<ui><menubar name="MiroMenu">')
-        for menu in menubar.menubar:
-            uistring.write('<menu action="Menu%s">' % menu.action)
-            for menuitem in menu.menuitems:
-                if isinstance(menuitem, menubar.Separator):
-                    uistring.write("<separator />")
-                else:
-                    uistring.write('<menuitem action="%s" />' % menuitem.action)
-            uistring.write('</menu>')
-        uistring.write('</menubar></ui>')
-        self.ui_manager.add_ui_from_string(uistring.getvalue())
+    def _add_menubar(self):
         self.menubar = self.ui_manager.get_widget("/MiroMenu")
         self.vbox.pack_start(self.menubar, expand=False)
         self.menubar.show_all()
-        self._window.add_accel_group(self.ui_manager.get_accel_group())
-
-    def make_action(self, action, label, shortcuts=None):
-        gtk_action = gtk.Action(action, label, None, get_stock_id(action))
-        callback = menus.lookup_handler(action)
-        if callback is not None:
-            gtk_action.connect("activate", self.on_activate, callback)
-        action_group_name = menus.get_action_group_name(action)
-        action_group = self.action_groups[action_group_name]
-        if shortcuts is None or len(shortcuts) == 0:
-            action_group.add_action(gtk_action)
-        else:
-            action_group.add_action_with_accel(gtk_action,
-                    get_accel_string(shortcuts[0]))
-
-    def make_actions(self):
-        self.action_groups = {}
-        for name in menus.action_group_names():
-            self.action_groups[name] = gtk.ActionGroup(name)
-
-        for menu in menubar.menubar:
-            self.make_action('Menu' + menu.action, menu.label)
-            for menuitem in menu.menuitems:
-                if isinstance(menuitem, menubar.Separator):
-                    continue
-                self.make_action(menuitem.action, menuitem.label,
-                        menuitem.shortcuts)
-        for action_group in self.action_groups.values():
-            self.ui_manager.insert_action_group(action_group, -1)
 
     def on_menu_change(self, menu_manager):
         for name, action_group in self.action_groups.items():
@@ -341,9 +368,6 @@ class MainWindow(Window):
         play_pause = menubar.menubar.getLabel("PlayPauseVideo", menu_manager.play_pause_state)
         self.action_groups["PlayPause"].get_action("PlayPauseVideo").set_property("label", play_pause)
 
-    def on_activate(self, action, callback):
-        callback()
-
     def _add_content_widget(self, widget):
         self.vbox.pack_start(widget._widget, expand=True)
 
@@ -509,30 +533,69 @@ class DirectorySelectDialog(FileDialogBase):
     def get_directory(self):
         return utils.FilenameType(self._files[0])
 
-class AboutDialog(DialogBase):
+class AboutDialog(Dialog):
     def __init__(self):
-        DialogBase.__init__(self)
-        self._text = None
-
-        ab = gtk.AboutDialog()
-        ab.set_name(config.get(prefs.SHORT_APP_NAME))
+        Dialog.__init__(self,
+                        _("About %(appname)s") % {
+                'appname': config.get(prefs.SHORT_APP_NAME)})
+        icon_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
+                resources.share_path('icons/hicolor/128x128/apps/miro.png'), 48, 48)
+        self.packing_vbox.pack_start(gtk.image_new_from_pixbuf(icon_pixbuf))
         if config.get(prefs.APP_REVISION_NUM):
-            ab.set_version("%s (r%s)" %
-                           (config.get(prefs.APP_VERSION),
-                            config.get(prefs.APP_REVISION_NUM)))
+            version = "%s (%s)" % (
+                    config.get(prefs.APP_VERSION),
+                    config.get(prefs.APP_REVISION_NUM))
         else:
-            ab.set_version("%s" % config.get(prefs.APP_VERSION))
-        ab.set_website(config.get(prefs.PROJECT_URL))
-        ab.set_copyright(_(
-            '%(copyright)s.  See LICENSE file for details.\n'
-            'Miro and Miro logo are trademarks of the Participatory '
-            'Culture Foundation.',
-            {"copyright": config.get(prefs.COPYRIGHT)}
-        ))
-        self._window = ab
+            version = "%s" % config.get(prefs.APP_VERSION)
+        name_label = gtk.Label('<span size="xx-large" weight="bold">%s %s</span>' % (
+                config.get(prefs.SHORT_APP_NAME), version))
+        name_label.set_use_markup(True)
+        self.packing_vbox.pack_start(name_label)
+        copyright_text = _('%(copyright)s.  See license.txt file for details.\n'
+                           '%(trademark)s') % (
+            {"copyright": config.get(prefs.COPYRIGHT),
+             "trademark": config.get(prefs.TRADEMARK)})
+        copyright_label = gtk.Label('<small>%s</small>' % copyright_text)
+        copyright_label.set_use_markup(True)
+        copyright_label.set_justify(gtk.JUSTIFY_CENTER)
+        self.packing_vbox.pack_start(copyright_label)
+        self.packing_vbox.pack_start(gtk.Label(config.get(prefs.PROJECT_URL)))
 
-    def _run(self):
-        self._window.run()
+        # this removes the newlines in the description so it wraps correctly
+        adopters_data = file(resources.path('ADOPTERS'), 'r').read()
+        first_double_newline = adopters_data.find('\n\n')
+        adopters_data = adopters_data[first_double_newline+2:-1].replace('\n', ', ')
+
+        # show the adopters
+        adopters_buffer = gtk.TextBuffer()
+        adopters_buffer.set_text(adopters_data)
+        iter = adopters_buffer.get_start_iter()
+        adopters_buffer.insert(iter,
+                               'Thanks to the following people who have adopted '
+                               'lines of Miro code to help support development! '
+                               'To adopt a line, visit the ')
+        adopt_link_tag = adopters_buffer.create_tag(
+            foreground='blue', underline=pango.UNDERLINE_SINGLE)
+        adopt_link_tag.connect('event', self.on_adopt_link_event)
+        adopters_buffer.insert_with_tags(iter, 'Miro Adoption Center',
+                                         adopt_link_tag)
+        adopters_buffer.insert(iter, '.\n\n')
+        adopters_view = gtk.TextView(adopters_buffer)
+        adopters_view.set_editable(False)
+        adopters_view.set_cursor_visible(False)
+        adopters_view.set_wrap_mode(gtk.WRAP_WORD)
+        adopters_window = gtk.ScrolledWindow()
+        adopters_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
+        adopters_window.add(adopters_view)
+        adopters_window.set_size_request(-1, 100)
+        self.packing_vbox.pack_start(adopters_window)
+
+        self.add_button(_("Close"))
+        self._window.set_has_separator(False)
+
+    def on_adopt_link_event(self, texttag, widget, event, iter):
+        if event.type == gtk.gdk.BUTTON_PRESS:
+            resources.open_url('https://www.getmiro.com/adopt/')
 
 type_map = {
     0: gtk.MESSAGE_WARNING,
