@@ -91,12 +91,16 @@ class ItemListController(object):
         self._search_text = ''
         self._init_widget()
         item_lists = set(iv.item_list for iv in self.all_item_views())
-        self.item_list_group = itemlist.ItemListGroup(item_lists)
+        sorter = app.frontend_states_memory.query_sort_state(type, id)
+        self.item_list_group = itemlist.ItemListGroup(item_lists, sorter)
         self._init_item_views()
         self.initialize_search()
+        sorter = self.item_list_group.get_sort()
+        self.widget.toolbar.change_sort_indicator(sorter.KEY, sorter.is_ascending())
+        self.list_item_view.change_sort_indicator(sorter.KEY, sorter.is_ascending())
 
     def _init_widget(self):
-        self.widget = itemlistwidgets.ItemContainerWidget()
+        self.widget = itemlistwidgets.ItemContainerWidget(self)
         self.item_list = itemlist.ItemList()
         self.list_item_view = self.build_list_item_view()
         scroller = widgetset.Scroller(True, True)
@@ -104,7 +108,14 @@ class ItemListController(object):
         self.widget.list_view_vbox.pack_start(scroller, expand=True)
         self.widget.toolbar.connect_weak('sort-changed', self.on_sort_changed)
         self.list_item_view.connect_weak('sort-changed', self.on_sort_changed)
+        if self.type in ('videos', 'audios'):
+            self.widget.toolbar.connect_weak('view-all-clicked', self.on_view_all_clicked)
+            self.widget.toolbar.connect_weak('toggle-unwatched-clicked', self.on_toggle_unwatched)
+            self.widget.toolbar.connect_weak('toggle-non-feed-clicked', self.on_toggle_non_feed)
         self.build_widget()
+        sorter = self.item_list.get_sort()
+        if sorter is not None:
+            self.widget.toolbar.change_sort_indicator(sorter.KEY, sorter.is_ascending())
 
     def build_list_item_view(self):
         return itemlistwidgets.ListItemView(self.item_list)
@@ -157,7 +168,7 @@ class ItemListController(object):
             self._play_item_list(items, presentation_mode)
             
     def filter_playable_items(self, items):
-        return [i for i in items if i.video_path is not None and i.video_path is not '' and i.downloaded]
+        return [i for i in items if i.is_playable]
 
     def _play_item_list(self, items, presentation_mode='fit-to-bounds'):
         playable = self.filter_playable_items(items)
@@ -184,26 +195,33 @@ class ItemListController(object):
         elif info.download_info is None:
             messages.StartDownload(info.id).send_to_backend()
 
+    def on_view_all_clicked(self, button):
+        self.widget.toolbar.switch_to_view_all()
+        self.item_list.view_all()
+        for item_view in self.all_item_views():
+            item_view.model_changed()
+
+    def on_toggle_unwatched(self, button):
+        self.widget.toolbar.toggle_unwatched_only()
+        self.item_list.toggle_unwatched_only()
+        for item_view in self.all_item_views():
+            item_view.model_changed()
+
+    def on_toggle_non_feed(self, button):
+        self.widget.toolbar.toggle_non_feed_only()
+        self.item_list.toggle_non_feed()
+        for item_view in self.all_item_views():
+            item_view.model_changed()
+
     def on_sort_changed(self, object, sort_key, ascending):
-        sort_key_map = {
-                'date': itemlist.DateSort,
-                'name': itemlist.NameSort,
-                'length': itemlist.LengthSort,
-                'size': itemlist.SizeSort,
-                'description': itemlist.DescriptionSort,
-                'feed-name': itemlist.FeedNameSort,
-                'state': itemlist.StatusCircleSort,
-                'status': itemlist.StatusSort,
-                'eta': itemlist.ETASort,
-                'rate': itemlist.DownloadRateSort,
-                'progress': itemlist.ProgressSort,
-        }
-        sorter = sort_key_map[sort_key](ascending)
+        sorter = itemlist.SORT_KEY_MAP[sort_key](ascending)
         for item_list in self.item_list_group.item_lists:
             item_list.set_sort(sorter)
         for item_view in self.all_item_views():
             item_view.model_changed()
+        self.widget.toolbar.change_sort_indicator(sort_key, ascending)
         self.list_item_view.change_sort_indicator(sort_key, ascending)
+        app.frontend_states_memory.set_sort_state(self.type, self.id, sorter)
 
     def on_key_press(self, view, key, mods):
         if key == menubar.DELETE:
@@ -250,6 +268,8 @@ class ItemListController(object):
             app.widgetapp.open_url(item_info.license)
         elif name == 'show_local_file':
             app.widgetapp.check_then_reveal_file(item_info.video_path)
+        elif name == 'show_contents':
+            app.display_manager.push_folder_contents_display(item_info)
         elif name.startswith('description-link:'):
             url = name.split(':', 1)[1]
             base_href = widgetutil.get_feed_info(item_info.feed_id).base_href
@@ -274,8 +294,7 @@ class ItemListController(object):
                 if other_view is not item_view:
                     other_view.unselect_all()
 
-        items = self.get_selection()
-        app.menu_manager.handle_item_list_selection(items)
+        app.menu_manager.update_menus()
 
     def start_tracking(self):
         """Send the message to start tracking items."""
@@ -284,6 +303,12 @@ class ItemListController(object):
                 self.handle_item_list)
         app.info_updater.item_changed_callbacks.add(self.type, self.id,
                 self.handle_items_changed)
+        self._playback_callbacks = [
+            app.playback_manager.connect('selecting-file',
+                self._on_playback_change),
+            app.playback_manager.connect('will-stop',
+                self._on_playback_change),
+        ]
 
     def stop_tracking(self):
         """Send the message to stop tracking items."""
@@ -292,6 +317,14 @@ class ItemListController(object):
                 self.handle_item_list)
         app.info_updater.item_changed_callbacks.remove(self.type, self.id,
                 self.handle_items_changed)
+        for handle in self._playback_callbacks:
+            app.playback_manager.disconnect(handle)
+
+    def _on_playback_change(self, playback_manager, *args):
+        # The currently playing item has changed, redraw the view to change
+        # which item gets the "currently playing" badge.
+        for item_view in self.all_item_views():
+            item_view.queue_redraw()
 
     def handle_item_list(self, message):
         """Handle an ItemList message meant for this ItemContainer."""
@@ -393,19 +426,6 @@ class SimpleItemListController(ItemListController):
         list_empty = (self.item_list.get_count() == 0)
         self.widget.set_list_empty_mode(list_empty)
 
-class NewController(SimpleItemListController):
-    type = 'new'
-    id = None
-    image_filename = 'icon-new_large.png'
-    title = _("New Videos")
-
-    def build_item_view(self):
-        return itemlistwidgets.ItemView(self.item_list, True)
-
-    def build_list_item_view(self):
-        return itemlistwidgets.ListItemView(self.item_list,
-                display_download_info=False)
-
 class SearchController(SimpleItemListController):
     type = 'search'
     id = None
@@ -482,11 +502,30 @@ class SearchController(SimpleItemListController):
         if search_manager.text != '' and result_count == 0:
             self.widget.set_list_empty_mode(True)
 
-class LibraryController(SimpleItemListController):
-    type = 'library'
+class AudioVideoItemsController(SimpleItemListController):
+    def build_item_view(self):
+        return itemlistwidgets.ItemView(self.item_list, True)
+    def build_list_item_view(self):
+        return itemlistwidgets.ListItemView(self.item_list,
+                display_download_info=False)
+
+class VideoItemsController(AudioVideoItemsController):
+    type = 'videos'
     id = None
-    image_filename = 'icon-library_large.png'
-    title = _("Library")
+    image_filename = 'icon-video_large.png'
+    title = _("Video")
+
+class AudioItemsController(AudioVideoItemsController):
+    type = 'audios'
+    id = None
+    image_filename = 'icon-audio_large.png'
+    title = _("Audio")
+
+class OtherItemsController(SimpleItemListController):
+    type = 'others'
+    id = None
+    image_filename = 'icon-other_large.png'
+    title = _("Other")
 
     def build_item_view(self):
         return itemlistwidgets.ItemView(self.item_list, True)
@@ -495,21 +534,29 @@ class LibraryController(SimpleItemListController):
         return itemlistwidgets.ListItemView(self.item_list,
                 display_download_info=False)
 
-class IndividualDownloadsController(SimpleItemListController):
-    type = 'individual_downloads'
-    id = None
-    image_filename = 'icon-individual_large.png'
-    title = _("Single Items")
+class FolderContentsController(SimpleItemListController):
+    """Controller object for feeds."""
+
+    def __init__(self, folder_info):
+        self.type = 'folder-contents'
+        self.id = folder_info.id
+        self.title = folder_info.name
+        self.info = folder_info
+        SimpleItemListController.__init__(self)
+
+    def _make_icon(self):
+        return imagepool.get(resources.path('images/folder-icon.png'))
 
     def build_widget(self):
         SimpleItemListController.build_widget(self)
-        text = _('No Items to Display')
-        self.widget.list_empty_mode_vbox.pack_start(
-                itemlistwidgets.EmptyListHeader(text))
-        text = _('This tab keeps track of downloads that aren\'t part of '
-                'a feed.')
-        self.widget.list_empty_mode_vbox.pack_start(
-                itemlistwidgets.EmptyListDescription(text))
+
+        button = widgetset.Button(_('Back to feed'))
+        button.connect('clicked', self._on_clicked)
+        self.widget.titlebar_vbox.pack_start(widgetutil.align_left(button,
+            left_pad=10, top_pad=6, bottom_pad=4))
+
+    def _on_clicked(self, button):
+        app.display_manager.pop_display()
 
 class ItemListControllerManager(object):
     """Manages ItemListController objects.
@@ -552,11 +599,10 @@ class ItemListControllerManager(object):
         else:
             return self.displayed.get_selection()
 
-    def handle_playable_items(self):
+    def get_current_playlist(self):
+        """Get the items that would be played if we started playback."""
         if self.displayed is not None:
             selection = self.displayed.get_selection_for_playing()
-            playable = self.displayed.filter_playable_items(selection)
-            has_playable = len(playable) > 0
+            return self.displayed.filter_playable_items(selection)
         else:
-            has_playable = False
-        app.widgetapp.window.videobox.handle_new_selection(has_playable)
+            return []
