@@ -46,6 +46,8 @@ Refer to documentation for those functions for help.
 """
 
 import logging
+import sys
+import os
 
 from miro import app
 from miro import config
@@ -264,7 +266,7 @@ class GeneralPanel(PanelBuilder):
     def build_widget(self):
         v = widgetset.VBox(8)
 
-        run_at_startup_cbx = widgetset.Checkbox(_("Automatically run Miro when I log in."))
+        run_at_startup_cbx = widgetset.Checkbox(_("Automatically run %(appname)s when I log in.", {'appname': config.get(prefs.SHORT_APP_NAME)}))
         attach_boolean(run_at_startup_cbx, prefs.RUN_AT_STARTUP)
         v.pack_start(run_at_startup_cbx)
 
@@ -362,7 +364,9 @@ class DownloadsPanel(PanelBuilder):
         limit = widgetset.TextEntry()
         limit.set_width(5)
         attach_boolean(cbx, prefs.LIMIT_UPSTREAM, (limit,))
-        attach_integer(limit, prefs.UPSTREAM_LIMIT_IN_KBS, create_integer_checker(min=0))
+        max_kbs = sys.maxint / (2**10) # highest value accepted: sys.maxint
+                                       # bits per second in kb/s
+        attach_integer(limit, prefs.UPSTREAM_LIMIT_IN_KBS, create_integer_checker(min=0, max=max_kbs))
 
         grid.pack(cbx)
         grid.pack(limit)
@@ -373,7 +377,7 @@ class DownloadsPanel(PanelBuilder):
         limit = widgetset.TextEntry()
         limit.set_width(5)
         attach_boolean(cbx, prefs.LIMIT_DOWNSTREAM_BT, (limit,))
-        attach_integer(limit, prefs.DOWNSTREAM_BT_LIMIT_IN_KBS, create_integer_checker(min=0))
+        attach_integer(limit, prefs.DOWNSTREAM_BT_LIMIT_IN_KBS, create_integer_checker(min=0, max=max_kbs))
 
         grid.pack(cbx)
         grid.pack(limit)
@@ -384,7 +388,7 @@ class DownloadsPanel(PanelBuilder):
         limit = widgetset.TextEntry()
         limit.set_width(5)
         attach_boolean(cbx, prefs.LIMIT_CONNECTIONS_BT, (limit,))
-        attach_integer(limit, prefs.CONNECTION_LIMIT_BT_NUM, create_integer_checker(min=0))
+        attach_integer(limit, prefs.CONNECTION_LIMIT_BT_NUM, create_integer_checker(min=0, max=65536))
 
         grid.pack(cbx)
         grid.pack(limit)
@@ -437,12 +441,28 @@ class _MovieDirectoryHelper(object):
         self.button.connect('clicked', self._on_button_clicked)
 
     def _on_button_clicked(self, button):
-        dir = dialogs.ask_for_directory(_("Choose Movies Directory"),
-                initial_directory=config.get(prefs.MOVIES_DIRECTORY),
-                transient_for=_pref_window)
-        if dir is not None:
-            self.path = dir
-            self.label.set_text(filenameToUnicode(dir))
+        d = dialogs.ask_for_directory(_("Choose Movies Directory"),
+                                      initial_directory=config.get(prefs.MOVIES_DIRECTORY),
+                                      transient_for=_pref_window)
+        if d is not None:
+            try:
+                if not os.path.exists(d):
+                    os.makedirs(d)
+                fn = os.path.join(d, "testfile")
+                f = open(fn, "w")
+                f.write("test")
+                f.close()
+                os.remove(fn)
+            except (OSError, IOError):
+                dialogs.show_message(_("Directory not valid"),
+                                     _("Directory '%s' could not be created.  " +
+                                       "Please choose a directory you have " +
+                                       "write access to."),
+                                     dialogs.WARNING_MESSAGE)
+                return
+            logging.info("Created directory.  It's valid.")
+            self.path = d
+            self.label.set_text(filenameToUnicode(d))
 
     def set_initial_path(self):
         self.path = self.initial_path = config.get(prefs.MOVIES_DIRECTORY)
@@ -452,9 +472,9 @@ class _MovieDirectoryHelper(object):
         if self.path != self.initial_path:
             title = _("Migrate existing movies?")
             description = _("You've selected a new folder to download movies "
-                    "to.  Should Miro migrate your existing downloads there? "
-                    "(Currently dowloading movies will not be moved until "
-                    "they finish.)")
+                    "to.  Should %(appname)s migrate your existing downloads there? "
+                    "(Currently downloading movies will not be moved until "
+                    "they finish.)", {'appname': config.get(prefs.SHORT_APP_NAME)})
             response = dialogs.show_choice_dialog(title, description, 
                     (dialogs.BUTTON_MIGRATE, dialogs.BUTTON_DONT_MIGRATE),
                     transient_for=_pref_window)
@@ -477,6 +497,7 @@ class _WatchedFolderHelper(object):
         visible_column.set_min_width(50)
         self._table.add_column(folder_column)
         self._table.add_column(visible_column)
+        self._table.set_fixed_height(True)
         self._table.allow_multiple_select(False)
         self._table.set_alternate_row_backgrounds(True)
         self.add_button = widgetset.Button(_("Add"))
@@ -546,8 +567,8 @@ class FoldersPanel(PanelBuilder):
         grid.pack(self.movie_dir_helper.label, grid.ALIGN_LEFT, pad_left=12)
         grid.pack(self.movie_dir_helper.button)
         grid.end_line(spacing=18)
-        grid.pack_label(_('Watch for new videos in these folders and include '
-            'them in library:'), span=2)
+        grid.pack_label(_('Watch for new video and audio items in these folders '
+                          'and include them in library:'), span=2)
         grid.end_line()
         grid.pack(self.watched_folder_helper.folder_list, pad_right=12)
         grid.pack(self.watched_folder_helper.button_box)
@@ -587,7 +608,7 @@ class DiskSpacePanel(PanelBuilder):
         attach_combo(expire_menu, prefs.EXPIRE_AFTER_X_DAYS,
                 [op[0] for op in expire_ops])
 
-        grid.pack_label(_('By default, videos expire after:'), extra_space=dialogwidgets.ControlGrid.ALIGN_RIGHT)
+        grid.pack_label(_('By default, video and audio items expire after:'), extra_space=dialogwidgets.ControlGrid.ALIGN_RIGHT)
         grid.pack(expire_menu, extra_space=dialogwidgets.ControlGrid.ALIGN_LEFT)
 
         return grid.make_table()
@@ -600,13 +621,13 @@ class PlaybackPanel(PanelBuilder):
         attach_boolean(cbx, prefs.PLAY_DETACHED)
         v.pack_start(widgetutil.align_left(cbx, bottom_pad=6))
 
-        cbx = widgetset.Checkbox(_('Resume playing a video from the point it was last stopped.'))
+        cbx = widgetset.Checkbox(_('Resume playing a video or audio item from the point it was last stopped.'))
         attach_boolean(cbx, prefs.RESUME_VIDEOS_MODE)
         v.pack_start(widgetutil.align_left(cbx, bottom_pad=12))
 
         rbg = widgetset.RadioButtonGroup()
-        play_rb = widgetset.RadioButton(_("Play videos one after another"), rbg)
-        stop_rb = widgetset.RadioButton(_("Stop after each video"), rbg)
+        play_rb = widgetset.RadioButton(_("Play video and audio items one after another"), rbg)
+        stop_rb = widgetset.RadioButton(_("Stop after each video or audio item"), rbg)
         attach_radio( [(stop_rb, True), (play_rb, False)], prefs.SINGLE_VIDEO_PLAYBACK_MODE)
         v.pack_start(widgetutil.align_left(play_rb), padding=2)
         v.pack_start(widgetutil.align_left(stop_rb))
