@@ -62,6 +62,7 @@ TAB_LIST_HEADER_COLOR = (100/255.0, 109/255.0, 125/255.0)
 TAB_LIST_SEPARATOR_COLOR = (209/255.0, 216/255.0, 220/255.0)
 
 ERROR_COLOR = (0.90, 0.0, 0.0)
+BLINK_COLOR = css_to_color('#fffb83')
 
 class LowerBox(widgetset.Background):
 
@@ -129,13 +130,15 @@ class TabRenderer(widgetset.CustomCellRenderer):
         if selected:
             bold = True
         elif not hasattr(self.data, "bolded") or self.data.bolded:
-            bold=self.BOLD_TITLE
+            bold = self.BOLD_TITLE
         layout.set_font(self.TITLE_FONT_SIZE, bold=bold)
         titlebox = layout.textbox(self.data.name)
 
         hbox = cellpack.HBox(spacing=4)
-        if hasattr(self.data, "indent") and self.data.indent:
-            hbox.pack_space(15)
+        if hasattr(self.data, "id") and (self.data.id == 'guide' or self.data.id == 'search'):
+            hbox.pack_space(6)
+        else:
+            hbox.pack_space(2)
         alignment = cellpack.Alignment(self.data.icon, yalign=0.5, yscale=0.0,
                 xalign=0.5, xscale=0.0, min_width=16)
         hbox.pack(alignment)
@@ -145,7 +148,12 @@ class TabRenderer(widgetset.CustomCellRenderer):
         self.pack_bubbles(hbox, layout)
         hbox.pack_space(2)
         alignment = cellpack.Alignment(hbox, yscale=0.0, yalign=0.5)
-        alignment.render_layout(context)
+        if self.blink:
+            renderer = cellpack.Background(alignment)
+            renderer.set_callback(self.draw_blink_background)
+        else:
+            renderer = alignment
+        renderer.render_layout(context)
 
     def pack_bubbles(self, hbox, layout):
         if getattr(self.data, 'is_updating', None):
@@ -181,6 +189,11 @@ class TabRenderer(widgetset.CustomCellRenderer):
         context.set_color(color)
         context.fill()
 
+    def draw_blink_background(self, context, x, y, width, height):
+        context.rectangle(x, y, width, height)
+        context.set_color(BLINK_COLOR)
+        context.fill()
+
 class StaticTabRenderer(TabRenderer):
     BOLD_TITLE = True
     MIN_HEIGHT_BIG = 28
@@ -223,6 +236,7 @@ class ItemRenderer(widgetset.CustomCellRenderer):
     COMMENTS_TEXT = _("Comments")
     REVEAL_IN_TEXT = (file_navigator_name and
             _("Reveal in %(progname)s", {"progname": file_navigator_name}) or _("Reveal File"))
+    SHOW_CONTENTS_TEXT = _("Display Contents")
     WEB_PAGE_TEXT = _("Web Page")
     FILE_URL_TEXT = _("File URL")
     LICENSE_PAGE_TEXT = _("License Page")
@@ -238,6 +252,7 @@ class ItemRenderer(widgetset.CustomCellRenderer):
     ERROR_TEXT = _("Error")
     QUEUED_TEXT = _("queued for autodownload")
     UNPLAYED_TEXT = _("Unplayed")
+    CURRENTLY_PLAYING_TEXT = _("Currently Playing")
     NEWLY_AVAILABLE_TEXT = _("Newly Available")
     KEEP_TEXT = _("Keep")
     REMOVE_TEXT = _("Remove")
@@ -373,7 +388,7 @@ class ItemRenderer(widgetset.CustomCellRenderer):
             title.set_width(main_width)
             vbox.pack(title)
 
-        if self.display_channel:
+        if self.display_channel and self.data.feed_name is not None:
             vbox.pack_space(1)
             hbox = cellpack.HBox()
             hbox.pack(cellpack.align_middle(self.channel_title_icon))
@@ -674,92 +689,123 @@ class ItemRenderer(widgetset.CustomCellRenderer):
         if self.show_progress_bar:
             return cellpack.align_bottom(self.pack_download_status(layout))
 
-        hbox = cellpack.HBox(spacing=10)
+        stack = cellpack.Stack()
+
+        main_hbox = cellpack.HBox(spacing=10)
+
         layout.set_font(0.85)
         if self.data.downloaded:
-            hbox.pack(cellpack.align_middle(cellpack.Hotspot('play', self.play_button)))
+            if self.data.is_playable:
+                button = cellpack.Hotspot('play', self.play_button)
+            else:
+                button = self._make_button(layout, self.REVEAL_IN_TEXT,
+                        'show_local_file')
+            main_hbox.pack(cellpack.align_middle(button))
         else:
-            if self.data.file_type == 'application/x-bittorrent':
+            if self.data.mime_type == 'application/x-bittorrent':
                 text = self.DOWNLOAD_TORRENT_TEXT
             else:
                 text = self.DOWNLOAD_TEXT
             hotspot = self._make_button(layout, text, 'download',
                     icon=self.download_arrow)
-            hbox.pack(cellpack.align_middle(cellpack.align_middle(hotspot)))
+            main_hbox.pack(cellpack.align_middle(cellpack.align_middle(hotspot)))
+
+        # If we are going to display an emblem (unwatched, expiring, etc).
+        # Then pack it now on to the stack, so the buttons and other things
+        # get packed on top.
+
+        emblem_hbox = cellpack.HBox()
+        main_width = main_hbox.get_current_size()[0]
+        emblem_hbox.pack_space(main_width)
 
         if self.data.download_info and self.data.download_info.state == 'failed':
             layout.set_font(0.80, bold=True)
 
-            inner_hbox = hbox
-
-            inner_hbox.pack(cellpack.align_middle(self.alert_image))
-            inner_hbox.pack(cellpack.align_middle(layout.textbox(self.ERROR_TEXT)))
-            inner_hbox.pack(cellpack.align_middle(layout.textbox(u"-")))
-            inner_hbox.pack(cellpack.align_middle(layout.textbox(self.data.download_info.short_reason_failed)))
+            emblem_hbox.pack(cellpack.align_middle(self.alert_image))
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(self.ERROR_TEXT)))
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(u"-")))
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(self.data.download_info.short_reason_failed)))
 
             emblem_color = (1.0, 252.0 / 255.0, 183.0 / 255.0)
-            emblem = cellpack.Background(inner_hbox, margin=(4, 20, 4, 4))
+            emblem = cellpack.Background(emblem_hbox, margin=(4, 20, 4, 4))
             emblem.set_callback(self.draw_emblem, emblem_color)
 
-            hbox = cellpack.HBox(spacing=5)
-            hbox.pack(cellpack.pad(emblem))
+            stack.pack(cellpack.align_left(emblem))
 
         elif self.data.pending_auto_dl:
-            hbox.pack_space(2)
+            emblem_hbox.pack_space(2)
             layout.set_font(0.80, bold=True)
-            hbox.pack(cellpack.align_middle(layout.textbox(self.QUEUED_TEXT)))
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(self.QUEUED_TEXT)))
 
-        elif self.data.downloaded and not self.data.video_watched:
+            stack.pack(cellpack.align_left(emblem_hbox))
+
+        elif (self.data.downloaded and app.playback_manager.get_playing_item() and
+                app.playback_manager.get_playing_item().id == self.data.id):
             layout.set_font(0.80, bold=True)
             layout.set_text_color((1, 1, 1))
-            inner_hbox = hbox
-            inner_hbox.pack(cellpack.align_middle(layout.textbox(self.UNPLAYED_TEXT)))
-            inner_hbox.pack_space(2)
+
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(self.CURRENTLY_PLAYING_TEXT)))
+            emblem_hbox.pack_space(2)
 
             emblem_color = UNPLAYED_COLOR
-            emblem = cellpack.Background(inner_hbox, margin=(5, 20, 4, 4))
+            emblem = cellpack.Background(emblem_hbox, margin=(5, 20, 4, 4))
             emblem.set_callback(self.draw_emblem, emblem_color)
 
-            hbox = cellpack.HBox(spacing=5)
-            hbox.pack(cellpack.pad(emblem))
+            stack.pack(cellpack.align_left(emblem))
 
-        elif self.data.expiration_date:
+        elif (self.data.downloaded and not self.data.video_watched and
+                self.data.is_playable):
+            layout.set_font(0.80, bold=True)
+            layout.set_text_color((1, 1, 1))
+
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(self.UNPLAYED_TEXT)))
+            emblem_hbox.pack_space(2)
+
+            emblem_color = UNPLAYED_COLOR
+            emblem = cellpack.Background(emblem_hbox, margin=(5, 20, 4, 4))
+            emblem.set_callback(self.draw_emblem, emblem_color)
+
+            stack.pack(cellpack.align_left(emblem))
+        elif self.data.expiration_date and self.data.is_playable:
             layout.set_font(0.80, bold=True)
             layout.set_text_color((154.0 / 255.0, 174.0 / 255.0, 181.0 / 255.0))
+
             text = displaytext.expiration_date(self.data.expiration_date)
-            inner_hbox = hbox
-            inner_hbox.pack(cellpack.align_middle(layout.textbox(text)))
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(text)))
             emblem_color = (232.0 / 255.0, 240.0 / 255.0, 242.0 / 255.0)
-            emblem = cellpack.Background(inner_hbox, margin=(4, 4, 4, 4))
+            emblem = cellpack.Background(emblem_hbox, margin=(4, 4, 4, 4))
             emblem.set_callback(self.draw_emblem, emblem_color)
 
-            hbox = cellpack.HBox(spacing=5)
-            hbox.pack(cellpack.pad(emblem))
-
+            stack.pack(cellpack.align_left(emblem))
         elif not self.data.item_viewed and self.data.state == "new":
             layout.set_font(0.80, bold=True)
             layout.set_text_color((1, 1, 1))
-            inner_hbox = hbox
-            inner_hbox.pack(cellpack.align_middle(layout.textbox(self.NEWLY_AVAILABLE_TEXT)))
-            inner_hbox.pack_space(2)
+
+            emblem_hbox.pack(cellpack.align_middle(layout.textbox(self.NEWLY_AVAILABLE_TEXT)))
+            emblem_hbox.pack_space(2)
 
             emblem_color = AVAILABLE_COLOR
-            emblem = cellpack.Background(inner_hbox, margin=(4, 4, 4, 4))
+            emblem = cellpack.Background(emblem_hbox, margin=(4, 4, 4, 4))
             emblem.set_callback(self.draw_emblem, emblem_color)
 
-            hbox = cellpack.HBox(spacing=5)
-            hbox.pack(cellpack.pad(emblem))
+            stack.pack(cellpack.align_left(emblem))
 
-        hbox.pack_space(2, expand=True)
+        main_hbox.pack_space(2, expand=True)
 
-        if self.data.is_external or self.data.is_single or self.data.downloaded:
-            hbox.pack(self.pack_video_buttons(layout))
+        if self.data.is_external or self.data.downloaded:
+            main_hbox.pack(self.pack_video_buttons(layout))
 
-        return cellpack.align_bottom(cellpack.pad(hbox, top=5, bottom=6))
+        stack.pack(main_hbox)
+
+        return cellpack.align_bottom(cellpack.pad(stack, top=5, bottom=6))
 
     def pack_video_buttons(self, layout):
         hbox = cellpack.HBox(spacing=5)
         layout.set_font(0.85)
+        if self.data.is_container_item:
+            hotspot = self._make_button(layout, self.SHOW_CONTENTS_TEXT,
+                    'show_contents')
+            hbox.pack(cellpack.align_middle(hotspot))
         if self.data.expiration_date:
             hotspot = self._make_button(layout, self.KEEP_TEXT, 'keep')
             hbox.pack(cellpack.align_middle(hotspot))
@@ -978,6 +1024,10 @@ class PlaylistItemRenderer(ItemRenderer):
     def pack_video_buttons(self, layout):
         hbox = cellpack.HBox(spacing=5)
         layout.set_font(0.85)
+        if self.data.is_container_item:
+            hotspot = self._make_button(layout, self.SHOW_CONTENTS_TEXT,
+                    'show_contents')
+            hbox.pack(cellpack.align_middle(hotspot))
         if self.data.expiration_date:
             hotspot = self._make_button(layout, _('Keep'), 'keep')
             hbox.pack(cellpack.align_middle(hotspot))
@@ -1096,15 +1146,18 @@ class StatusRenderer(ListViewRenderer):
 
     def _setup_layout(self):
         if self.info.downloaded:
-            if not self.info.video_watched:
-                self.text = _('Unplayed')
-                self.color = UNPLAYED_COLOR
-            elif self.info.expiration_date:
-                self.text = displaytext.expiration_date_short(
-                        self.info.expiration_date)
-                self.color = EXPIRING_TEXT_COLOR
-            else:
+            if not self.info.is_playable:
                 self.text = ''
+            else:
+                if not self.info.video_watched:
+                    self.text = _('Unplayed')
+                    self.color = UNPLAYED_COLOR
+                elif self.info.expiration_date:
+                    self.text = displaytext.expiration_date_short(
+                        self.info.expiration_date)
+                    self.color = EXPIRING_TEXT_COLOR
+                else:
+                    self.text = ''
         elif (self.info.download_info and
                 self.info.download_info.rate == 0):
             if self.info.download_info.state == 'paused':
@@ -1190,9 +1243,9 @@ class StateCircleRenderer(widgetset.CustomCellRenderer):
     def render(self, context, layout, selected, hotspot, hover):
         if self.info.state == 'downloading':
             icon = self.downloading_icon
-        elif self.info.downloaded and not self.info.video_watched:
+        elif self.info.downloaded and self.info.is_playable and not self.info.video_watched:
             icon = self.unwatched_icon
-        elif not self.info.item_viewed and not self.info.expiration_date:
+        elif not self.info.item_viewed and not self.info.expiration_date and not self.info.is_external:
             icon = self.new_icon
         else:
             return
