@@ -29,18 +29,19 @@
 from miro.download_utils import parseURL
 from miro import dialogs
 from miro import eventloop
+from miro import models
 
 def formatAuthString(auth):
-    return "%s %s" % (auth.getAuthScheme(), auth.getAuthToken())
+    return "%s %s" % (auth.get_auth_scheme(), auth.get_auth_token())
 
-def findHTTPAuth(callback, host, path):
+def find_http_auth(callback, host, path):
     """Find an HTTPAuthPassword object stored in the database.  Callback will
     be called with a string to use for the Authorization header or None if we
     can't find anything in the DB.
     """
     from miro import downloader
 
-    auth = downloader.findHTTPAuth(host, path)
+    auth = downloader.find_http_auth(host, path)
     if auth is not None:
         auth = formatAuthString(auth)
     eventloop.addIdle(callback, "http auth callback", args=(auth,))
@@ -53,9 +54,8 @@ def askForHTTPAuth(callback, url, realm, authScheme):
 
     scheme, host, port, path = parseURL(url)
     def handleLoginResponse(dialog):
-        from miro import downloader
         if dialog.choice == dialogs.BUTTON_OK:
-            auth = downloader.HTTPAuthPassword(dialog.username,
+            auth = models.HTTPAuthPassword(dialog.username,
                     dialog.password, host, realm, path, authScheme)
             callback(formatAuthString(auth))
         else:
