@@ -62,22 +62,21 @@ import time
 import urllib
 from miro import signals
 from miro import trapcall
+from miro.plat.resources import get_osname
 
 PIPELINING_ENABLED = True
 SOCKET_READ_TIMEOUT = 60
 SOCKET_INITIAL_READ_TIMEOUT = 30
 SOCKET_CONNECT_TIMEOUT = 15
 
-def userAgent():
-    osname = '%s %s %s' % (platform.system(), platform.release(),
-            platform.machine())
+def user_agent():
     return "%s/%s (%s; %s)" % (config.get(prefs.SHORT_APP_NAME),
             config.get(prefs.APP_VERSION),
             config.get(prefs.PROJECT_URL),
-            osname)
+            get_osname())
 
 class NetworkError(Exception):
-    """Base class for all errors that will be passed to errbacks from getURL
+    """Base class for all errors that will be passed to errbacks from get_url
     and friends.  NetworkErrors can be display in 2 ways:
 
     getFriendlyDescription() -- short, newbie friendly description 
@@ -618,7 +617,7 @@ class ProxiedAsyncSSLStream(AsyncSSLStream):
             eventloop.callInThread(onSSLOpen, handleSSLError, lambda: openProxyConnection(self),
                                    "ProxiedAsyncSSL openProxyConnection()")
         def openProxyConnection(self):
-            headers = {'User-Agent': userAgent(), "Host": host}
+            headers = {'User-Agent': user_agent(), "Host": host}
             if config.get(prefs.HTTP_PROXY_AUTHORIZATION_ACTIVE):
                 username = config.get(prefs.HTTP_PROXY_AUTHORIZATION_USERNAME)
                 password = config.get(prefs.HTTP_PROXY_AUTHORIZATION_PASSWORD)
@@ -1027,28 +1026,24 @@ class HTTPConnection(ConnectionHandler):
             self.reason = reason
 
     def handleHeaderLine(self, line):
-        if self.unparsedHeaderLine == '':
+        # "unfold" the continuation of a header
+        if len(line) > 0 and line[0] in (' ', '\t'):
+            self.unparsedHeaderLine += line
+        else:
+            # this line is not a continuation, so parse the last line
+            if self.unparsedHeaderLine != '':
+                if ':' in self.unparsedHeaderLine:
+                    self.parseHeader(self.unparsedHeaderLine)
+                else:
+                    msg = "HTTP header line without colon: %s" % \
+                            self.unparsedHeaderLine
+                    self.handleError(BadHeaderLine(msg))
             if line == '':
                 if self.status != 100:
                     self.startBody()
                 else:
                     self.changeState('response-status')
-            elif ':' in line:
-                self.parseHeader(line)
-            else:
-                self.unparsedHeaderLine = line
-        else:
-            # our last line may have been a continued header, or it may be
-            # garbage, 
-            if len(line) > 0 and line[0] in (' ', '\t'):
-                self.unparsedHeaderLine += line.lstrip()
-                if ':' in self.unparsedHeaderLine:
-                    self.parseHeader(self.unparsedHeaderLine)
-                    self.unparsedHeaderLine = ''
-            else:
-                msg = "line: %s, next line: %s" % (self.unparsedHeaderLine, 
-                        line)
-                self.handleError(BadHeaderLine(msg))
+            self.unparsedHeaderLine = line
 
     def parseHeader(self, line):
         header, value = line.split(":", 1)
@@ -1069,7 +1064,7 @@ class HTTPConnection(ConnectionHandler):
             trap_call(self, self.headerCallback, self.makeResponse())
         if self.state == 'closed':
             return # maybe the header callback cancelled this request
-        if ((100 <= self.status <= 199) or self.status in (204, 304) or
+        if ((100 <= self.status <= 199) or self.status in (204, 301, 303, 304, 307) or
                 self.method == 'HEAD' or self.contentLength == 0):
             self.finishRequest()
         else:
@@ -1483,7 +1478,7 @@ class HTTPClient(object):
         self.authAttempts = 0
         self.updateURLOk = True
         self.originalURL = self.updatedURL = self.redirectedURL = url
-        self.userAgent = userAgent()
+        self.user_agent = user_agent()
         self.connection = None
         self.cancelled = False
         self.initHeaders()
@@ -1540,7 +1535,7 @@ class HTTPClient(object):
             self.headers["If-None-Match"] = self.etag
         if not self.modified is None:
             self.headers["If-Modified-Since"] = self.modified
-        self.headers['User-Agent'] = self.userAgent
+        self.headers['User-Agent'] = self.user_agent
         self.setCookieHeader()
 
     def setCookieHeader(self):
@@ -1574,9 +1569,9 @@ class HTTPClient(object):
                     self.headers["Authorization"] = authHeader
                 self.reallyStartRequest()
             try:
-                httpauth.findHTTPAuth(callback, host.decode('ascii', 'replace'), path.decode('ascii', 'replace'))
+                httpauth.find_http_auth(callback, host.decode('ascii', 'replace'), path.decode('ascii', 'replace'))
             except UnicodeDecodeError:
-                httpauth.findHTTPAuth(callback, host.decode('ascii', 'ignore'), path.decode('ascii', 'ignore'))
+                httpauth.find_http_auth(callback, host.decode('ascii', 'ignore'), path.decode('ascii', 'ignore'))
         else:
             self.reallyStartRequest()
 
