@@ -24,6 +24,7 @@ void bind_peer_info();
 void bind_ip_filter();
 void bind_magnet_uri();
 void bind_converters();
+void bind_create_torrent();
 
 BOOST_PYTHON_MODULE(libtorrent)
 {
@@ -52,4 +53,5 @@ BOOST_PYTHON_MODULE(libtorrent)
     bind_ip_filter();
     bind_magnet_uri();
     bind_converters();
+    bind_create_torrent();
 }
