@@ -51,6 +51,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/io.hpp"
 #include "libtorrent/version.hpp"
+#include "libtorrent/escape_string.hpp"
 
 using boost::ref;
 using boost::lexical_cast;
@@ -58,7 +59,6 @@ using libtorrent::dht::node_impl;
 using libtorrent::dht::node_id;
 using libtorrent::dht::packet_t;
 using libtorrent::dht::msg;
-using libtorrent::dht::packet_iterator;
 namespace messages = libtorrent::dht::messages;
 using namespace libtorrent::detail;
 
@@ -67,9 +67,6 @@ enum
 	key_refresh = 5 // generate a new write token key every 5 minutes
 };
 
-using asio::ip::udp;
-typedef asio::ip::address_v4 address;
-
 namespace
 {
 	const int tick_period = 1; // minutes
@@ -86,29 +83,16 @@ namespace
 		}
 	};
 	
-	boost::optional<node_id> read_id(libtorrent::entry const& d)
-	{
-		using namespace libtorrent;
-		using libtorrent::dht::node_id;
-
-		if (d.type() != entry::dictionary_t) return boost::optional<node_id>();
-		entry const* nid = d.find_key("node-id");
-		if (!nid
-			|| nid->type() != entry::string_t
-			|| nid->string().length() != 40)
-			return boost::optional<node_id>();
-		return boost::optional<node_id>(
-			boost::lexical_cast<node_id>(nid->string()));
-	}
-
 	template <class EndpointType>
 	void read_endpoint_list(libtorrent::entry const* n, std::vector<EndpointType>& epl)				
 	{
 		using namespace libtorrent;
+		if (n->type() != entry::list_t) return;
 		entry::list_type const& contacts = n->list();
 		for (entry::list_type::const_iterator i = contacts.begin()
 			, end(contacts.end()); i != end; ++i)
 		{
+			if (i->type() != entry::string_t) return;
 			std::string const& p = i->string();
 			if (p.size() < 6) continue;
 			std::string::const_iterator in = p.begin();
@@ -143,28 +127,33 @@ namespace libtorrent { namespace dht
 	TORRENT_DEFINE_LOG(dht_tracker)
 #endif
 
+	boost::optional<node_id> extract_node_id(entry const* e)
+	{
+		if (e == 0 || e->type() != entry::dictionary_t) return boost::optional<node_id>();
+		entry const* nid = e->find_key("node-id");
+		if (nid == 0 || nid->type() != entry::string_t || nid->string().length() != 20)
+			return boost::optional<node_id>();
+		return boost::optional<node_id>(node_id(nid->string().c_str()));
+	}
+
 	// class that puts the networking and the kademlia node in a single
 	// unit and connecting them together.
-	dht_tracker::dht_tracker(asio::io_service& ios, dht_settings const& settings
-		, asio::ip::address listen_interface, entry const& bootstrap)
-		: m_strand(ios)
-		, m_socket(ios, udp::endpoint(listen_interface, settings.service_port))
-		, m_dht(bind(&dht_tracker::send_packet, this, _1), settings
-			, read_id(bootstrap))
-		, m_buffer(0)
+	dht_tracker::dht_tracker(udp_socket& sock, dht_settings const& settings
+		, entry const* state)
+		: m_dht(bind(&dht_tracker::send_packet, this, _1), settings, extract_node_id(state))
+		, m_sock(sock)
 		, m_last_new_key(time_now() - minutes(key_refresh))
-		, m_timer(ios)
-		, m_connection_timer(ios)
-		, m_refresh_timer(ios)
+		, m_timer(sock.get_io_service())
+		, m_connection_timer(sock.get_io_service())
+		, m_refresh_timer(sock.get_io_service())
 		, m_settings(settings)
 		, m_refresh_bucket(160)
-		, m_host_resolver(ios)
+		, m_abort(false)
+		, m_host_resolver(sock.get_io_service())
 		, m_refs(0)
 	{
 		using boost::bind;
 
-		m_in_buf[0].resize(1000);
-		m_in_buf[1].resize(1000);
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 		m_counter = 0;
 		std::fill_n(m_replies_bytes_sent, 5, 0);
@@ -191,6 +180,10 @@ namespace libtorrent { namespace dht
 //		dht_tracker_log.enable(false);
 
 #endif
+	}
+
+	void dht_tracker::start(entry const& bootstrap)
+	{
 		std::vector<udp::endpoint> initial_nodes;
 
 		if (bootstrap.type() == entry::dictionary_t)
@@ -202,28 +195,29 @@ namespace libtorrent { namespace dht
 			} catch (std::exception&) {}
 		}
 
-		m_socket.async_receive_from(asio::buffer(&m_in_buf[m_buffer][0]
-			, m_in_buf[m_buffer].size()), m_remote_endpoint[m_buffer]
-			, m_strand.wrap(bind(&dht_tracker::on_receive, self(), _1, _2)));
-		m_timer.expires_from_now(seconds(1));
-		m_timer.async_wait(m_strand.wrap(bind(&dht_tracker::tick, self(), _1)));
+		error_code ec;
+		m_timer.expires_from_now(seconds(1), ec);
+		m_timer.async_wait(bind(&dht_tracker::tick, self(), _1));
 
-		m_connection_timer.expires_from_now(seconds(10));
-		m_connection_timer.async_wait(m_strand.wrap(
-			bind(&dht_tracker::connection_timeout, self(), _1)));
+		m_connection_timer.expires_from_now(seconds(10), ec);
+		m_connection_timer.async_wait(
+			bind(&dht_tracker::connection_timeout, self(), _1));
 
-		m_refresh_timer.expires_from_now(seconds(5));
-		m_refresh_timer.async_wait(m_strand.wrap(bind(&dht_tracker::refresh_timeout, self(), _1)));
+		m_refresh_timer.expires_from_now(seconds(5), ec);
+		m_refresh_timer.async_wait(bind(&dht_tracker::refresh_timeout, self(), _1));
 
 		m_dht.bootstrap(initial_nodes, bind(&dht_tracker::on_bootstrap, self()));
 	}
 
 	void dht_tracker::stop()
 	{
-		m_timer.cancel();
-		m_connection_timer.cancel();
-		m_refresh_timer.cancel();
-		m_socket.close();
+		mutex_t::scoped_lock l(m_mutex);
+		m_abort = true;
+		error_code ec;
+		m_timer.cancel(ec);
+		m_connection_timer.cancel(ec);
+		m_refresh_timer.cancel(ec);
+		m_host_resolver.cancel();
 	}
 
 	void dht_tracker::dht_status(session_status& s)
@@ -233,54 +227,49 @@ namespace libtorrent { namespace dht
 		s.dht_global_nodes = m_dht.num_global_nodes();
 	}
 
-	void dht_tracker::connection_timeout(asio::error_code const& e)
+	void dht_tracker::connection_timeout(error_code const& e)
 		try
 	{
-		if (e) return;
-		if (!m_socket.is_open()) return;
+		mutex_t::scoped_lock l(m_mutex);
+		if (e || m_abort) return;
+
 		time_duration d = m_dht.connection_timeout();
 		m_connection_timer.expires_from_now(d);
-		m_connection_timer.async_wait(m_strand.wrap(bind(&dht_tracker::connection_timeout, self(), _1)));
+		m_connection_timer.async_wait(bind(&dht_tracker::connection_timeout, self(), _1));
 	}
 	catch (std::exception& exc)
 	{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		std::cerr << "exception-type: " << typeid(exc).name() << std::endl;
 		std::cerr << "what: " << exc.what() << std::endl;
 		TORRENT_ASSERT(false);
 #endif
 	};
 
-	void dht_tracker::refresh_timeout(asio::error_code const& e)
+	void dht_tracker::refresh_timeout(error_code const& e)
 		try
 	{
-		if (e) return;
-		if (!m_socket.is_open()) return;
+		mutex_t::scoped_lock l(m_mutex);
+		if (e || m_abort) return;
+
 		time_duration d = m_dht.refresh_timeout();
 		m_refresh_timer.expires_from_now(d);
-		m_refresh_timer.async_wait(m_strand.wrap(
-			bind(&dht_tracker::refresh_timeout, self(), _1)));
+		m_refresh_timer.async_wait(
+			bind(&dht_tracker::refresh_timeout, self(), _1));
 	}
 	catch (std::exception&)
 	{
 		TORRENT_ASSERT(false);
 	};
 
-	void dht_tracker::rebind(asio::ip::address listen_interface, int listen_port)
-	{
-		m_socket.close();
-		udp::endpoint ep(listen_interface, listen_port);
-		m_socket.open(ep.protocol());
-		m_socket.bind(ep);
-	}
-
-	void dht_tracker::tick(asio::error_code const& e)
+	void dht_tracker::tick(error_code const& e)
 		try
 	{
-		if (e) return;
-		if (!m_socket.is_open()) return;
+		mutex_t::scoped_lock l(m_mutex);
+		if (e || m_abort) return;
+
 		m_timer.expires_from_now(minutes(tick_period));
-		m_timer.async_wait(m_strand.wrap(bind(&dht_tracker::tick, self(), _1)));
+		m_timer.async_wait(bind(&dht_tracker::tick, self(), _1));
 
 		ptime now = time_now();
 		if (now - m_last_new_key > minutes(key_refresh))
@@ -309,26 +298,26 @@ namespace libtorrent { namespace dht
 		int peers = 0;
 		std::for_each(m_dht.begin_data(), m_dht.end_data(), count_peers(peers));
 
-		std::ofstream pc("libtorrent_logs/dht_stats.log", std::ios_base::app);
+		std::ofstream pc("libtorrent_logs/dht_stats.log", first ? std::ios_base::trunc : std::ios_base::app);
 		if (first)
 		{
 			first = false;
 			pc << "\n\n *****   starting log at " << time_now_string() << "   *****\n\n"
 				<< "minute:active nodes:passive nodes"
-				":ping replies sent:ping queries recvd:ping"
-				":ping replies sent:ping queries recvd:ping"
+				":ping replies sent:ping queries recvd"
+				":ping replies bytes sent:ping queries bytes recvd"
+				":find_node replies sent:find_node queries recv"
 				":find_node replies bytes sent:find_node queries bytes recv"
-				":find_node replies bytes sent:find_node queries bytes recv"
-				":get_peers replies sent:get_peers queries recvd:get_peers"
+				":get_peers replies sent:get_peers queries recvd"
 				":get_peers replies bytes sent:get_peers queries bytes recv"
-				":announce_peer replies sent:announce_peer queries recvd:announce_peer"
+				":announce_peer replies sent:announce_peer queries recvd"
 				":announce_peer replies bytes sent:announce_peer queries bytes recv"
-				":error replies sent:error queries recvd:error"
+				":error replies sent:error queries recvd"
 				":error replies bytes sent:error queries bytes recv"
 				":num torrents:num peers:announces per min"
 				":failed announces per min:total msgs per min"
 				":ut msgs per min:lt msgs per min:mp msgs per min"
-				":gr msgs per min:bytes in per sec:bytes out per sec"
+				":gr msgs per min:mo msgs per min:bytes in per sec:bytes out per sec"
 				":queries out bytes per sec\n\n";
 		}
 
@@ -385,21 +374,63 @@ namespace libtorrent { namespace dht
 		m_dht.announce(ih, listen_port, f);
 	}
 
+
+	void dht_tracker::on_unreachable(udp::endpoint const& ep)
+	{
+		m_dht.unreachable(ep);
+	}
+
 	// translate bittorrent kademlia message into the generice kademlia message
 	// used by the library
-	void dht_tracker::on_receive(asio::error_code const& error, size_t bytes_transferred)
+	void dht_tracker::on_receive(udp::endpoint const& ep, char const* buf, int bytes_transferred)
 		try
 	{
-		if (error == asio::error::operation_aborted) return;
-		if (!m_socket.is_open()) return;
-	
-		int current_buffer = m_buffer;
-		m_buffer = (m_buffer + 1) & 1;
-		m_socket.async_receive_from(asio::buffer(&m_in_buf[m_buffer][0]
-			, m_in_buf[m_buffer].size()), m_remote_endpoint[m_buffer]
-			, m_strand.wrap(bind(&dht_tracker::on_receive, self(), _1, _2)));
+		node_ban_entry* match = 0;
+		node_ban_entry* min = m_ban_nodes;
+		ptime now = time_now();
+		for (node_ban_entry* i = m_ban_nodes; i < m_ban_nodes + num_ban_nodes; ++i)
+		{
+			if (i->src == ep)
+			{
+				match = i;
+				break;
+			}
+			if (i->count < min->count) min = i;
+		}
 
-		if (error) return;
+		if (match)
+		{
+			++match->count;
+			if (match->count >= 20)
+			{
+				if (now < match->limit)
+				{
+#ifdef TORRENT_DHT_VERBOSE_LOGGING
+					if (match->count == 20)
+					{
+						TORRENT_LOG(dht_tracker) << time_now_string() << " BANNING PEER [ ip: "
+							<< ep << " time: " << total_milliseconds((now - match->limit) + seconds(5)) / 1000.f
+							<< " count: " << match->count << " ]";
+					}
+#endif
+					// we've received 20 messages in less than 5 seconds from
+					// this node. Ignore it until it's silent for 5 minutes
+					match->limit = now + minutes(5);
+					return;
+				}
+
+				// we got 50 messages from this peer, but it was in
+				// more than 5 seconds. Reset the counter and the timer
+				match->count = 0;
+				match->limit = now + seconds(5);
+			}
+		}
+		else
+		{
+			min->count = 1;
+			min->limit = now + seconds(5);
+			min->src = ep;
+		}
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 		++m_total_message_input;
@@ -413,20 +444,30 @@ namespace libtorrent { namespace dht
 			
 			TORRENT_ASSERT(bytes_transferred > 0);
 
-			entry e = bdecode(m_in_buf[current_buffer].begin()
-				, m_in_buf[current_buffer].end());
+			entry e = bdecode(buf, buf + bytes_transferred);
+			if (e.type() == entry::undefined_t)
+			{
+#ifdef TORRENT_DHT_VERBOSE_LOGGING
+				std::string msg(buf, buf + bytes_transferred);
+				TORRENT_LOG(dht_tracker) << "invalid incoming packet\n";
+#endif
+				return;
+			}
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-			TORRENT_LOG(dht_tracker) << time_now_string() << " RECEIVED ["
-				<< m_remote_endpoint[current_buffer] << "]:";
+			std::stringstream log_line;
+			log_line << time_now_string() << " RECEIVED ["
+				" ip: " << ep;
 #endif
 
 			libtorrent::dht::msg m;
 			m.message_id = 0;
-			m.addr = m_remote_endpoint[current_buffer];
+			m.addr = ep;
 			m.transaction_id = e["t"].string();
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
+			log_line << " t: " << to_hex(m.transaction_id);
+
 			try
 			{
 				entry const* ver = e.find_key("v");
@@ -436,36 +477,36 @@ namespace libtorrent { namespace dht
 				if (client.size() > 1 && std::equal(client.begin(), client.begin() + 2, "UT"))
 				{
 					++m_ut_message_input;
-					TORRENT_LOG(dht_tracker) << "   client: uTorrent";
+					log_line << " c: uTorrent";
 				}
 				else if (client.size() > 1 && std::equal(client.begin(), client.begin() + 2, "LT"))
 				{
 					++m_lt_message_input;
-					TORRENT_LOG(dht_tracker) << "   client: libtorrent";
+					log_line << " c: libtorrent";
 				}
 				else if (client.size() > 1 && std::equal(client.begin(), client.begin() + 2, "MP"))
 				{
 					++m_mp_message_input;
-					TORRENT_LOG(dht_tracker) << "   client: MooPolice";
+					log_line << " c: MooPolice";
 				}
 				else if (client.size() > 1 && std::equal(client.begin(), client.begin() + 2, "GR"))
 				{
 					++m_gr_message_input;
-					TORRENT_LOG(dht_tracker) << "   client: GetRight";
+					log_line << " c: GetRight";
 				}
 				else if (client.size() > 1 && std::equal(client.begin(), client.begin() + 2, "MO"))
 				{
 					++m_mo_message_input;
-					TORRENT_LOG(dht_tracker) << "   client: Mono Torrent";
+					log_line << " c: Mono Torrent";
 				}
 				else
 				{
-					TORRENT_LOG(dht_tracker) << "   client: " << client;
+					log_line << " c: " << client;
 				}
 			}
 			catch (std::exception&)
 			{
-				TORRENT_LOG(dht_tracker) << "   client: generic";
+				log_line << " c: generic";
 			};
 #endif
 
@@ -474,8 +515,7 @@ namespace libtorrent { namespace dht
 			if (msg_type == "r")
 			{
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-				TORRENT_LOG(dht_tracker) << "   reply: transaction: "
-					<< m.transaction_id;
+				log_line << " r: " << messages::ids[m.message_id];
 #endif
 
 				m.reply = true;
@@ -503,7 +543,7 @@ namespace libtorrent { namespace dht
 						read_endpoint_list<tcp::endpoint>(n, m.peers);
 					}
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   peers: " << m.peers.size();
+					log_line << " p: " << m.peers.size();
 #endif
 				}
 
@@ -523,7 +563,7 @@ namespace libtorrent { namespace dht
 							id, read_v4_endpoint<udp::endpoint>(i)));
 					}
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   nodes: " << m.nodes.size();
+					log_line << " n: " << m.nodes.size();
 #endif
 				}
 
@@ -548,7 +588,7 @@ namespace libtorrent { namespace dht
 								id, read_v6_endpoint<udp::endpoint>(in)));
 					}
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   nodes2 + nodes: " << m.nodes.size();
+					log_line << " n2: " << m.nodes.size();
 #endif
 				}
 
@@ -565,7 +605,7 @@ namespace libtorrent { namespace dht
 
 				std::string request_kind(e["q"].string());
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-				TORRENT_LOG(dht_tracker) << "   query: " << request_kind;
+				log_line << " q: " << request_kind;
 #endif
 
 				if (request_kind == "ping")
@@ -578,8 +618,7 @@ namespace libtorrent { namespace dht
 					if (target.size() != 20) throw std::runtime_error("invalid size of target id");
 					std::copy(target.begin(), target.end(), m.info_hash.begin());
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   target: "
-						<< boost::lexical_cast<std::string>(m.info_hash);
+					log_line << " target: " << boost::lexical_cast<std::string>(m.info_hash);
 #endif
 
 					m.message_id = libtorrent::dht::messages::find_node;
@@ -591,8 +630,7 @@ namespace libtorrent { namespace dht
 					std::copy(info_hash.begin(), info_hash.end(), m.info_hash.begin());
 					m.message_id = libtorrent::dht::messages::get_peers;
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   info_hash: "
-						<< boost::lexical_cast<std::string>(m.info_hash);
+					log_line << " ih: " << boost::lexical_cast<std::string>(m.info_hash);
 #endif
 				}
 				else if (request_kind == "announce_peer")
@@ -608,9 +646,8 @@ namespace libtorrent { namespace dht
 					m.write_token = a["token"];
 					m.message_id = libtorrent::dht::messages::announce_peer;
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   info_hash: "
-						<< boost::lexical_cast<std::string>(m.info_hash);
-					TORRENT_LOG(dht_tracker) << "   port: " << m.port;
+					log_line << " ih: " << boost::lexical_cast<std::string>(m.info_hash);
+					log_line << " p: " << m.port;
 
 					if (!m_dht.verify_token(m))
 						++m_failed_announces;
@@ -632,8 +669,8 @@ namespace libtorrent { namespace dht
 				m.error_msg = list.back().string();
 				m.error_code = list.front().integer();
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-				TORRENT_LOG(dht_tracker) << "   incoming error: " << m.error_code << " "
-					<< m.error_msg;
+				log_line << " incoming error: " << m.error_code
+					<< " " << m.error_msg;
 #endif
 				throw std::runtime_error("DHT error message");
 			}
@@ -652,7 +689,7 @@ namespace libtorrent { namespace dht
 				++m_queries_received[m.message_id];
 				m_queries_bytes_received[m.message_id] += int(bytes_transferred);
 			}
-			TORRENT_LOG(dht_tracker) << e;
+			TORRENT_LOG(dht_tracker) << log_line.str() << " ]";
 #endif
 			TORRENT_ASSERT(m.message_id != messages::error);
 			m_dht.incoming(m);
@@ -660,9 +697,7 @@ namespace libtorrent { namespace dht
 		catch (std::exception& e)
 		{
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-			int current_buffer = (m_buffer + 1) & 1;
-			std::string msg(m_in_buf[current_buffer].begin()
-				, m_in_buf[current_buffer].begin() + bytes_transferred);
+			std::string msg(buf, buf + bytes_transferred);
 			TORRENT_LOG(dht_tracker) << "invalid incoming packet: "
 				<< e.what() << "\n" << msg << "\n";
 #endif
@@ -712,15 +747,14 @@ namespace libtorrent { namespace dht
 	void dht_tracker::add_node(std::pair<std::string, int> const& node)
 	{
 		udp::resolver::query q(node.first, lexical_cast<std::string>(node.second));
-		m_host_resolver.async_resolve(q, m_strand.wrap(
-			bind(&dht_tracker::on_name_lookup, self(), _1, _2)));
+		m_host_resolver.async_resolve(q,
+			bind(&dht_tracker::on_name_lookup, self(), _1, _2));
 	}
 
-	void dht_tracker::on_name_lookup(asio::error_code const& e
+	void dht_tracker::on_name_lookup(error_code const& e
 		, udp::resolver::iterator host) try
 	{
 		if (e || host == udp::resolver::iterator()) return;
-		if (!m_socket.is_open()) return;
 		add_node(host->endpoint());
 	}
 	catch (std::exception&)
@@ -731,15 +765,14 @@ namespace libtorrent { namespace dht
 	void dht_tracker::add_router_node(std::pair<std::string, int> const& node)
 	{
 		udp::resolver::query q(node.first, lexical_cast<std::string>(node.second));
-		m_host_resolver.async_resolve(q, m_strand.wrap(
-			bind(&dht_tracker::on_router_name_lookup, self(), _1, _2)));
+		m_host_resolver.async_resolve(q,
+			bind(&dht_tracker::on_router_name_lookup, self(), _1, _2));
 	}
 
-	void dht_tracker::on_router_name_lookup(asio::error_code const& e
+	void dht_tracker::on_router_name_lookup(error_code const& e
 		, udp::resolver::iterator host) try
 	{
 		if (e || host == udp::resolver::iterator()) return;
-		if (!m_socket.is_open()) return;
 		m_dht.add_router_node(host->endpoint());
 	}
 	catch (std::exception&)
@@ -755,7 +788,6 @@ namespace libtorrent { namespace dht
 		void write_nodes_entry(entry& r, libtorrent::dht::msg const& m)
 		{
 			bool ipv6_nodes = false;
-			r["nodes"] = entry(entry::string_t);
 			entry& n = r["nodes"];
 			std::back_insert_iterator<std::string> out(n.string());
 			for (msg::nodes_t::const_iterator i = m.nodes.begin()
@@ -772,7 +804,6 @@ namespace libtorrent { namespace dht
 
 			if (ipv6_nodes)
 			{
-				r["nodes2"] = entry(entry::list_t);
 				entry& p = r["nodes2"];
 				std::string endpoint;
 				for (msg::nodes_t::const_iterator i = m.nodes.begin()
@@ -788,9 +819,6 @@ namespace libtorrent { namespace dht
 					p.list().push_back(entry(endpoint));
 				}
 			}
-#ifdef TORRENT_DHT_VERBOSE_LOGGING
-			TORRENT_LOG(dht_tracker) << "   nodes: " << m.nodes.size();
-#endif
 		}
 	}
 
@@ -807,9 +835,10 @@ namespace libtorrent { namespace dht
 		e["v"] = std::string(version_str, version_str + 4);
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-		TORRENT_LOG(dht_tracker) << time_now_string()
-			<< " SENDING [" << m.addr << "]:";
-		TORRENT_LOG(dht_tracker) << "   transaction: " << m.transaction_id;
+		std::stringstream log_line;
+		log_line << time_now_string()
+			<< " SENDING [ ip: " << m.addr
+			<< " t: " << to_hex(m.transaction_id);
 #endif
 
 		if (m.message_id == messages::error)
@@ -822,8 +851,8 @@ namespace libtorrent { namespace dht
 			error_list.list().push_back(entry(m.error_msg));
 			e["e"] = error_list;
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-			TORRENT_LOG(dht_tracker) << time_now_string()
-				<< "   outgoing error: " << m.error_code << " " << m.error_msg;
+			log_line << " err: " << m.error_code
+				<< " msg: " << m.error_msg;
 #endif
 		}
 		else if (m.reply)
@@ -834,8 +863,7 @@ namespace libtorrent { namespace dht
 			r["id"] = std::string(m.id.begin(), m.id.end());
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-			TORRENT_LOG(dht_tracker) << time_now_string()
-				<< "   reply: " << messages::ids[m.message_id];
+			log_line << " r: " << messages::ids[m.message_id];
 #endif
 
 			if (m.write_token.type() != entry::undefined_t)
@@ -871,7 +899,7 @@ namespace libtorrent { namespace dht
 							p.list().push_back(entry(endpoint));
 						}
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-						TORRENT_LOG(dht_tracker) << "   peers: " << m.peers.size();
+						log_line << " p: " << m.peers.size();
 #endif
 					}
 					break;
@@ -895,8 +923,7 @@ namespace libtorrent { namespace dht
 			e["q"] = messages::ids[m.message_id];
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-			TORRENT_LOG(dht_tracker) << "   query: "
-				<< messages::ids[m.message_id];
+			log_line << " q: " << messages::ids[m.message_id];
 #endif
 
 			switch (m.message_id)
@@ -905,8 +932,7 @@ namespace libtorrent { namespace dht
 				{
 					a["target"] = std::string(m.info_hash.begin(), m.info_hash.end());
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   target: "
-						<< boost::lexical_cast<std::string>(m.info_hash);
+					log_line << " target: " << boost::lexical_cast<std::string>(m.info_hash);
 #endif
 					break;
 				}
@@ -914,19 +940,17 @@ namespace libtorrent { namespace dht
 				{
 					a["info_hash"] = std::string(m.info_hash.begin(), m.info_hash.end());
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   info_hash: "
-						<< boost::lexical_cast<std::string>(m.info_hash);
+					log_line << " ih: " << boost::lexical_cast<std::string>(m.info_hash);
 #endif
 					break;	
 				}
 				case messages::announce_peer:
-					a["port"] = m_settings.service_port;
+					a["port"] = m.port;
 					a["info_hash"] = std::string(m.info_hash.begin(), m.info_hash.end());
 					a["token"] = m.write_token;
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-					TORRENT_LOG(dht_tracker) << "   port: "
-						<< m_settings.service_port
-						<< "   info_hash: " << boost::lexical_cast<std::string>(m.info_hash);
+					log_line << " p: " << m.port
+						<< " ih: " << boost::lexical_cast<std::string>(m.info_hash);
 #endif
 					break;
 				default: break;
@@ -936,10 +960,8 @@ namespace libtorrent { namespace dht
 
 		m_send_buf.clear();
 		bencode(std::back_inserter(m_send_buf), e);
-		asio::error_code ec;
-		m_socket.send_to(asio::buffer(&m_send_buf[0]
-			, (int)m_send_buf.size()), m.addr, 0, ec);
-		if (ec) return;
+		error_code ec;
+		m_sock.send(m.addr, &m_send_buf[0], (int)m_send_buf.size(), ec);
 
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
 		m_total_out_bytes += m_send_buf.size();
@@ -953,7 +975,7 @@ namespace libtorrent { namespace dht
 		{
 			m_queries_out_bytes += m_send_buf.size();
 		}
-		TORRENT_LOG(dht_tracker) << e;
+		TORRENT_LOG(dht_tracker) << log_line.str() << " ]";
 #endif
 
 		if (!m.piggy_backed_ping) return;
