@@ -26,13 +26,35 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
+"""``miro.prefs`` -- Defines :class:`Pref` and lists global preferences.
+"""
+
 import os
 from miro import util
 
 class Pref:
-    def __init__(self, **kwds):
-        self.__dict__.update(kwds)
+    """Class for defining preferences.  Preferences are defined using
+    keywords:
 
+    * **key** -- the name of the key--must be unique among all preferences
+    * **default** -- the default value to use
+    * **platformSpecific** -- whether or not this is platform specific;
+      this should usually be False
+    * **possible_values** -- a list of possible values for this preference;
+      if the saved value gets corrupted for some reason and therefore does not
+      correspond to any of the possible values, the failsafe is picked instead.
+    * **failsafe_value** -- value to be used in case of corruption (see above).
+
+    Pref example::
+
+        FOO = Pref(key="foo", default=None, platformSpecific=False)
+    """
+    def __init__(self, key, default, platformSpecific, possible_values=None, failsafe_value=None):
+        self.key = key
+        self.default = default
+        self.platformSpecific = platformSpecific
+        self.possible_values = possible_values
+        self.failsafe_value = failsafe_value
     def __eq__(self, other):
         return self.key == other.key
     def __ne__(self, other):
@@ -48,13 +70,14 @@ UPSTREAM_LIMIT_IN_KBS       = Pref( key='upstreamLimitInKBS',    default=12,
 UPSTREAM_TORRENT_LIMIT      = Pref( key='upstreamTorrentLimit',  default=10,    platformSpecific=False )
 LIMIT_DOWNSTREAM_BT         = Pref( key='limitDownstreamBT',     default=False, platformSpecific=False )
 DOWNSTREAM_BT_LIMIT_IN_KBS  = Pref( key='downstreamBTLimitInKBS', default=200,   platformSpecific=False )
-LIMIT_CONNECTIONS_BT = Pref( key='limitConnectionsBT',     default=False, platformSpecific=False )
-CONNECTION_LIMIT_BT_NUM  = Pref( key='connectionLimitBTNum', default=100,   platformSpecific=False )
+LIMIT_CONNECTIONS_BT        = Pref( key='limitConnectionsBT',     default=False, platformSpecific=False )
+CONNECTION_LIMIT_BT_NUM     = Pref( key='connectionLimitBTNum', default=100,   platformSpecific=False )
 PRESERVE_DISK_SPACE         = Pref( key='preserveDiskSpace',     default=True,  platformSpecific=False )
 PRESERVE_X_GB_FREE          = Pref( key='preserveXGBFree',       default=0.2,   platformSpecific=False )
-EXPIRE_AFTER_X_DAYS         = Pref( key='expireAfterXDays',      default=6,     platformSpecific=False )
-DOWNLOADS_TARGET            = Pref( key='DownloadsTarget',       default=8,     platformSpecific=False ) # max auto downloads
-MAX_MANUAL_DOWNLOADS        = Pref( key='MaxManualDownloads',    default=10,    platformSpecific=False )
+EXPIRE_AFTER_X_DAYS         = Pref( key='expireAfterXDays',      default=6,     platformSpecific=False,
+                                    possible_values=[1,3,6,10,30,-1], failsafe_value=-1 )
+DOWNLOADS_TARGET            = Pref( key='DownloadsTarget',       default=4,     platformSpecific=False ) # max auto downloads
+MAX_MANUAL_DOWNLOADS        = Pref( key='MaxManualDownloads',    default=5,    platformSpecific=False )
 VOLUME_LEVEL                = Pref( key='VolumeLevel',           default=1.0,   platformSpecific=False )
 BT_MIN_PORT                 = Pref( key='BitTorrentMinPort',     default=8500,  platformSpecific=False )
 BT_MAX_PORT                 = Pref( key='BitTorrentMaxPort',     default=8600,  platformSpecific=False )
@@ -83,32 +106,28 @@ LAST_SEARCH_ENGINE = \
 SEARCH_ORDERING = \
     Pref(key='SearchOrdering', default=None, platformSpecific=False)
 
+
 # These have a hardcoded default which can be overridden by setting an
 # environment variable.
 
-if 'DTV_CHANNELGUIDE_URL' in os.environ:
-    effectiveChannelGuide = util.unicodify(os.environ['DTV_CHANNELGUIDE_URL'])
-else:
-    effectiveChannelGuide = u'https://www.miroguide.com/'
+def get_from_environ(key, default):
+    if key in os.environ:
+        return util.unicodify(os.environ[key])
 
-if 'DTV_CHANNELGUIDE_FIRST_TIME_URL' in os.environ:
-    effectiveChannelGuideFirstTime = util.unicodify(os.environ['DTV_CHANNELGUIDE_FIRST_TIME_URL'])
-else:
-    effectiveChannelGuideFirstTime = u'https://www.miroguide.com/firsttime'
+    return default
 
-if 'DTV_SHARE_URL' in os.environ:
-    effectiveShare = util.unicodify(os.environ['DTV_SHARE_URL'])
-else:
-    effectiveShare = u'https://www.miroguide.com/share'
+default_guide = get_from_environ('DTV_CHANNELGUIDE_URL',
+                                 u'https://www.miroguide.com/')
+default_guide_first_time = get_from_environ('DTV_CHANNELGUIDE_FIRST_TIME_URL',
+                                            u'https://www.miroguide.com/firsttime')
+default_share = get_from_environ('DTV_SHARE_URL',
+                                 u'https://www.miroguide.com/share')
+default_autoupdate = get_from_environ('DTV_AUTOUPDATE_URL',
+                                      u'http://www.participatoryculture.org/democracy-appcast.xml')
 
-if 'DTV_AUTOUPDATE_URL' in os.environ:
-    effectiveAutoupdate = util.unicodify(os.environ['DTV_AUTOUPDATE_URL'])
-else:
-    effectiveAutoupdate = u'http://www.participatoryculture.org/democracy-appcast.xml'
-
-CHANNEL_GUIDE_URL = Pref(key='ChannelGuideURL', default=effectiveChannelGuide,
+CHANNEL_GUIDE_URL = Pref(key='ChannelGuideURL', default=default_guide,
                          platformSpecific=False)
-CHANNEL_GUIDE_FIRST_TIME_URL = Pref(key='ChannelGuideFirstTimeURL', default=effectiveChannelGuideFirstTime,
+CHANNEL_GUIDE_FIRST_TIME_URL = Pref(key='ChannelGuideFirstTimeURL', default=default_guide_first_time,
                          platformSpecific=False)
 CHANNEL_GUIDE_ALLOWED_URLS = Pref(key='ChannelGuideAllowedURLs',
                               default='http://pagead2.googlesyndication.com/ '
@@ -117,9 +136,9 @@ CHANNEL_GUIDE_ALLOWED_URLS = Pref(key='ChannelGuideAllowedURLs',
                               platformSpecific=False)
 ADDITIONAL_CHANNEL_GUIDES = Pref(key='additionalChannelGuides', default='',
                                   platformSpecific=False)
-SHARE_URL         = Pref(key='ShareURL',        default=effectiveShare,
+SHARE_URL         = Pref(key='ShareURL',        default=default_share,
                          platformSpecific=False)
-AUTOUPDATE_URL    = Pref(key='AutoupdateURL',   default=effectiveAutoupdate,
+AUTOUPDATE_URL    = Pref(key='AutoupdateURL',   default=default_autoupdate,
                          platformSpecific=False)
 DONATE_URL        = Pref(key='DonateURL', default=u"http://www.getmiro.com/donate/",
                          platformSpecific=False)
@@ -194,6 +213,8 @@ PUBLISHER = \
     Pref(key='publisher',         default=None, platformSpecific=False)
 COPYRIGHT = \
     Pref(key='copyright',         default=None, platformSpecific=False)
+TRADEMARK = \
+    Pref(key='trademark',         default=None, platformSpecific=False)
 APP_VERSION = \
     Pref(key='appVersion',        default=None, platformSpecific=False)
 APP_REVISION = \
