@@ -26,19 +26,22 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-"""The schema module is responsible for defining what data in the database
-gets stored on disk.  
+"""``miro.schema`` -- The schema module is responsible for defining
+what data in the database gets stored on disk.
 
 The goals of this modules are:
 
-* Clearly defining which data from DDBObjects gets stored and which doesn't.
+* Clearly defining which data from DDBObjects gets stored and which
+  doesn't.
 * Validating that all data we write can be read back in
 * Making upgrades of the database schema as easy as possible
 
 Module-level variables:
-    objectSchemas -- Schemas to use with the current database.
-    VERSION -- Current schema version.  If you change the schema you must bump
-    this number and add a function in the databaseupgrade module.
+
+* ``object_schemas`` -- Schemas to use with the current database.
+* ``VERSION`` -- Current schema version.  If you change the schema you
+  must bump this number and add a function in the databaseupgrade
+  module.
 
 Go to the bottom of this file for the current database schema.
 """
@@ -62,10 +65,11 @@ class SchemaItem(object):
     """SchemaItem represents a single attribute that gets stored on disk.
 
     SchemaItem is an abstract class.  Subclasses of SchemaItem such as
-    SchemaAttr, SchemaObject, SchemaList are used in actual object schemas.
+    SchemaAttr, SchemaList are used in actual object schemas.
 
     Member variables:
-        noneOk -- specifies if None is a valid value for this attribute
+
+    * ``noneOk`` -- specifies if None is a valid value for this attribute
     """
 
     def __init__(self, noneOk=False):
@@ -105,31 +109,37 @@ class SchemaSimpleItem(SchemaItem):
     """Base class for SchemaItems for simple python types."""
 
 class SchemaBool(SchemaSimpleItem):
+    """Defines the SchemaBool type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, bool)
 
 class SchemaFloat(SchemaSimpleItem):
+    """Defines the SchemaFloat type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, float)
 
 class SchemaString(SchemaSimpleItem):
+    """Defines the SchemaString type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, unicode)
 
 class SchemaBinary(SchemaSimpleItem):
+    """Defines the SchemaBinary type for blobs."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, str)
 
 class SchemaFilename(SchemaSimpleItem):
+    """Defines the SchemaFilename type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, FilenameType)
 
 class SchemaURL(SchemaSimpleItem):
+    """Defines the SchemaURL type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, unicode)
@@ -140,31 +150,88 @@ class SchemaURL(SchemaSimpleItem):
                 ValidationError(u"URL (%s) is not ASCII" % data)
 
 class SchemaInt(SchemaSimpleItem):
+    """Defines the SchemaInt type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateTypes(data, [int, long])
 
 class SchemaDateTime(SchemaSimpleItem):
+    """Defines the SchemaDateTime type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, datetime.datetime)
 
 class SchemaTimeDelta(SchemaSimpleItem):
+    """Defines the SchemaTimeDelta type."""
     def validate(self, data):
         super(SchemaSimpleItem, self).validate(data)
         self.validateType(data, datetime.timedelta)
 
-class SchemaList(SchemaItem):
+class SchemaReprContainer(SchemaItem):
+    """SchemaItem saved using repr() to save nested lists, dicts and
+    tuples that store simple types.  The look is similar to JSON, but
+    supports a couple different things, for example unicode and str
+    values are distinct.  The types that we support are bool, int,
+    long, float, unicode, None and datetime.  Dictionary keys can also
+    be byte strings (AKA str types)
+    """
+
+    VALID_TYPES = [bool, int, long, float, unicode, NoneType,
+            datetime.datetime, time.struct_time]
+
+    VALID_KEY_TYPES = VALID_TYPES + [str]
+
+    def validate(self, data):
+        if data is None:
+            # let the super class handle the noneOkay attribute
+            super(SchemaReprContainer, self).validate(data)
+            return
+        memory = set()
+        to_validate = [data]
+
+        while to_validate:
+            obj = to_validate.pop()
+            if id(obj) in memory:
+                continue
+            else:
+                memory.add(id(obj))
+
+            if isinstance(obj, list) or isinstance(obj, tuple):
+                for item in obj:
+                    to_validate.append(item)
+            elif isinstance(obj, dict):
+                for key, value in obj.items():
+                    self.validateTypes(key, self.VALID_KEY_TYPES)
+                    to_validate.append(value)
+            else:
+                self.validateTypes(obj, self.VALID_TYPES)
+
+class SchemaList(SchemaReprContainer):
+    """Special case of SchemaReprContainer that stores a simple list
+
+    All values in the list must have the same type.
+    """
     def __init__(self, childSchema, noneOk=False):
         super(SchemaList, self).__init__(noneOk)
         self.childSchema = childSchema
 
     def validate(self, data):
-        super(SchemaList, self).validate(data)
+        if data is None:
+            super(SchemaList, self).validate(data)
+            return
         self.validateType(data, list)
+        for i, value in enumerate(data):
+            try:
+                self.childSchema.validate(value)
+            except ValidationError:
+                raise ValidationError("%r (index: %s) has the wrong type" %
+                        (value, i))
 
-class SchemaDict(SchemaItem):
-    type = dict
+class SchemaDict(SchemaReprContainer):
+    """Special case of SchemaReprContainer that stores a simple dict
+
+    All keys and values must have the same type.
+    """
 
     def __init__(self, keySchema, valueSchema, noneOk=False):
         super(SchemaDict, self).__init__(noneOk)
@@ -172,66 +239,49 @@ class SchemaDict(SchemaItem):
         self.valueSchema = valueSchema
 
     def validate(self, data):
-        super(SchemaDict, self).validate(data)
+        if data is None:
+            super(SchemaDict, self).validate(data)
+            return
         self.validateType(data, dict)
-
-class SchemaSimpleContainer(SchemaSimpleItem):
-    """Allows nested dicts, lists and tuples, however the only thing they can
-    store are simple objects.  This currently includes bools, ints, longs,
-    floats, strings, unicode, None, datetime and struct_time objects.
-    """
-
-    def validate(self, data):
-        super(SchemaSimpleContainer, self).validate(data)
-        self.validateTypes(data, (dict, list, tuple))
-        self.memory = set()
-        toValidate = LinkedList()
-        while data:
-            if id(data) in self.memory:
-                return
-            else:
-                self.memory.add(id(data))
-    
-            if isinstance(data, list) or isinstance(data, tuple):
-                for item in data:
-                    toValidate.append(item)
-            elif isinstance(data, dict):
-                for key, value in data.items():
-                    self.validateTypes(key, [bool, int, long, float, unicode,
-                        str, NoneType, datetime.datetime, time.struct_time])
-                    toValidate.append(value)
-            else:
-                self.validateTypes(data, [bool, int, long, float, unicode,
-                        NoneType, datetime.datetime, time.struct_time])
+        for key, value in data.items():
             try:
-                data = toValidate.pop()
-            except (KeyboardInterrupt, SystemExit):
-                raise
-            except:
-                data = None
+                self.keySchema.validate(key)
+            except ValidationError:
+                raise ValidationError("key %r has the wrong type" % key)
+            try:
+                self.valueSchema.validate(value)
+            except ValidationError:
+                raise ValidationError("value %r (key: %r) has the wrong type"
+                        % (value, key))
 
-class SchemaStatusContainer(SchemaSimpleContainer):
-    """Allows nested dicts, lists and tuples, however the only thing they can
-    store are simple objects.  This currently includes bools, ints, longs,
-    floats, strings, unicode, None, datetime and struct_time objects.
+class SchemaStatusContainer(SchemaReprContainer):
+    """Version of SchemaReprContainer that stores the status dict for
+    RemoteDownloaders.  It allows some values to be byte strings
+    rather than unicode objects.
     """
 
+
     def validate(self, data):
-        if FilenameType == unicode:
-            binaryFields = ['metainfo', 'fastResumeData']
-        else:
-            binaryFields = ['channelName', 'shortFilename', 'filename', 'metainfo', 'fastResumeData']
+        binary_fields = self._binary_fields()
         self.validateType(data, dict)
         for key, value in data.items():
             self.validateTypes(key, [bool, int, long, float, unicode,
                     str, NoneType, datetime.datetime, time.struct_time])
-            if key not in binaryFields:
+            if key not in binary_fields:
                 self.validateTypes(value, [bool, int, long, float, unicode,
                         NoneType, datetime.datetime, time.struct_time])
             else:
                 self.validateType(value, str)
 
+    def _binary_fields(self):
+        if FilenameType == unicode:
+            return ('metainfo', 'fastResumeData')
+        else:
+            return ('channelName', 'shortFilename', 'filename', 'metainfo',
+                    'fastResumeData')
+
 class SchemaObject(SchemaItem):
+    """SchemaObject type."""
     def __init__(self, klass, noneOk=False):
         super(SchemaObject, self).__init__(noneOk)
         self.klass = klass
@@ -246,47 +296,78 @@ class ObjectSchema(object):
 
     Member variables:
 
-    klass -- the python class that this schema is for
-    classString -- a human readable string that represents objectClass
-    fields -- list of  (name, SchemaItem) pairs.  One item for each attribute
-        that should be stored to disk.
+    * ``klass`` -- the python class that this schema is for
+    * ``table_name`` -- SQL table name to store the class in
+    * ``fields`` -- list of (name, SchemaItem) pairs.  One item for
+      each attribute that should be stored to disk.
+    """
+
+    @classmethod
+    def ddb_object_classes(cls):
+        return (cls.klass,)
+
+    @classmethod
+    def get_ddb_class(cls, restored_data):
+        return cls.klass
+
+    indexes = ()
+
+class MultiClassObjectSchema(ObjectSchema):
+    """ObjectSchema where rows will be restored to different python classes.
+
+    Instead of the klass attribute, MultiClassObjectSchema should
+    define 2 class methods: ddb_object_classes() returns the list of
+    all classes that this schema works with and get_ddb_class() which
+    inputs a dictionary containing db data and return which class we
+    should use to restore that row.
     """
-    pass
 
 from miro.database import DDBObject
 from miro.downloader import RemoteDownloader, HTTPAuthPassword
 from miro.feed import Feed, FeedImpl, RSSFeedImpl, RSSMultiFeedImpl, ScraperFeedImpl
 from miro.feed import SearchFeedImpl, DirectoryWatchFeedImpl, DirectoryFeedImpl, SearchDownloadsFeedImpl
 from miro.feed import ManualFeedImpl, SingleFeedImpl
-from miro.folder import ChannelFolder, PlaylistFolder
+from miro.folder import ChannelFolder, PlaylistFolder, PlaylistFolderItemMap
 from miro.guide import ChannelGuide
 from miro.item import Item, FileItem
 from miro.iconcache import IconCache
-from miro.playlist import SavedPlaylist
+from miro.playlist import SavedPlaylist, PlaylistItemMap
 from miro.tabs import TabOrder
 from miro.theme import ThemeHistory
 
 class DDBObjectSchema(ObjectSchema):
     klass = DDBObject
-    classString = 'ddb-object'
+    table_name = 'ddb_object'
     fields = [
         ('id', SchemaInt())
     ]
 
-class IconCacheSchema (ObjectSchema):
+class IconCacheSchema (DDBObjectSchema):
     klass = IconCache
-    classString = 'icon-cache'
-    fields = [
+    table_name = 'icon_cache'
+    fields = DDBObjectSchema.fields + [
         ('etag', SchemaString(noneOk=True)),
         ('modified', SchemaString(noneOk=True)),
         ('filename', SchemaFilename(noneOk=True)),
         ('url', SchemaURL(noneOk=True)),
         ]
 
-class ItemSchema(DDBObjectSchema):
-    klass = Item
-    classString = 'item'
+class ItemSchema(MultiClassObjectSchema):
+    table_name = 'item'
+
+    @classmethod
+    def ddb_object_classes(cls):
+        return (Item, FileItem)
+
+    @classmethod
+    def get_ddb_class(cls, restored_data):
+        if restored_data['is_file_item']:
+            return FileItem
+        else:
+            return Item
+
     fields = DDBObjectSchema.fields + [
+        ('is_file_item', SchemaBool()),
         ('feed_id', SchemaInt(noneOk=True)),
         ('downloader_id', SchemaInt(noneOk=True)),
         ('parent_id', SchemaInt(noneOk=True)),
@@ -295,46 +376,64 @@ class ItemSchema(DDBObjectSchema):
         ('pendingManualDL', SchemaBool()),
         ('pendingReason', SchemaString()),
         ('title', SchemaString()),
-        ('entry', SchemaSimpleContainer()),
         ('expired', SchemaBool()),
         ('keep', SchemaBool()),
         ('creationTime', SchemaDateTime()),
         ('linkNumber', SchemaInt(noneOk=True)),
-        ('iconCache', SchemaObject(IconCache, noneOk=True)),
+        ('icon_cache_id', SchemaInt(noneOk=True)),
         ('downloadedTime', SchemaDateTime(noneOk=True)),
         ('watchedTime', SchemaDateTime(noneOk=True)),
         ('isContainerItem', SchemaBool(noneOk=True)),
-        ('videoFilename', SchemaFilename()),
-        ('isVideo', SchemaBool()),
         ('releaseDateObj', SchemaDateTime()),
         ('eligibleForAutoDownload', SchemaBool()),
         ('duration', SchemaInt(noneOk=True)),
         ('screenshot', SchemaFilename(noneOk=True)),
-        ('resized_screenshots', SchemaDict(SchemaString(), SchemaFilename())),
         ('resumeTime', SchemaInt()),
         ('channelTitle', SchemaString(noneOk=True)),
-    ]
-
-class FileItemSchema(ItemSchema):
-    klass = FileItem
-    classString = 'file-item'
-    fields = ItemSchema.fields + [
-        ('filename', SchemaFilename()),
-        ('deleted', SchemaBool()),
+        ('license', SchemaString(noneOk=True)),
+        ('rss_id', SchemaString(noneOk=True)),
+        ('thumbnail_url', SchemaURL(noneOk=True)),
+        ('entry_title', SchemaString(noneOk=True)),
+        ('raw_descrption', SchemaString(noneOk=False)),
+        ('link', SchemaURL(noneOk=False)),
+        ('payment_link', SchemaURL(noneOk=False)),
+        ('comments_link', SchemaURL(noneOk=False)),
+        ('url', SchemaURL(noneOk=False)),
+        ('enclosure_size', SchemaInt(noneOk=True)),
+        ('enclosure_type', SchemaString(noneOk=True)),
+        ('enclosure_format', SchemaString(noneOk=True)),
+        ('feedparser_output', SchemaReprContainer()),
+        ('was_downloaded', SchemaBool()),
+        ('filename', SchemaFilename(noneOk=True)),
+        ('deleted', SchemaBool(noneOk=True)),
         ('shortFilename', SchemaFilename(noneOk=True)),
         ('offsetPath', SchemaFilename(noneOk=True)),
+        ('file_type', SchemaString(noneOk=True)),
     ]
 
+    indexes = (
+            ('item_feed', ('feed_id',)),
+            ('item_feed_visible', ('feed_id', 'deleted')),
+            ('item_parent', ('parent_id',)),
+            ('item_downloader', ('downloader_id',)),
+            ('item_feed_downloader', ('feed_id', 'downloader_id',)),
+            ('item_file_type', ('file_type',)),
+    )
+
+    @staticmethod
+    def handle_malformed_feedparser_output(row):
+        return {}
+
 class FeedSchema(DDBObjectSchema):
     klass = Feed
-    classString = 'feed'
+    table_name = 'feed'
     fields = DDBObjectSchema.fields + [
         ('origURL', SchemaURL()),
         ('baseTitle', SchemaString(noneOk=True)),
         ('errorState', SchemaBool()),
         ('loading', SchemaBool()),
-        ('actualFeed', SchemaObject(FeedImpl)),
-        ('iconCache', SchemaObject(IconCache, noneOk=True)),
+        ('feed_impl_id', SchemaInt()),
+        ('icon_cache_id', SchemaInt(noneOk=True)),
         ('folder_id', SchemaInt(noneOk=True)),
         ('searchTerm', SchemaString(noneOk=True)),
         ('userTitle', SchemaString(noneOk=True)),
@@ -346,18 +445,18 @@ class FeedSchema(DDBObjectSchema):
         ('expire', SchemaString()),
         ('expireTime', SchemaTimeDelta(noneOk=True)),
         ('section', SchemaString()),
+        ('visible', SchemaBool()),
+        ('last_viewed', SchemaDateTime()),
     ]
 
-class FeedImplSchema(ObjectSchema):
+class FeedImplSchema(DDBObjectSchema):
     klass = FeedImpl
-    classString = 'field-impl'
-    fields = [
+    table_name = 'feed_impl'
+    fields = DDBObjectSchema.fields + [
         ('url', SchemaURL()),
-        ('ufeed', SchemaObject(Feed)),
+        ('ufeed_id', SchemaInt()),
         ('title', SchemaString(noneOk=True)),
         ('created', SchemaDateTime()),
-        ('visible', SchemaBool()),
-        ('lastViewed', SchemaDateTime()),
         ('thumbURL', SchemaURL(noneOk=True)),
         ('updateFreq', SchemaInt()),
         ('initialUpdate', SchemaBool()),
@@ -365,7 +464,7 @@ class FeedImplSchema(ObjectSchema):
 
 class RSSFeedImplSchema(FeedImplSchema):
     klass = RSSFeedImpl
-    classString = 'rss-feed-impl'
+    table_name = 'rss_feed_impl'
     fields = FeedImplSchema.fields + [
         ('initialHTML', SchemaBinary(noneOk=True)),
         ('etag', SchemaString(noneOk=True)),
@@ -374,25 +473,37 @@ class RSSFeedImplSchema(FeedImplSchema):
 
 class RSSMultiFeedImplSchema(FeedImplSchema):
     klass = RSSMultiFeedImpl
-    classString = 'rss-multi-feed-impl'
+    table_name = 'rss_multi_feed_impl'
     fields = FeedImplSchema.fields + [
         ('etag', SchemaDict(SchemaString(),SchemaString(noneOk=True))),
         ('modified', SchemaDict(SchemaString(),SchemaString(noneOk=True))),
         ('query', SchemaString(noneOk=True)),
     ]
 
+    @staticmethod
+    def handle_malformed_etag(row):
+        return {}
+
+    @staticmethod
+    def handle_malformed_modified(row):
+        return {}
+
 class ScraperFeedImplSchema(FeedImplSchema):
     klass = ScraperFeedImpl
-    classString = 'scraper-feed-impl'
+    table_name = 'scraper_feed_impl'
     fields = FeedImplSchema.fields + [
         ('initialHTML', SchemaBinary(noneOk=True)),
         ('initialCharset', SchemaString(noneOk=True)),
-        ('linkHistory', SchemaSimpleContainer()),
+        ('linkHistory', SchemaReprContainer()),
     ]
 
+    @staticmethod
+    def handle_malformed_linkHistory(row):
+        return {}
+
 class SearchFeedImplSchema(RSSMultiFeedImplSchema):
     klass = SearchFeedImpl
-    classString = 'search-feed-impl'
+    table_name = 'search_feed_impl'
     fields = RSSMultiFeedImplSchema.fields + [
         ('searching', SchemaBool()),
         ('lastEngine', SchemaString()),
@@ -401,7 +512,7 @@ class SearchFeedImplSchema(RSSMultiFeedImplSchema):
 
 class DirectoryWatchFeedImplSchema(FeedImplSchema):
     klass = DirectoryWatchFeedImpl
-    classString = 'directory-watch-feed-impl'
+    table_name = 'directory_watch_feed_impl'
     fields = FeedImplSchema.fields + [
         ('firstUpdate', SchemaBool()),
         ('dir', SchemaFilename(noneOk=True)),
@@ -409,27 +520,27 @@ class DirectoryWatchFeedImplSchema(FeedImplSchema):
 
 class DirectoryFeedImplSchema(FeedImplSchema):
     klass = DirectoryFeedImpl
-    classString = 'directory-feed-impl'
+    table_name = 'directory_feed_impl'
     # DirectoryFeedImpl doesn't have any addition fields over FeedImpl
 
 class SearchDownloadsFeedImplSchema(FeedImplSchema):
     klass = SearchDownloadsFeedImpl
-    classString = 'search-downloads-feed-impl'
+    table_name = 'search_downloads_feed_impl'
     # SearchDownloadsFeedImpl doesn't have any addition fields over FeedImpl
 
 class ManualFeedImplSchema(FeedImplSchema):
     klass = ManualFeedImpl
-    classString = 'manual-feed-impl'
+    table_name = 'manual_feed_impl'
     # no addition fields over FeedImplSchema
 
 class SingleFeedImplSchema(FeedImplSchema):
     klass = SingleFeedImpl
-    classString = 'single-feed-impl'
+    table_name = 'single_feed_impl'
     # no addition fields over FeedImplSchema
 
 class RemoteDownloaderSchema(DDBObjectSchema):
     klass = RemoteDownloader
-    classString = 'remote-downloader'
+    table_name = 'remote_downloader'
     fields = DDBObjectSchema.fields + [
         ('url', SchemaURL()),
         ('origURL', SchemaURL()),
@@ -438,11 +549,22 @@ class RemoteDownloaderSchema(DDBObjectSchema):
         ('channelName', SchemaFilename(noneOk=True)),
         ('status', SchemaStatusContainer()),
         ('manualUpload', SchemaBool()),
+        ('state', SchemaString()),
+        ('main_item_id', SchemaInt(noneOk=True)),
+        ('child_deleted', SchemaBool()),
     ]
 
+    indexes = (
+            ('downloader_state', ('state',)),
+    )
+
+    @staticmethod
+    def handle_malformed_status(row):
+        return {}
+
 class HTTPAuthPasswordSchema(DDBObjectSchema):
     klass = HTTPAuthPassword
-    classString = 'http-auth-password'
+    table_name = 'http_auth_password'
     fields = DDBObjectSchema.fields + [
         ('username', SchemaString()),
         ('password', SchemaString()),
@@ -454,7 +576,7 @@ class HTTPAuthPasswordSchema(DDBObjectSchema):
 
 class ChannelFolderSchema(DDBObjectSchema):
     klass = ChannelFolder
-    classString = 'channel-folder'
+    table_name = 'channel_folder'
     fields = DDBObjectSchema.fields + [
         ('expanded', SchemaBool()),
         ('title', SchemaString()),
@@ -463,33 +585,54 @@ class ChannelFolderSchema(DDBObjectSchema):
 
 class PlaylistFolderSchema(DDBObjectSchema):
     klass = PlaylistFolder
-    classString = 'playlist-folder'
+    table_name = 'playlist_folder'
     fields = DDBObjectSchema.fields + [
         ('expanded', SchemaBool()),
         ('title', SchemaString()),
-        ('item_ids', SchemaList(SchemaInt())),
     ]
 
 class PlaylistSchema(DDBObjectSchema):
     klass = SavedPlaylist
-    classString = 'playlist'
+    table_name = 'playlist'
     fields = DDBObjectSchema.fields + [
         ('title', SchemaString()),
-        ('item_ids', SchemaList(SchemaInt())),
         ('folder_id', SchemaInt(noneOk=True)),
     ]
 
+class PlaylistItemMapSchema(DDBObjectSchema):
+    klass = PlaylistItemMap
+    table_name = 'playlist_item_map'
+    fields = DDBObjectSchema.fields + [
+        ('playlist_id', SchemaInt()),
+        ('item_id', SchemaInt()),
+        ('position', SchemaInt()),
+    ]
+
+class PlaylistFolderItemMapSchema(DDBObjectSchema):
+    klass = PlaylistFolderItemMap
+    table_name = 'playlist_folder_item_map'
+    fields = DDBObjectSchema.fields + [
+        ('playlist_id', SchemaInt()),
+        ('item_id', SchemaInt()),
+        ('position', SchemaInt()),
+        ('count', SchemaInt()),
+    ]
+
 class TabOrderSchema(DDBObjectSchema):
     klass = TabOrder
-    classString = 'taborder-order'
+    table_name = 'taborder_order'
     fields = DDBObjectSchema.fields + [
         ('type', SchemaString()),
         ('tab_ids', SchemaList(SchemaInt())),
     ]
 
+    @staticmethod
+    def handle_malformed_tab_ids(row):
+        return []
+
 class ChannelGuideSchema(DDBObjectSchema):
     klass = ChannelGuide
-    classString = 'channel-guide'
+    table_name = 'channel_guide'
     fields = DDBObjectSchema.fields + [
         ('url', SchemaURL(noneOk=True)),
         ('allowedURLs', SchemaList(SchemaURL())),
@@ -497,32 +640,45 @@ class ChannelGuideSchema(DDBObjectSchema):
         ('favicon', SchemaURL(noneOk=True)),
         ('title', SchemaString(noneOk=True)),
         ('userTitle', SchemaString(noneOk=True)),
-        ('iconCache', SchemaObject(IconCache, noneOk=True)),
+        ('icon_cache_id', SchemaInt(noneOk=True)),
         ('firstTime', SchemaBool()),
     ]
 
+    @staticmethod
+    def handle_malformed_allowedURLs(row):
+        return []
+
 class ThemeHistorySchema(DDBObjectSchema):
     klass = ThemeHistory
-    classString = 'theme-history'
+    table_name = 'theme_history'
     fields = DDBObjectSchema.fields + [
         ('lastTheme', SchemaString(noneOk=True)),
         ('pastThemes', SchemaList(SchemaString(noneOk=True), noneOk=False)),
     ]
 
+    @staticmethod
+    def handle_malformed_pastThemes(row):
+        return []
+
 class WidgetsFrontendStateSchema(DDBObjectSchema):
     klass = WidgetsFrontendState
-    classString = 'widgets-frontend-state'
+    table_name = 'widgets_frontend_state'
     fields = DDBObjectSchema.fields + [
         ('list_view_displays', SchemaList(SchemaBinary())),
     ]
 
-VERSION = 73
-objectSchemas = [
-    DDBObjectSchema, IconCacheSchema, ItemSchema, FileItemSchema, FeedSchema,
+    @staticmethod
+    def handle_malformed_list_view_displays(row):
+        return []
+
+VERSION = 104
+object_schemas = [
+    IconCacheSchema, ItemSchema, FeedSchema,
     FeedImplSchema, RSSFeedImplSchema, RSSMultiFeedImplSchema, ScraperFeedImplSchema,
     SearchFeedImplSchema, DirectoryFeedImplSchema, DirectoryWatchFeedImplSchema,
     SearchDownloadsFeedImplSchema, RemoteDownloaderSchema,
     HTTPAuthPasswordSchema, ChannelGuideSchema, ManualFeedImplSchema, SingleFeedImplSchema,
     PlaylistSchema, ChannelFolderSchema, PlaylistFolderSchema,
+    PlaylistItemMapSchema, PlaylistFolderItemMapSchema,
     TabOrderSchema, ThemeHistorySchema, WidgetsFrontendStateSchema
 ]
