@@ -39,7 +39,9 @@ from miro.gtcache import gettext as _
 import logging
 import os
 import traceback
+import threading
 import platform
+import time
 
 from miro.clock import clock
 from miro import app
@@ -52,22 +54,26 @@ from miro import databaseupgrade
 from miro import downloader
 from miro import eventloop
 from miro import fileutil
+from miro import guide
 from miro import iconcache
-from miro import indexes
 from miro import item
 from miro import feed
 from miro import messages
 from miro import messagehandler
+from miro import models
 from miro import moviedata
 from miro import prefs
 from miro.plat.utils import setup_logging
+from miro.plat import config as platformcfg
 from miro import tabs
 from miro import theme
 from miro import util
 from miro import searchengines
 from miro import storedatabase
-from miro import views
-from miro.singleclick import parse_command_line_args
+from miro.commandline import parse_command_line_args
+
+DEBUG_DB_MEM_USAGE = False
+mem_usage_test_event = threading.Event()
 
 class StartupError(Exception):
     def __init__(self, summary, description):
@@ -90,39 +96,49 @@ def startup_function(func):
             logging.warn("Unknown startup error: %s", traceback.format_exc())
             m = messages.StartupFailure(_("Unknown Error"),
                     _(
-                        "An unknown error prevented Miro from startup.  Please "
+                        "An unknown error prevented %(appname)s from startup.  Please "
                         "file a bug report at %(url)s.",
-                        {"url": config.get(prefs.BUG_REPORT_URL)}
+                        {"appname": config.get(prefs.SHORT_APP_NAME),
+                         "url": config.get(prefs.BUG_REPORT_URL)}
                     ))
             m.send_to_frontend()
     return wrapped
 
-_movies_directory_gone_handler = None
+def _movies_directory_gone_handler(callback):
+    """Default _movies_directory_gone_handler.  The frontend should
+    override this using the ``install_movies_directory_gone_handler``
+    function.
+    """
+    logging.error("Movies directory is gone -- no handler installed!")
+    eventloop.addUrgentCall(callback, "continuing startup")
 
 def install_movies_directory_gone_handler(callback):
     global _movies_directory_gone_handler
     _movies_directory_gone_handler = callback
 
-_first_time_handler = None
+def _first_time_handler(callback):
+    """Default _first_time_handler.  The frontend should override this
+    using the ``install_first_time_handler`` function.
+    """
+    logging.error("First time -- no handler installed.")
+    eventloop.addUrgentCallback(callback, "continuing startup")
 
 def install_first_time_handler(callback):
     global _first_time_handler
     _first_time_handler = callback
 
 def setup_global_feed(url, *args, **kwargs):
-    feedView = views.feeds.filterWithIndex(indexes.feedsByURL, url)
-    try:
-        if feedView.len() == 0:
-            logging.info("Spawning global feed %s", url)
-            feed.Feed(url, *args, **kwargs)
-        elif feedView.len() > 1:
-            allFeeds = [f for f in feedView]
-            for extra in allFeeds[1:]:
-                extra.remove()
-            raise StartupError("Database inconsistent",
-                    "Too many db objects for %s" % url)
-    finally:
-        feedView.unlink()
+    view = feed.Feed.make_view('origURL=?', (url,))
+    view_count = view.count()
+    if view_count == 0:
+        logging.info("Spawning global feed %s", url)
+        feed.Feed(url, *args, **kwargs)
+    elif view_count > 1:
+        allFeeds = [f for f in view]
+        for extra in allFeeds[1:]:
+            extra.remove()
+        raise StartupError("Database inconsistent",
+                "Too many db objects for %s" % url)
 
 def initialize(themeName):
     """Initialize Miro.  This sets up things like logging and the config
@@ -132,7 +148,6 @@ def initialize(themeName):
     setup_logging()
     # this is portable general
     util.setup_logging()
-    app.db = database.defaultDatabase
     app.controller = controller.Controller()
     config.load(themeName)
 
@@ -157,18 +172,20 @@ def startup():
     logging.info("Revision:   %s", config.get(prefs.APP_REVISION))
     logging.info("Builder:    %s", config.get(prefs.BUILD_MACHINE))
     logging.info("Build Time: %s", config.get(prefs.BUILD_TIME))
-    eventloop.connect('thread-started', lambda obj, thread: database.set_thread(thread))
+    eventloop.connect('thread-started', finish_startup)
     logging.info("Starting event loop thread")
     eventloop.startup()
-    eventloop.addIdle(finish_startup, "finish startup")
+    if DEBUG_DB_MEM_USAGE:
+        mem_usage_test_event.wait()
 
 @startup_function
-def finish_startup():
-    views.initialize()
+def finish_startup(obj, thread):
+    database.set_thread(thread)
     logging.info("Restoring database...")
+    start = time.time()
+    app.db = storedatabase.LiveStorage()
     try:
-        database.defaultDatabase.liveStorage = storedatabase.LiveStorage()
-
+        app.db.upgrade_database()
     except databaseupgrade.DatabaseTooNewError:
         summary = _("Database too new")
         description = _(
@@ -178,11 +195,30 @@ def finish_startup():
             {"appname": config.get(prefs.SHORT_APP_NAME)},
         )
         raise StartupError(summary, description)
-    database.defaultDatabase.recomputeFilters()
+    except storedatabase.UpgradeDiskSpaceError:
+        summary = _("Insufficient disk space")
+        description = _(
+            "Not enough disk space to upgrade your database from the previous "
+            "version of %(appname)s.  Please delete some files and try again.",
+            {'appname': config.get(prefs.SHORT_APP_NAME)}
+        )
+        raise StartupError(summary, description)
+    database.update_last_id()
+    end = time.time()
+    logging.timing ("Database upgrade time: %.3f", end - start)
+
+    models.initialize()
+    if DEBUG_DB_MEM_USAGE:
+        util.db_mem_usage_test()
+        mem_usage_test_event.set()
 
     searchengines.create_engines()
     setup_global_feeds()
+    # call move_orphaned_items() ASAP after the manual feed is set up
+    item.move_orphaned_items()
+    logging.info("setup tabs")
     setup_tabs()
+    logging.info(" theme")
     setup_theme()
     install_message_handler()
 
@@ -192,13 +228,11 @@ def finish_startup():
 def check_firsttime():
     """Run the first time wizard if need be.
     """
+    callback = lambda: eventloop.addUrgentCall(check_movies_gone, "check movies gone")
     if is_first_time():
-        if _first_time_handler:
-            logging.info("First time -- calling handler.")
-            _first_time_handler(lambda: eventloop.addUrgentCall(check_movies_gone, "check movies gone"))
-            return
-        else:
-            logging.warn("First time -- no handler installed!")
+        logging.info("First time -- calling handler.")
+        _first_time_handler(callback)
+        return
 
     eventloop.addUrgentCall(check_movies_gone, "check movies gone")
 
@@ -206,29 +240,53 @@ def check_firsttime():
 def check_movies_gone():
     """Checks to see if the movies directory is gone.
     """
-    if is_movies_directory_gone():
-        if _movies_directory_gone_handler:
-            logging.info("Movies directory is gone -- calling handler.")
-            _movies_directory_gone_handler(lambda: eventloop.addUrgentCall(fix_movies_gone, "startup network stuff"))
-            return
-        else:
-            logging.warn("Movies directory is gone -- no handler installed!")
+    callback = lambda: eventloop.addUrgentCall(fix_movies_gone,
+                                               "fix movies gone")
 
-    eventloop.addUrgentCall(reconnect_downloaders, "reconnect downloaders")
+    if is_movies_directory_gone():
+        logging.info("Movies directory is gone -- calling handler.")
+        _movies_directory_gone_handler(callback)
+        return
+
+    movies_dir = fileutil.expand_filename(config.get(prefs.MOVIES_DIRECTORY))
+    # if the directory doesn't exist, create it.
+    if not os.path.exists(movies_dir):
+        try:
+            os.makedirs(movies_dir)
+        except OSError:
+            logging.info("Movies directory can't be created -- calling handler")
+            # FIXME - this isn't technically correct, but it's probably
+            # close enough that a user can fix the issue and Miro can
+            # run happily.
+            _movies_directory_gone_handler(callback)
+            return
+
+    # make sure the directory is writeable
+    try:
+        fn = os.path.join(movies_dir, "testfile")
+        f = open(fn, "w")
+        f.write("test")
+        f.close()
+        os.remove(fn)
+    except IOError:
+        _movies_directory_gone_handler(callback)
+        return
+    eventloop.addUrgentCall(finish_backend_startup, "reconnect downloaders")
 
 @startup_function
 def fix_movies_gone():
-    from miro.plat import config as platformcfg
     config.set(prefs.MOVIES_DIRECTORY, platformcfg.get(prefs.MOVIES_DIRECTORY))
-    eventloop.addUrgentCall(reconnect_downloaders, "reconnect downloaders")
+    eventloop.addUrgentCall(finish_backend_startup, "reconnect downloaders")
 
 @startup_function
-def reconnect_downloaders():
+def finish_backend_startup():
     """Last bit of startup required before we load the frontend.  """
     # Uncomment the next line to test startup error handling
     # raise StartupError("Test Error", "Startup Failed")
-    item.reconnect_downloaders()
-    downloader.initController()
+    reconnect_downloaders()
+    downloader.init_controller()
+    guide.download_guides()
+    feed.remove_orphaned_feed_impls()
     messages.StartupSuccess().send_to_frontend()
 
 @eventloop.idle_iterator
@@ -236,15 +294,13 @@ def on_frontend_started():
     """Perform startup actions that should happen after the frontend is
     already up and running.
     """
-
     logging.info("Starting auto downloader...")
     autodler.start_downloader()
     yield None
     feed.expire_items()
     yield None
-    starttime = clock()
-    logging.timing("Icon clear: %.3f", clock() - starttime)
     logging.info("Starting movie data updates")
+    item.update_incomplete_movie_data()
     yield None
     moviedata.movieDataUpdater.startThread()
     yield None
@@ -254,13 +310,13 @@ def on_frontend_started():
     yield None
     # Wait a bit before starting the downloader daemon.  It can cause a bunch
     # of disk/CPU load, so try to avoid it slowing other stuff down.
-    eventloop.addTimeout(5, downloader.startupDownloader,
+    eventloop.addTimeout(5, downloader.startup_downloader,
             "start downloader daemon")
     # ditto for feed updates
     eventloop.addTimeout(30, feed.start_updates, "start feed updates")
     # ditto for clearing stale icon cache files, except it's the very lowest
     # priority
-    eventloop.addTimeout(10, iconcache.clear_orphans, "clear orphans")
+    eventloop.addTimeout(10, clear_icon_cache_orphans, "clear orphans")
 
 def setup_global_feeds():
     setup_global_feed(u'dtv:manualFeed', initiallyAutoDownloadable=False)
@@ -270,16 +326,17 @@ def setup_global_feeds():
     setup_global_feed(u'dtv:directoryfeed')
 
 def setup_tabs():
-    def setup_tab_order(view, key):
-        try:
-            tabOrder = util.getSingletonDDBObject(view)
-        except LookupError:
-            logging.info("Creating %s tab order" % key)
-            tabs.TabOrder(key)
-    setup_tab_order(views.siteTabOrder, u'site')
-    setup_tab_order(views.channelTabOrder, u'channel')
-    setup_tab_order(views.audioChannelTabOrder, u'audio-channel')
-    setup_tab_order(views.playlistTabOrder, u'playlist')
+    def setup_tab_order(type):
+        current_tab_orders = list(tabs.TabOrder.view_for_type(type))
+        if len(current_tab_orders) == 0:
+            logging.info("Creating %s tab order" % type)
+            tab_order = tabs.TabOrder(type)
+        else:
+            current_tab_orders[0].restore_tab_list()
+    setup_tab_order(u'site')
+    setup_tab_order(u'channel')
+    setup_tab_order(u'audio-channel')
+    setup_tab_order(u'playlist')
 
 def is_first_time():
     """Checks to see if this is the first time that Miro has been run.
@@ -306,24 +363,26 @@ def is_movies_directory_gone():
         if os.path.exists(movies_dir):
             contents = os.listdir(movies_dir)
             if contents:
-                # There's something inside the directory consider it present (even
-                # if all our items are missing.
+                # there's something inside the directory consider it
+                # present (even if all our items are missing).
                 return False
 
     except OSError:
-        # We can't access the directory.  Seems like it's gone.
+        # we can't access the directory--treat it as if it's gone.
         logging.info("Can't access directory.")
         return True
 
-    # make sure that we have actually downloaded something into the movies
-    # directory.
-    movies_dir = config.get(prefs.MOVIES_DIRECTORY)
-    for downloader_ in views.remoteDownloads:
-        if (downloader_.isFinished()
-                and downloader_.get_filename().startswith(movies_dir)):
+    # at this point either there's no movies_dir or there is an empty
+    # movies_dir.  we check to see if we think something is downloaded.
+    for downloader_ in downloader.RemoteDownloader.make_view():
+        if ((downloader_.isFinished()
+             and downloader_.get_filename().startswith(movies_dir))):
+            # we think something is downloaded, so it seems like the
+            # movies directory is gone.
             logging.info("Directory there, but missing files.")
             return True
 
+    # we have no content, so everything's fine.
     return False
 
 def setup_theme():
@@ -335,7 +394,56 @@ def install_message_handler():
     messages.BackendMessage.install_handler(handler)
 
 def _get_theme_history():
-    if len(views.themeHistories) > 0:
-        return views.themeHistories[0]
+    current_themes = list(theme.ThemeHistory.make_view())
+    if len(current_themes) > 0:
+        return current_themes[0]
     else:
         return theme.ThemeHistory()
+
+@eventloop.idle_iterator
+def clear_icon_cache_orphans():
+    # delete icon_cache rows from the database with no associated
+    # item/feed/guide.
+    for ic in iconcache.IconCache.orphaned_view():
+        logging.warn("No object for IconCache: %s.  Discarding", ic)
+        ic.remove()
+    yield None
+
+    # delete files in the icon cache directory that don't belong to IconCache
+    # objects.
+
+    cachedir = fileutil.expand_filename(config.get(prefs.ICON_CACHE_DIRECTORY))
+    if not os.path.isdir(cachedir):
+        return
+
+    existingFiles = [os.path.normcase(os.path.join(cachedir, f))
+            for f in os.listdir(cachedir)]
+    yield None
+
+    knownIcons = iconcache.IconCache.all_filenames()
+    yield None
+
+    knownIcons = [ os.path.normcase(fileutil.expand_filename(path)) for path in
+            knownIcons]
+    yield None
+
+    for filename in existingFiles:
+        if (os.path.exists(filename)
+                and os.path.basename(filename)[0] != '.'
+                and os.path.basename(filename) != 'extracted'
+                and not filename in knownIcons):
+            try:
+                os.remove(filename)
+            except OSError:
+                pass
+        yield None
+
+def reconnect_downloaders():
+    for downloader_ in downloader.RemoteDownloader.orphaned_view():
+        logging.warn("removing orphaned downloader: %s", downloader_.url)
+        downloader_.remove()
+    manualItems = item.Item.feed_view(feed.Feed.get_manual_feed().getID())
+    for item_ in manualItems:
+        if item_.downloader is None and item_.__class__ == item.Item:
+            logging.warn("removing cancelled external torrent: %s", item_)
+            item_.remove()
