@@ -1,11 +1,18 @@
+import os
 import unittest
-from tempfile import mkstemp
+from tempfile import mkstemp, gettempdir
+from datetime import datetime
 from time import sleep
 
 from miro import config
+from miro import feedparser
 from miro import prefs
 from miro import dialogs
 from miro import database
+from miro import storedatabase
+from miro import feedparserutil
+from miro.plat import resources
+from miro.item import Item
 from miro.feed import validate_feed_url, normalize_feed_url, Feed
 
 from miro.test.framework import MiroTestCase, EventLoopTest
@@ -76,8 +83,7 @@ class FeedURLNormalizationTest(MiroTestCase):
 class FeedTestCase(EventLoopTest):
     def setUp(self):
         EventLoopTest.setUp(self)
-        self.everything = database.defaultDatabase
-        [handle, self.filename] = mkstemp(".xml")
+        self.filename = self.make_temp_path()
 
     def writefile(self, content):
         self.url = u'file://%s' % self.filename
@@ -154,19 +160,71 @@ class SimpleFeedTestCase(FeedTestCase):
         # thus it should not cause a dialog to pop up and ask the user if they
         # want to scrape.
         self.assertEqual(dialogs.delegate.calls, 0)
-        self.assertEqual(self.everything.len(), 2) 
-
         # the Feed, plus the 1 item that is a video
-        items = self.everything.filter(lambda x:x.__class__.__name__ == 'Item')
-        self.assertEqual(items.len(), 1)
+        items = list(Item.make_view())
+        self.assertEqual(len(items), 1)
 
         # make sure that re-updating doesn't re-create the items
         myFeed.update()
-        self.assertEqual(self.everything.len(), 2)
-        items = self.everything.filter(lambda x:x.__class__.__name__ == 'Item')
-        self.assertEqual(items.len(), 1)
+        items = list(Item.make_view())
+        self.assertEqual(len(items), 1)
         myFeed.remove()
 
+class MultiFeedExpireTest(FeedTestCase):
+    def write_files(self, subfeed_count, feed_item_count):
+        all_urls = []
+        self.filenames = []
+
+        content = self.make_feed_content(feed_item_count)
+        for i in xrange(subfeed_count):
+            filename = self.make_temp_path()
+            open(filename, 'wb').write(content)
+            all_urls.append(u"file://%s" % filename)
+            self.filenames.append(filename)
+
+        self.url = u'dtv:multi:' + ','.join(all_urls) + "," + 'testquery'
+
+    def rewrite_files(self, feed_item_count):
+        content = self.make_feed_content(feed_item_count)
+        for filename in self.filenames:
+            open(filename, 'wb').write(content)
+
+    def make_feed_content(self, entry_count):
+        # make a feed with a new item and parse it
+        items = []
+        counter = 0
+        for x in range(entry_count):
+            counter += 1
+            items.append("""\
+<item>
+ <title>Bumper Sticker</title>
+ <guid>guid-%s</guid>
+ <enclosure url="http://downhillbattle.org/key/gallery/%s.mpg" />
+ <description>I'm a musician and I support filesharing.</description>
+</item>
+""" % (counter, counter))
+        return """<?xml version="1.0"?>
+<rss version="2.0">
+   <channel>
+      <title>Downhill Battle Pics</title>
+      <link>http://downhillbattle.org/</link>
+      <description>Downhill Battle is a non-profit organization working to support participatory culture and build a fairer music industry.</description>
+      <pubDate>Wed, 16 Mar 2005 12:03:42 EST</pubDate>
+      %s
+   </channel>
+</rss>""" % '\n'.join(items)
+
+    def test_multi_feed_expire(self):
+        # test what happens when a RSSMultiFeed has feeds that reference the
+        # same item, and they are truncated at the same time (#11756)
+
+        self.write_files(5, 10) # 5 feeds containing 10 identical items
+        self.feed = self.makeFeed()
+        config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 4)
+        config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 5)
+        self.rewrite_files(1) # now only 5 items in each feed
+        self.updateFeed(self.feed)
+
 class EnclosureFeedTestCase(FeedTestCase):
     def setUp(self):
         FeedTestCase.setUp(self)
@@ -202,18 +260,12 @@ class EnclosureFeedTestCase(FeedTestCase):
 </rss>""")
     def testRun(self):
         myFeed = self.makeFeed()
-        self.assertEqual(self.everything.len(),5)
-        items = self.everything.filter(lambda x:x.__class__.__name__ == 'Item')
-        self.assertEqual(items.len(),4)
+        items = list(Item.make_view())
+        self.assertEqual(len(items),4)
         #Make sure that re-updating doesn't re-create the items
         myFeed.update()
-        self.assertEqual(self.everything.len(),5)
-        items = self.everything.filter(lambda x:x.__class__.__name__ == 'Item')
-        self.assertEqual(items.len(),4)
-        for item in items:
-            #Generate an exception if we didn't get one of the enclosures
-            item.entry["enclosures"][0]
-            self.assertRaises(IndexError,lambda :item.entry["enclosures"][1])
+        items = list(Item.make_view())
+        self.assertEqual(len(items),4)
         myFeed.remove()
 
 class OldItemExpireTest(FeedTestCase):
@@ -223,7 +275,6 @@ class OldItemExpireTest(FeedTestCase):
         self.counter = 0
         self.writeNewFeed()
         self.feed = self.makeFeed()
-        self.items = self.everything.filter(lambda x:x.__class__.__name__ == 'Item')
         config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 4)
         config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 20)
 
@@ -240,7 +291,6 @@ class OldItemExpireTest(FeedTestCase):
  <description>I'm a musician and I support filesharing.</description>
 </item>
 """ % (self.counter, self.counter))
-
         self.writefile("""<?xml version="1.0"?>
 <rss version="2.0">
    <channel>
@@ -254,7 +304,7 @@ class OldItemExpireTest(FeedTestCase):
 
     def checkGuids(self, *ids):
         actual = set()
-        for i in self.items:
+        for i in Item.make_view():
             actual.add(i.getRSSID())
         correct = set(['guid-%d' % i for i in ids])
         self.assertEquals(actual, correct)
@@ -264,17 +314,18 @@ class OldItemExpireTest(FeedTestCase):
         self.updateFeed(self.feed)
 
     def testSimpleOverflow(self):
-        self.assertEqual(self.items.len(), 2)
+        self.assertEqual(Item.make_view().count(), 2)
         self.parseNewFeed()
-        self.assertEqual(self.items.len(), 4)
+        self.assertEqual(Item.make_view().count(), 4)
         self.parseNewFeed()
-        self.assertEqual(self.items.len(), 4)
+        self.assertEqual(Item.make_view().count(), 4)
         self.checkGuids(3, 4, 5, 6)
 
     def testOverflowWithDownloads(self):
-        self.items[0].downloader = FakeDownloader()
-        self.items[1].downloader = FakeDownloader()
-        self.assertEqual(self.items.len(), 2)
+        items = list(Item.make_view())
+        items[0]._downloader = FakeDownloader()
+        items[1]._downloader = FakeDownloader()
+        self.assertEqual(len(items), 2)
         self.parseNewFeed()
         self.parseNewFeed()
         self.checkGuids(1, 2, 5, 6)
@@ -293,47 +344,150 @@ class OldItemExpireTest(FeedTestCase):
 
     def testOverflowWithMaxOldItems(self):
         config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 1000) # don't bother
-        self.assertEqual(self.items.len(), 2)
+        self.assertEqual(Item.make_view().count(), 2)
         self.parseNewFeed()
-        self.assertEquals(self.items.len(), 4)
+        self.assertEquals(Item.make_view().count(), 4)
         self.parseNewFeed()
         self.feed.setMaxOldItems(4)
-        self.feed.clean_old_items()
+        self.feed.actualFeed.clean_old_items()
         while self.feed.actualFeed.updating:
             self.processThreads()
             self.processIdles()
             sleep(0.1)
-        self.assertEquals(self.items.len(), 6)            
+        self.assertEquals(Item.make_view().count(), 6)            
         self.feed.setMaxOldItems(2)
-        self.feed.clean_old_items()
+        self.feed.actualFeed.clean_old_items()
         while self.feed.actualFeed.updating:
             self.processThreads()
             self.processIdles()
             sleep(0.1)
-        self.assertEquals(self.items.len(), 4)
+        self.assertEquals(Item.make_view().count(), 4)
         self.checkGuids(3, 4, 5, 6)
 
     def testOverflowWithGlobalMaxOldItems(self):
         config.set(prefs.TRUNCATE_CHANNEL_AFTER_X_ITEMS, 1000) # don't bother
-        self.assertEqual(self.items.len(), 2)
+        self.assertEqual(Item.make_view().count(), 2)
         self.parseNewFeed()
-        self.assertEquals(self.items.len(), 4)
+        self.assertEquals(Item.make_view().count(), 4)
         self.parseNewFeed()
         config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 4)
-        self.feed.clean_old_items()
+        self.feed.actualFeed.clean_old_items()
         while self.feed.actualFeed.updating:
             self.processThreads()
             self.processIdles()
             sleep(0.1)
-        self.assertEquals(self.items.len(), 6)
+        self.assertEquals(Item.make_view().count(), 6)
         config.set(prefs.MAX_OLD_ITEMS_DEFAULT, 2)
-        self.feed.clean_old_items()
+        self.feed.actualFeed.clean_old_items()
         while self.feed.actualFeed.updating:
             self.processThreads()
             self.processIdles()
             sleep(0.1)
-        self.assertEquals(self.items.len(), 4)
+        self.assertEquals(Item.make_view().count(), 4)
         self.checkGuids(3, 4, 5, 6)
 
+class FeedParserAttributesTestCase(FeedTestCase):
+    """Test that we save/restore attributes from feedparser correctly.
+
+    We don't store feedparser dicts in the database.  This test case is
+    checking if the values we to the database are the same as the original
+    ones from feedparser.
+    """
+
+    def setUp(self):
+        FeedTestCase.setUp(self)
+        self.tempdb = os.path.join(gettempdir(), 'democracy-temp-db')
+        if os.path.exists(self.tempdb):
+            os.remove(self.tempdb)
+        self.reload_database(self.tempdb)
+        self.write_feed()
+        self.parsed_feed = feedparser.parse(self.filename)
+        self.makeFeed()
+        self.save_then_restore_db()
+
+    def tearDown(self):
+        self.runPendingIdles()
+        os.remove(self.tempdb)
+        FeedTestCase.tearDown(self)
+
+    def save_then_restore_db(self):
+        self.reload_database(self.tempdb)
+        self.feed = Feed.make_view().get_singleton()
+        self.item = Item.make_view().get_singleton()
+
+    def write_feed(self):
+        self.writefile("""<?xml version="1.0"?>
+<rss version="2.0">
+    <channel>
+        <title>Downhill Battle Pics</title>
+        <link>http://downhillbattle.org/</link>
+        <description>Downhill Battle is a non-profit organization working to support participatory culture and build a fairer music industry.</description>
+        <pubDate>Wed, 16 Mar 2005 12:03:42 EST</pubDate>
+
+        <item>
+            <title>Bumper Sticker</title>
+            <link>http://downhillbattle.org/item</link>
+            <comments>http://downhillbattle.org/item/comments</comments>
+            <creativeCommons:license>http://www.creativecommons.org/licenses/by-nd/1.0</creativeCommons:license>
+            <guid>guid-1234</guid>
+            <enclosure url="http://downhillbattle.org/key/gallery/movie.mpg"
+                length="1234"
+                type="video/mpeg"
+                />
+            <description>I'm a musician and I support filesharing.</description>
+            <pubDate>Fri, 18 Mar 2005 12:03:42 EST</pubDate>
+            <media:thumbnail url="%(thumburl)s" />
+            <dtv:paymentlink url="http://www.example.com/payment.html" />
+        </item>
+
+    </channel>
+</rss>
+""" % {'thumburl': resources.url("testdata/democracy-now-unicode-bug.xml")})
+
+
+    def test_attributes(self):
+        entry = self.parsed_feed.entries[0]
+        self.assertEquals(self.item.getRSSID(), entry.id)
+        self.assertEquals(self.item.get_thumbnail_url(), entry.thumbnail['url'])
+        self.assertEquals(self.item.get_title(), entry.title)
+        self.assertEquals(self.item.get_raw_description(), entry.description)
+        self.assertEquals(self.item.get_link(), entry.link)
+        self.assertEquals(self.item.get_payment_link(), entry.payment_url)
+        self.assertEquals(self.item.get_license(), entry.license)
+        self.assertEquals(self.item.get_comments_link(), entry.comments)
+
+        enclosure = entry.enclosures[0]
+        self.assertEquals(self.item.get_url(), enclosure.url)
+        self.assertEquals(self.item.get_size(), int(enclosure.length))
+        self.assertEquals(self.item.get_format(), '.mpeg')
+
+    def test_remove_rssid(self):
+        self.item.removeRSSID()
+        self.save_then_restore_db()
+        self.assertEquals(self.item.getRSSID(), None)
+
+    def test_change_title(self):
+        entry = self.parsed_feed.entries[0]
+        self.item.set_title(u"new title")
+        self.save_then_restore_db()
+        self.assertEquals(self.item.get_title(), "new title")
+        self.assert_(not self.item.has_original_title())
+
+        self.item.revert_title()
+        self.save_then_restore_db()
+        self.assertEquals(self.item.get_title(), entry.title)
+        self.assert_(self.item.has_original_title())
+
+    def test_feedparser_output(self):
+        # test a couple entries from the feedparser_output attribute
+        feedparser_output = self.item.feedparser_output
+        self.assertEquals(feedparser_output['title'], 'Bumper Sticker')
+        self.assertEquals(feedparser_output['id'], 'guid-1234')
+        self.assertEquals(feedparser_output['summary'],
+            "I'm a musician and I support filesharing.")
+        self.assertEquals(feedparser_output['enclosures'],
+                [{'href': u'http://downhillbattle.org/key/gallery/movie.mpg',
+                    'type': u'video/mpeg', 'filesize': u'1234'}])
+
 if __name__ == "__main__":
     unittest.main()
