@@ -1,18 +1,20 @@
+import os
 import unittest
+import tempfile
 import threading
 
 from miro import database
 from miro import eventloop
 from miro import app
 from miro import downloader
-from miro import views
-from miro import indexes
 from miro import util
 from miro import databaseupgrade
+from miro import searchengines
 from miro import signals
 from miro import storedatabase
 from miro import subscription
 from time import sleep
+from miro import models
 
 util.setup_logging()
 
@@ -70,25 +72,60 @@ class DummyController:
 
 class MiroTestCase(unittest.TestCase):
     def setUp(self):
-        app.db = database.defaultDatabase
+        models.initialize()
+        app.in_unit_tests = True
         database.set_thread(threading.currentThread())
-        views.initialize()
+        database.ViewTracker.reset_trackers()
+        app.db = None
+        self.reload_database()
+        searchengines._engines = [ searchengines.SearchEngineInfo(u"all",
+           u"Search All", u"", -1) ]
         # reset the event loop
         util.chatter = False
         self.sawError = False
         self.errorSignalOkay = False
         signals.system.connect('error', self.handle_error)
         app.controller = DummyController()
+        self.temp_files = []
 
     def tearDown(self):
         signals.system.disconnect_all()
         util.chatter = True
-
         # Remove any leftover database
-        database.resetDefaultDatabase()
-
+        app.db.close()
+        app.db = None
+        database.ViewTracker.reset_trackers()
         # Remove anything that may have been accidentally queued up
         eventloop._eventLoop = eventloop.EventLoop()
+        for filename in self.temp_files:
+            try:
+                os.remove(filename)
+            except OSError:
+                pass
+
+    def make_temp_path(self):
+        [handle, filename] = tempfile.mkstemp(".xml")
+        self.temp_files.append(filename)
+        return filename
+
+    def reload_database(self, path=':memory:', schema_version=None,
+            object_schemas=None, upgrade=True):
+        if app.db:
+            try:
+                app.db.close()
+            except:
+                pass
+        app.db = storedatabase.LiveStorage(path,
+                schema_version=schema_version, object_schemas=object_schemas)
+        if upgrade:
+            app.db.upgrade_database()
+            database.update_last_id()
+
+    def reload_object(self, obj):
+        # force an object to be reloaded from the databas.
+        del app.db._object_map[obj.id]
+        app.db._ids_loaded.remove(obj.id)
+        return obj.__class__.get_by_id(obj.id)
 
     def handle_error(self, obj, report):
         if self.errorSignalOkay:
@@ -167,9 +204,9 @@ class EventLoopTest(MiroTestCase):
 class DownloaderTestCase(EventLoopTest):
     def setUp(self):
         EventLoopTest.setUp(self)
-        downloader.startupDownloader()
+        downloader.startup_downloader()
 
     def tearDown(self):
-        downloader.shutdownDownloader(eventloop.quit)
+        downloader.shutdown_downloader(eventloop.quit)
         self.runEventLoop()
         EventLoopTest.tearDown(self)
