@@ -3,9 +3,10 @@ from miro import prefs
 
 from miro.feed import Feed
 from miro.guide import ChannelGuide
-from miro.item import Item, get_entry_for_url
+from miro.item import Item
 from miro.playlist import SavedPlaylist
 from miro.folder import PlaylistFolder, ChannelFolder
+from miro.singleclick import _build_entry
 from miro.tabs import TabOrder
 from miro import messages
 from miro import messagehandler
@@ -59,6 +60,7 @@ class TestFrontendMessageHandler(object):
 class TrackerTest(EventLoopTest):
     def setUp(self):
         EventLoopTest.setUp(self)
+        Feed(u'dtv:search')
         self.test_handler = TestFrontendMessageHandler()
         messages.FrontendMessage.install_handler(self.test_handler)
         self.backend_message_handler = messagehandler.BackendMessageHandler(None)
@@ -127,8 +129,8 @@ class GuideTrackTest(TrackerTest):
     def checkInfo(self, guideInfo, guide):
         self.assertEquals(guideInfo.name, guide.get_title())
         self.assertEquals(guideInfo.id, guide.id)
-        self.assertEquals(guideInfo.url, guide.getURL())
-        self.assertEquals(guideInfo.default, guide.getDefault())
+        self.assertEquals(guideInfo.url, guide.get_url())
+        self.assertEquals(guideInfo.default, guide.get_default())
 
     def testInitialList(self):
         self.assertEquals(len(self.test_handler.messages), 1)
@@ -154,7 +156,7 @@ class GuideTrackTest(TrackerTest):
         self.checkChangedMessage(1, removed=[self.guide1])
 
     def testChange(self):
-        self.guide1.setTitle(u"Booya")
+        self.guide1.set_title(u"Booya")
         self.runUrgentCalls()
         self.checkMessageCount(2)
         self.checkChangedMessage(1, changed=[self.guide1])
@@ -163,7 +165,7 @@ class GuideTrackTest(TrackerTest):
         self.checkMessageCount(1)
         messages.StopTrackingGuides().send_to_backend()
         self.runUrgentCalls()
-        self.guide.setTitle(u"Booya")
+        self.guide.set_title(u"Booya")
         g = ChannelGuide(u'http://example.com/3')
         self.guide1.remove()
         self.checkMessageCount(1)
@@ -173,7 +175,7 @@ class PlaylistTrackTest(TrackerTest):
         TrackerTest.setUp(self)
         self.playlist1 = SavedPlaylist(u'Playlist 1')
         self.playlist2 = SavedPlaylist(u'Playlist 2')
-        self.folder = PlaylistFolder('Playlist Folder')
+        self.folder = PlaylistFolder(u'Playlist Folder')
 #        self.folder.handleDNDAppend( set([self.playlist2.id]))
         self.runUrgentCalls()
         messages.TrackPlaylists().send_to_backend()
@@ -207,11 +209,11 @@ class PlaylistTrackTest(TrackerTest):
         self.checkChangedMessage(1, added=[p])
 
     def testAddedOrder(self):
-        p1 = SavedPlaylist('Playlist 2')
-        p2 = SavedPlaylist('Playlist 3')
-        p3 = SavedPlaylist('Playlist 4')
-        p4 = SavedPlaylist('Playlist 5')
-        p5 = SavedPlaylist('Playlist 6')
+        p1 = SavedPlaylist(u'Playlist 2')
+        p2 = SavedPlaylist(u'Playlist 3')
+        p3 = SavedPlaylist(u'Playlist 4')
+        p4 = SavedPlaylist(u'Playlist 5')
+        p5 = SavedPlaylist(u'Playlist 6')
         self.runUrgentCalls()
         # We want the PlaylistAdded messages to come in the same order the
         # feeds were added.
@@ -225,7 +227,7 @@ class PlaylistTrackTest(TrackerTest):
         self.checkChangedMessage(1, removed=[self.playlist2])
 
     def testChange(self):
-        self.playlist1.setTitle(u"Booya")
+        self.playlist1.set_title(u"Booya")
         self.runUrgentCalls()
         self.checkMessageCount(2)
         self.checkChangedMessage(1, changed=[self.playlist1])
@@ -234,7 +236,7 @@ class PlaylistTrackTest(TrackerTest):
         p1 = SavedPlaylist(u'Playlist')
         p1.remove()
         p2 = SavedPlaylist(u'Playlist 2')
-        p2.setTitle(u'New Title')
+        p2.set_title(u'New Title')
         self.runUrgentCalls()
         # We don't need to see that p1 was added because it got removed
         # immediately after.  We don't need to see that p2 was changed because
@@ -246,7 +248,7 @@ class PlaylistTrackTest(TrackerTest):
         self.checkMessageCount(1)
         messages.StopTrackingPlaylists().send_to_backend()
         self.runUrgentCalls()
-        self.playlist1.setTitle(u"Booya")
+        self.playlist1.set_title(u"Booya")
         f = Feed(u'http://example.com/3')
         self.playlist2.remove()
         self.checkMessageCount(1)
@@ -256,7 +258,7 @@ class FeedTrackTest(TrackerTest):
         TrackerTest.setUp(self)
         self.feed1 = Feed(u'http://example.com/')
         self.feed2 = Feed(u'http://example.com/2')
-        self.feed_folder = ChannelFolder('test channel folder')
+        self.feed_folder = ChannelFolder(u'test channel folder')
         m = messages.TabsReordered()
         m.toplevels['feed'] = [messages.ChannelInfo(self.feed1)]
         m.toplevels['audio-feed'] = [messages.ChannelInfo(self.feed_folder)]
@@ -269,8 +271,8 @@ class FeedTrackTest(TrackerTest):
     def checkInfo(self, channelInfo, feed):
         self.assertEquals(channelInfo.name, feed.get_title())
         self.assertEquals(channelInfo.id, feed.id)
-        self.assertEquals(channelInfo.unwatched, feed.numUnwatched())
-        self.assertEquals(channelInfo.available, feed.numAvailable())
+        self.assertEquals(channelInfo.unwatched, feed.num_unwatched())
+        self.assertEquals(channelInfo.available, feed.num_available())
         self.assertEquals(channelInfo.is_folder,
                 isinstance(feed, ChannelFolder))
 
@@ -316,7 +318,7 @@ class FeedTrackTest(TrackerTest):
         self.checkChangedMessage(2, removed=[self.feed2])
 
     def testChange(self):
-        self.feed1.setTitle(u"Booya")
+        self.feed1.set_title(u"Booya")
         self.runUrgentCalls()
         self.checkMessageCount(3)
         self.checkChangedMessage(2, changed=[self.feed1])
@@ -325,7 +327,7 @@ class FeedTrackTest(TrackerTest):
         f1 = Feed(u'http://example.com/3')
         f1.remove()
         f2 = Feed(u'http://example.com/4')
-        f2.setTitle(u'New Title')
+        f2.set_title(u'New Title')
         self.runUrgentCalls()
         # We don't need to see that f1 was added because it got removed
         # immediately after.  We don't need to see that f2 was changed because
@@ -337,7 +339,7 @@ class FeedTrackTest(TrackerTest):
         self.checkMessageCount(2)
         messages.StopTrackingChannels().send_to_backend()
         self.runUrgentCalls()
-        self.feed1.setTitle(u"Booya")
+        self.feed1.set_title(u"Booya")
         f = Feed(u'http://example.com/3')
         self.feed2.remove()
         self.checkMessageCount(2)
@@ -369,14 +371,14 @@ class FeedItemTrackTest(TrackerTest):
         self.runUrgentCalls()
 
     def make_item(self, url):
-        self.items.append(Item(entry=get_entry_for_url(url),
+        self.items.append(Item(entry=_build_entry(url, 'video/x-unknown'),
             feed_id=self.feed.id))
 
     def checkDownloadInfo(self, info, item):
         downloader = item.downloader
         self.assertEquals(info.current_size, downloader.get_current_size())
         self.assertEquals(info.rate, downloader.getRate())
-        self.assertEquals(info.state, downlader.get_state())
+        self.assertEquals(info.state, downloader.get_state())
 
     def checkInfo(self, itemInfo, item):
         self.assertEquals(itemInfo.name, item.get_title())
@@ -385,8 +387,8 @@ class FeedItemTrackTest(TrackerTest):
         self.assertEquals(itemInfo.size, item.get_size())
         self.assertEquals(itemInfo.permalink, item.get_link())
         self.assertEquals(itemInfo.id, item.id)
-        self.assertEquals(itemInfo.expiration_date, item.getExpirationTime())
-        self.assertEquals(itemInfo.thumbnail, item.getThumbnail())
+        self.assertEquals(itemInfo.expiration_date, item.get_expiration_time())
+        self.assertEquals(itemInfo.thumbnail, item.get_thumbnail())
         if item.downloader:
             self.checkDownloadInfo(itemInfo.download_info)
         else:
@@ -408,8 +410,8 @@ class FeedItemTrackTest(TrackerTest):
         self.checkInfoList(message.items, self.items)
 
     def testUpdate(self):
-        self.items[0].entry.title = u'new name'
-        self.items[0].signalChange()
+        self.items[0].entry_title = u'new name'
+        self.items[0].signal_change()
         self.runUrgentCalls()
         self.assertEquals(len(self.test_handler.messages), 2)
         self.checkChangedMessage(1, changed=[self.items[0]])
@@ -431,8 +433,8 @@ class FeedItemTrackTest(TrackerTest):
     def testStop(self):
         messages.StopTrackingItems('feed', self.feed.id).send_to_backend()
         self.runUrgentCalls()
-        self.items[0].entry.title = u'new name'
-        self.items[0].signalChange()
+        self.items[0].entry_title = u'new name'
+        self.items[0].signal_change()
         self.items[1].remove()
         self.make_item(u'http://example.com/4')
         self.runUrgentCalls()
@@ -451,15 +453,16 @@ class PlaylistItemTrackTest(TrackerTest):
         self.runUrgentCalls()
 
     def make_item(self, url):
-        item = Item(entry=get_entry_for_url(url), feed_id=self.feed.id)
+        item = Item(entry=_build_entry(url, 'video/x-unknown'),
+                    feed_id=self.feed.id)
         self.items.append(item)
-        self.playlist.addItem(item)
+        self.playlist.add_item(item)
 
     def checkDownloadInfo(self, info, item):
         downloader = item.downloader
         self.assertEquals(info.current_size, downloader.get_current_size())
         self.assertEquals(info.rate, downloader.getRate())
-        self.assertEquals(info.state, downlader.get_state())
+        self.assertEquals(info.state, downloader.get_state())
 
     def checkInfo(self, itemInfo, item):
         self.assertEquals(itemInfo.name, item.get_title())
@@ -468,8 +471,8 @@ class PlaylistItemTrackTest(TrackerTest):
         self.assertEquals(itemInfo.size, item.get_size())
         self.assertEquals(itemInfo.permalink, item.get_link())
         self.assertEquals(itemInfo.id, item.id)
-        self.assertEquals(itemInfo.expiration_date, item.getExpirationTime())
-        self.assertEquals(itemInfo.thumbnail, item.getThumbnail())
+        self.assertEquals(itemInfo.expiration_date, item.get_expiration_time())
+        self.assertEquals(itemInfo.thumbnail, item.get_thumbnail())
         if item.downloader:
             self.checkDownloadInfo(itemInfo.download_info)
         else:
@@ -491,8 +494,8 @@ class PlaylistItemTrackTest(TrackerTest):
         self.checkInfoList(message.items, self.items)
 
     def testUpdate(self):
-        self.items[0].entry.title = u'new name'
-        self.items[0].signalChange()
+        self.items[0].entry_title = u'new name'
+        self.items[0].signal_change()
         self.runUrgentCalls()
         self.assertEquals(len(self.test_handler.messages), 2)
         self.checkChangedMessage(1, changed=[self.items[0]])
@@ -514,8 +517,8 @@ class PlaylistItemTrackTest(TrackerTest):
     def testStop(self):
         messages.StopTrackingItems('playlist', self.playlist.id).send_to_backend()
         self.runUrgentCalls()
-        self.items[0].entry.title = u'new name'
-        self.items[0].signalChange()
+        self.items[0].entry_title = u'new name'
+        self.items[0].signal_change()
         self.items[1].remove()
         self.make_item(u'http://example.com/4')
         self.runUrgentCalls()
