@@ -44,14 +44,14 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
             for value in parsed:
                 self.isProperFeedParserDict(value, name)
         elif isinstance(parsed, types.StringType):
-            self.assert_(name in ["base","type","encoding","version","href","rel"])
+            self.assert_(name in ["base", "type", "encoding", "version", "href", "rel"])
         elif isinstance(parsed, time.struct_time):
             self.assert_(name in ["updated_parsed"])
         elif isinstance(parsed, types.IntType):
-            self.assert_(name in ["bozo"])
+            self.assert_(name in ["updated_parsed", "bozo"])
         else:
-            self.assert_((isinstance(parsed,types.UnicodeType) or
-                          isinstance(parsed,types.NoneType)))
+            self.assert_((isinstance(parsed, types.UnicodeType) or
+                          isinstance(parsed, types.NoneType)))
 
     # Returns true iff value is a python unicode string containing
     # only ascii characters
@@ -69,7 +69,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         myFeed = feed.Feed(u"file://"+self.filename)
         self.forceFeedParserCallback(myFeed)
 
-        self.isProperFeedParserDict(myFeed.parsed)
+        self.isProperFeedParserDict(myFeed.actualFeed.parsed)
 
         # We need to explicitly check that the type is unicode because
         # Python automatically converts bytes strings to unicode strings
@@ -81,9 +81,7 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         self.assertEqual(type(myFeed.get_description()), types.UnicodeType)
         self.assertEqual(u"<span>Chinese Numbers \u25cb\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d</span>",myFeed.get_description())
 
-        items = database.defaultDatabase.filter(lambda x:x.__class__ == item.Item)
-        self.assertEqual(items.len(),1)
-        i = items[0]
+        i = item.Item.make_view().get_singleton()
         self.assertEqual(type(i.get_title()), types.UnicodeType)
         self.assertEqual(u"\u25cb\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d",i.get_title())
 
@@ -92,7 +90,8 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
 
     def forceFeedParserCallback(self, myFeed):
         # a hack to get the feed to update without eventloop
-        myFeed.feedparser_callback(feedparser.parse(myFeed.initialHTML))
+        feedimpl = myFeed.actualFeed
+        feedimpl.feedparser_callback(feedparser.parse(feedimpl.initialHTML))
 
     # This is a latin1 feed that claims to be UTF-8
     def testInvalidLatin1Feed(self):
@@ -108,8 +107,8 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         myFeed.update()
         self.processThreads()
         self.processIdles()
-        self.assertEqual(len(myFeed.items), 1)
-        myItem = myFeed.items[0]
+        self.assertEqual(myFeed.items.count(), 1)
+        myItem = list(myFeed.items)[0]
         self.assertEqual(len(myItem.get_title()), 14)
         self.assertEqual(myItem.get_title(), u"H\xe4ppy Birthday")
 
@@ -126,8 +125,8 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         
         self.assertEqual(self.numDialogs, 1)
         myFeed.update()
-        self.assertEqual(len(myFeed.items), 1)
-        myItem = myFeed.items[0]
+        self.assertEqual(myFeed.items.count(), 1)
+        myItem = list(myFeed.items)[0]
         self.assertEqual(len(myItem.get_title()), 14)
         self.assertEqual(myItem.get_title(), u"H\xe4ppy Birthday")
 
@@ -144,8 +143,8 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         
         self.assertEqual(self.numDialogs,1)
         myFeed.update()
-        self.assertEqual(len(myFeed.items),1)
-        myItem = myFeed.items[0]
+        self.assertEqual(myFeed.items.count(),1)
+        myItem = list(myFeed.items)[0]
         self.assertEqual(len(myItem.get_title()),14)
         self.assertEqual(myItem.get_title(), u"H\xe4ppy Birthday")
 
@@ -162,8 +161,8 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         
         self.assertEqual(self.numDialogs,1)
         myFeed.update()
-        self.assertEqual(len(myFeed.items),1)
-        myItem = myFeed.items[0]
+        self.assertEqual(myFeed.items.count(),1)
+        myItem = list(myFeed.items)[0]
         self.assertEqual(len(myItem.get_title()),14)
         self.assertEqual(myItem.get_title(), u"H\xe4ppy Birthday")
 
@@ -175,16 +174,16 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
 
         self.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed(u"file://"+self.filename)
+        myFeed = feed.Feed(u"file://" + self.filename)
         
-        self.assertEqual(self.numDialogs,1)
+        self.assertEqual(self.numDialogs, 1)
         myFeed.update()
         # Either the item isn't added or it's added with an ascii URL
-        if len(myFeed.items) > 0:
-            self.assertEqual(len(myFeed.items),1)
-            myItem = myFeed.items[0]
-            myURL = myItem.getURL()
-            self.assertEqual(str(myURL),myURL)
+        if myFeed.items.count() > 0:
+            self.assertEqual(myFeed.items.count(), 1)
+            myItem = list(myFeed.items)[0]
+            myURL = myItem.get_url()
+            self.assertEqual(str(myURL), myURL)
 
     def testLatin1HTMLLinks(self):
         [handle, self.filename] = mkstemp(".html")
@@ -199,10 +198,10 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         self.assertEqual(self.numDialogs,1)
         myFeed.update()
         # Either the item isn't added or it's added with an ascii URL
-        if len(myFeed.items) > 0:
-            self.assertEqual(len(myFeed.items),1)
-            myItem = myFeed.items[0]
-            myURL = myItem.getURL()
+        if myFeed.items.count() > 0:
+            self.assertEqual(myFeed.items.count(),1)
+            myItem = list(myFeed.items)[0]
+            myURL = myItem.get_url()
             self.assertEqual(str(myURL),myURL)
 
     def testInvalidLatin1HTMLLinks(self):
@@ -213,16 +212,16 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
 
         self.choice = dialogs.BUTTON_YES
 
-        myFeed = feed.Feed(u"file://"+self.filename)
+        myFeed = feed.Feed(u"file://" + self.filename)
         
-        self.assertEqual(self.numDialogs,1)
+        self.assertEqual(self.numDialogs, 1)
         myFeed.update()
         # Either the item isn't added or it's added with an ascii URL
-        if len(myFeed.items) > 0:
-            self.assertEqual(len(myFeed.items),1)
-            myItem = myFeed.items[0]
-            myURL = myItem.getURL()
-            self.assertEqual(str(myURL),myURL)
+        if myFeed.items.count() > 0:
+            self.assertEqual(myFeed.items.count(), 1)
+            myItem = list(myFeed.items)[0]
+            myURL = myItem.get_url()
+            self.assertEqual(str(myURL), myURL)
 
     def testUTF8HTMLThumbs(self):
         [handle, self.filename] = mkstemp(".html")
@@ -237,12 +236,12 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         self.assertEqual(self.numDialogs,1)
         myFeed.update()
 
-        self.assertEqual(len(myFeed.items),1)
-        myItem = myFeed.items[0]
-        myURL = myItem.getURL()
+        self.assertEqual(myFeed.items.count(),1)
+        myItem = list(myFeed.items)[0]
+        myURL = myItem.get_url()
         self.assertEqual(str(myURL),myURL)
 
-        thumb = myItem.getThumbnailURL()
+        thumb = myItem.get_thumbnail_url()
         if thumb is not None:
             self.assertEqual(str(thumb),thumb)
 
@@ -259,12 +258,12 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         self.assertEqual(self.numDialogs,1)
         myFeed.update()
 
-        self.assertEqual(len(myFeed.items),1)
-        myItem = myFeed.items[0]
-        myURL = myItem.getURL()
+        self.assertEqual(myFeed.items.count(),1)
+        myItem = list(myFeed.items)[0]
+        myURL = myItem.get_url()
         self.assertEqual(str(myURL),myURL)
 
-        thumb = myItem.getThumbnailURL()
+        thumb = myItem.get_thumbnail_url()
         if thumb is not None:
             self.assertEqual(str(thumb),thumb)
 
@@ -281,12 +280,12 @@ class UnicodeFeedTestCase(framework.EventLoopTest):
         self.assertEqual(self.numDialogs,1)
         myFeed.update()
 
-        self.assertEqual(len(myFeed.items),1)
-        myItem = myFeed.items[0]
-        myURL = myItem.getURL()
+        self.assertEqual(myFeed.items.count(),1)
+        myItem = list(myFeed.items)[0]
+        myURL = myItem.get_url()
         self.assertEqual(str(myURL),myURL)
 
-        thumb = myItem.getThumbnailURL()
+        thumb = myItem.get_thumbnail_url()
         if thumb is not None:
             self.assertEqual(str(thumb),thumb)
 
