@@ -26,15 +26,16 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
+"""``miro.theme`` -- Holds the ThemeHistory object.
+"""
+
 from miro.gtcache import gettext as _
 import logging
-import platform
 from miro import config
 from miro import prefs
-from miro import app
 import os
 from miro.eventloop import asUrgent
-from miro.database import DDBObject
+from miro.database import DDBObject, ObjectNotFoundError
 from miro import opml
 from miro.plat import resources
 from miro import guide
@@ -44,8 +45,10 @@ from miro import playlist
 from miro import signals
 
 class ThemeHistory(DDBObject):
-    def __init__(self):
-        DDBObject.__init__(self)
+    """DDBObject that keeps track of the themes used in regards
+    to setting up new themes and changing themes.
+    """
+    def setup_new(self):
         self.lastTheme = None
         self.pastThemes = []
         self.theme = config.get(prefs.THEME_NAME)
@@ -65,7 +68,7 @@ class ThemeHistory(DDBObject):
         if self.theme not in self.pastThemes:
             self.pastThemes.append(self.theme)
             self.on_first_run()
-            self.signalChange()
+            self.signal_change()
         if self.lastTheme != self.theme:
             self.lastTheme = self.theme
             self.on_theme_change()
@@ -74,7 +77,7 @@ class ThemeHistory(DDBObject):
     def on_theme_change(self):
         if self.theme is None: # vanilla Miro
             guideURL = config.get(prefs.CHANNEL_GUIDE_URL)
-            if guide.getGuideByURL(guideURL) is None:
+            if guide.get_guide_by_url(guideURL) is None:
                 # This happens when the DB is initialized with a theme that
                 # doesn't have it's own set of default channels; None is
                 # artificially added to the pastThemes lists to prevent the
@@ -84,19 +87,20 @@ class ThemeHistory(DDBObject):
                 guide.ChannelGuide(guideURL,
                                    unicode(config.get(
                             prefs.CHANNEL_GUIDE_ALLOWED_URLS)).split())
-        self.signalChange()
+        self.signal_change()
 
     def on_first_run(self):
         logging.info("Spawning Miro Guide...")
         guideURL = unicode(config.get(prefs.CHANNEL_GUIDE_URL))
-        if guide.getGuideByURL(guideURL) is None:
+        if guide.get_guide_by_url(guideURL) is None:
             guide.ChannelGuide(guideURL,
             unicode(config.get(prefs.CHANNEL_GUIDE_ALLOWED_URLS)).split())
 
-        if self.theme is not None: # we have a theme
+        if self.theme is not None:
+            # we have a theme
             new_guides = unicode(config.get(prefs.ADDITIONAL_CHANNEL_GUIDES)).split()
             for temp_guide in new_guides:
-                if guide.getGuideByURL(temp_guide) is None:
+                if guide.get_guide_by_url(temp_guide) is None:
                     guide.ChannelGuide(temp_guide)
             if ((config.get(prefs.DEFAULT_CHANNELS_FILE) is not None) and
                     (config.get(prefs.THEME_NAME) is not None)):
@@ -117,59 +121,81 @@ class ThemeHistory(DDBObject):
 
                 # This code would install the Miro Guide if it isn't
                 # already installed
-                # if guide.getGuideByURL(prefs.CHANNEL_GUIDE_URL.default) is None:
+                # if guide.get_guide_by_url(prefs.CHANNEL_GUIDE_URL.default) is None:
                 #     guide.ChannelGuide(prefs.CHANNEL_GUIDE_URL.default)
                 self.pastThemes.append(None)
                 self._install_default_feeds()
-        else: # no theme
+        else:
+            # no theme
             self._install_default_feeds()
         signals.system.theme_first_run(self.theme)
 
+    def _add_default(self, default, section):
+        # folder
+        if isinstance(default, tuple) and isinstance(default[1], list):
+            defaultFolder = default
+            try:
+                c_folder = folder.Folder.get_by_title(defaultFolder[0])
+            except ObjectNotFoundError:
+                c_folder = folder.ChannelFolder(defaultFolder[0])
+                c_folder.section = section
+                c_folder.signal_change()
+            for url, autodownload in defaultFolder[1]:
+                logging.info("adding feed %s in section %s" % (url, section))
+                try:
+                    d_feed = feed.Feed.get_by_url(default[0])
+                except ObjectNotFoundError:
+                    d_feed = feed.Feed(url, initiallyAutoDownloadable=autodownload)
+                    d_feed.set_folder(c_folder)
+                    d_feed.section = section
+                    d_feed.signal_change()
+        # feed
+        else:
+            try:
+                d_feed = feed.Feed.get_by_url(default[0])
+            except ObjectNotFoundError:
+                logging.info("adding feed %s in section %s" % (default, section))
+                d_feed = feed.Feed(default[0], initiallyAutoDownloadable=default[1])
+                d_feed.section = section
+                d_feed.signal_change()
+
     @asUrgent
     def _install_default_feeds(self):
-        initialFeeds = resources.path("initial-feeds.democracy")
-        if os.path.exists(initialFeeds):
-            type, urls = subscription.parse_file(initialFeeds)
-            if urls is not None and type == 'rss':
-                for url in urls:
-                    feed.Feed(url, initiallyAutoDownloadable=False)
-            dialog = dialogs.MessageBoxDialog(_("Custom Channels"),
-                    text = _(
-                        "You are running a version of %(appname)s with a "
-                        "custom set of channels.",
-                        {"appname": config.get(prefs.LONG_APP_NAME)}
-                    ))
-            dialog.run()
-            app.controller.initial_feeds = True
-        else:
-            logging.info("Adding default feeds")
-
-            defaultFeedURLs = []
-
-            defaultFeedURLs.extend([
-                (u'http://feeds.miroguide.com/miroguide/new', False),
-                (u'http://feeds.miroguide.com/miroguide/featured', False),
-                (u'http://feeds.feedburner.com/earth-touch_podcast_720p', False),
-                (u'http://www.linktv.org/rss/hq/globalpulse.xml', False),
-
+        logging.info("Adding default feeds")
+        default_video_feeds = []
+        default_video_feeds.extend([
+            (u'http://feeds.feedburner.com/earth-touch_podcast_720p', False),
+            (u'http://www.linktv.org/rss/hq/globalpulse.xml', False),
             ])
 
-            for default in defaultFeedURLs:
-                # folder
-                if isinstance(default, tuple) and isinstance(default[1], list):
-                    defaultFolder = default
-                    c_folder = folder.ChannelFolder(defaultFolder[0])
-                    for url, autodownload in defaultFolder[1]:
-                        d_feed = feed.Feed(url, initiallyAutoDownloadable=autodownload)
-                        d_feed.setFolder(c_folder)
+        default_audio_feeds = []
+        default_audio_feeds.extend([
+            (u'http://feeds.thisamericanlife.org/talpodcast', False)
+            ])
 
-                # feed
-                else:
-                    d_feed = feed.Feed(default[0], initiallyAutoDownloadable=default[1])
+        for default in default_video_feeds:
+            self._add_default(default, u"video")
 
-            # create example playlist
-            playlist.SavedPlaylist(_(u"Example Playlist"))
+        for default in default_audio_feeds:
+            self._add_default(default, u"audio")
 
-            # create default site
-            cg = guide.ChannelGuide(u"http://beta.legaltorrents.com/")
-            cg.setTitle(u"LegalTorrents")
+        # create example playlist
+        default_playlists = [
+            u"Example Playlist"
+            ]
+        for default in default_playlists:
+            try:
+                pl = playlist.SavedPlaylist.get_by_title(default)
+            except ObjectNotFoundError:
+                playlist.SavedPlaylist(_(u"Example Playlist"))
+
+        default_guides = [
+            (u"http://beta.legaltorrents.com/", u"LegalTorrents"),
+            (u"https://www.miroguide.com/audio/", u"Miro Audio Guide")
+            ]
+        for default in default_guides:
+            try:
+                cg = guide.ChannelGuide.get_by_url(default[0])
+            except ObjectNotFoundError:
+                cg = guide.ChannelGuide(default[0])
+                cg.set_title(default[1])
