@@ -26,7 +26,7 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-"""util.py -- Utility functions.
+"""``miro.util`` -- Utility functions.
 
 This module contains self-contained utility functions.  It shouldn't import
 any other Miro modules.
@@ -35,7 +35,10 @@ any other Miro modules.
 import os
 import random
 import re
-import sha
+try:
+    from hashlib import sha1 as sha
+except ImportError:
+    from sha import sha
 import string
 import sys
 import urllib
@@ -130,40 +133,35 @@ def write_simple_config_file(path, data):
     
     f.close()
 
-def query_revision(fn):
-    """Called at build-time to ask Subversion for the revision number of
-    this checkout.  Going to fail without Cygwin. Yeah, oh well.  Pass the
-    file or directory you want to use as a reference point.
+def query_revision():
+    """Called at build-time to ask git for the revision of this
+    checkout.
 
     Returns the (url, revision) on success and None on failure.
     """
+    url = "unknown"
+    revision = "unknown"
     try:
-        # first try straight-up svn
-        p = subprocess.Popen(["svn", "info", fn], stdout=subprocess.PIPE)
+        p = subprocess.Popen(["git", "config", "--list"], stdout=subprocess.PIPE)
+        info = p.stdout.read().splitlines()
+        p.stdout.close()
+        origline = "remote.origin.url"
+        info = [m for m in info if m.startswith(origline)]
+        if info:
+            url = info[0][len(origline)+1:].strip()
+
+        p = subprocess.Popen(["git", "rev-parse", "HEAD"], stdout=subprocess.PIPE)
         info = p.stdout.read()
         p.stdout.close()
-        url_match = re.search("URL: (.*)", info)
-        # FIXME - this doesn't work on non English systems because the word
-        # we're looking for will be in another language!
-        revision_match = re.search("Revision: (.*)", info)
-
-        # if that doesn't work, try git over svn.
-        if not url_match:
-            p = subprocess.Popen(["git", "svn", "info"], stdout=subprocess.PIPE)
-            info = p.stdout.read()
-            p.stdout.close()
-            url_match = re.search("URL: (.*)", info)
-            revision_match = re.search("Revision: (.*)", info)
-
-        url = url_match.group(1).strip()
-        revision = revision_match.group(1).strip()
+        revision = info[0:8]
         return (url, revision)
     except (SystemExit, KeyboardInterrupt):
         raise
     except Exception, e:
         print "Exception thrown when querying revision: %s" % e
+    return None
 
-class AutoflushingStream:
+class AutoFlushingStream:
     """Converts a stream to an auto-flushing one.  It behaves in exactly the
     same way, except all write() calls are automatically followed by a
     flush().
@@ -241,7 +239,7 @@ def get_torrent_info_hash(path):
     try:
         data = f.read()
         metainfo = lt.bdecode(data)
-        infohash = sha.sha(lt.bencode(metainfo['info'])).digest()
+        infohash = sha(lt.bencode(metainfo['info'])).digest()
         return infohash
     finally:
         f.close()
@@ -332,11 +330,44 @@ def print_mem_usage(message):
 #    print message
 #    os.system("ps huwwwp %d" % (os.getpid(),))
 
+def db_mem_usage_test():
+    from miro import models
+    from miro.database import DDBObject
+    last_usage = get_mem_usage()
+    logging.info("baseline memory usage: %s", last_usage)
+    for name in dir(models):
+        ddb_object_class = getattr(models, name)
+        try:
+            if not issubclass(ddb_object_class, DDBObject):
+                continue
+        except TypeError:
+            continue
+        if name == 'FileItem':
+            # Item and FileItem share a db table, so we only need to load one
+            continue
+
+        # make sure each object is loaded in memory and count the total
+        count = len(list(ddb_object_class.make_view()))
+        current_usage = get_mem_usage()
+        class_usage = current_usage-last_usage
+        if count == 0:
+            count = 1 # prevent zero division errors
+        logging.info("memory usage for %s: %s (%d bytes per object)",
+                ddb_object_class.__name__, class_usage,
+                class_usage * 1024 / count)
+        last_usage = current_usage
+    logging.info("total memory usage: %s", last_usage)
+    logging.info("feed count: %s", models.Feed.make_view().count())
+    logging.info("item count: %s", models.Item.make_view().count())
+
+def get_mem_usage():
+    return int(call_command('ps', '-o', 'rss', 'hp', str(os.getpid())))
+
 class TooManySingletonsError(Exception):
     pass
 
 def getSingletonDDBObject(view):
-    view.confirmDBThread()
+    view.confirm_db_thread()
     viewLength = view.len()
     if viewLength == 1:
         view.resetCursor()
@@ -354,6 +385,7 @@ class ThreadSafeCounter:
         self.lock = threading.Lock()
 
     def inc(self):
+        """Increments the value by 1."""
         self.lock.acquire()
         try:
             self.value += 1
@@ -361,6 +393,7 @@ class ThreadSafeCounter:
             self.lock.release()
 
     def dec(self):
+        """Decrements the value by 1."""
         self.lock.acquire()
         try:
             self.value -= 1
@@ -368,6 +401,7 @@ class ThreadSafeCounter:
             self.lock.release()
 
     def getvalue(self):
+        """Returns the current value."""
         self.lock.acquire()
         try:
             return self.value
@@ -375,6 +409,8 @@ class ThreadSafeCounter:
             self.lock.release()
 
 def setup_logging():
+    """Adds TIMING and JSALERT logging levels.
+    """
     logging.addLevelName(25, "TIMING")
     logging.timing = lambda msg, *args, **kargs: logging.log(25, msg, *args, **kargs)
     logging.addLevelName(26, "JSALERT")
@@ -475,13 +511,16 @@ def stringify(u, handleerror="xmlcharrefreplace"):
     This is required for some logging especially where the things being
     logged are filenames which can be Unicode in the Windows platform.
 
-    Note that this is not the inverse of unicodify.
+    You can pass in a handleerror argument which defaults to
+    ``"xmlcharrefreplace"``.  This will increase the string size as it
+    converts unicode characters that don't have ascii equivalents into
+    escape sequences.  If you don't want to increase the string length, use
+    ``"replace"`` which will use ? for unicode characters that don't have
+    ascii equivalents.
 
-    You can pass in a handleerror argument which defaults to "xmlcharrefreplace".
-    This will increase the string size as it converts unicode characters that
-    don't have ascii equivalents into escape sequences.  If you don't want to
-    increase the string length, use "replace" which will use ? for unicode
-    characters that don't have ascii equivalents.
+    .. note::
+
+       This is not the inverse of unicodify!
     """
     if isinstance(u, unicode):
         return u.encode("ascii", handleerror)
@@ -686,8 +725,11 @@ def toUni(orig, encoding=None):
 import sgmllib
 
 class HTMLStripper(sgmllib.SGMLParser):
-    """
-    Strips html from text while maintaining links and newline-like HTML bits.
+    """Strips html from text while maintaining links and newline-like HTML
+    bits.
+
+    This class resets itself after every ``strip`` call, so you can re-use
+    the class if you want.  However, this class is not threadsafe.
     """
     def __init__(self):
         sgmllib.SGMLParser.__init__(self)
@@ -706,14 +748,19 @@ class HTMLStripper(sgmllib.SGMLParser):
         self.__unaryre = re.compile("\\<[ ]*([A-Za-z]+)[ ]*[/]?\\>", re.M)
 
     def strip(self, s):
+        """Takes a string ``s`` and returns the stripped version.
+        """
         if "<" not in s:
             return (s.strip(), [])
 
         s = s.replace("\r\n", "\n")
         s = self.__unaryre.sub("<\\1 />", s)
 
-        self.feed(s)
-        self.close()
+        try:
+            self.feed(s)
+            self.close()
+        except:
+            pass
 
         try:
             self.__flush()
@@ -752,7 +799,11 @@ class HTMLStripper(sgmllib.SGMLParser):
         self.__add(data)
 
     def handle_charref(self, ref):
-        self.__add(unichr(int(ref)))
+        if ref.startswith('x'):
+            charnum = int(ref[1:], 16)
+        else:
+            charnum = int(ref)
+        self.__add(unichr(charnum))
 
     def start_p(self, attributes):
         self.__add("\n")
@@ -787,7 +838,16 @@ class Matrix(object):
     """2 Dimensional matrix.
     
     Matrix objects are accessed like a list, except tuples are used as
-    indices, for example matrix[3,4] = foo
+    indices, for example:
+
+    >>> m = Matrix(5, 5)
+    >>> m[3, 4] = "foo"
+    >>> m
+    None, None, None, None, None
+    None, None, None, None, None
+    None, None, None, None, None
+    None, None, None, None, None
+    None, None, None, 'foo', None
     """
 
     def __init__(self, columns, rows, initial_value=None):
@@ -840,3 +900,15 @@ def entity_replace(text):
     for src, dest in replacements:
         text = text.replace(src, dest)
     return text
+
+_lower_translate = string.maketrans(string.ascii_uppercase,
+        string.ascii_lowercase)
+
+def ascii_lower(str):
+    """Converts a string to lower case, using a simple translations of ASCII
+    characters.
+
+    This method is not locale-dependant, which is useful in some cases.
+    Normally str.lower() should be used though.
+    """
+    return str.translate(_lower_translate)
